progress=0
verbose=0

Usage()
{
  print "Usage: ParseRODM [-b<batchid>] [-o<output-base>] [-p] [-v] [<input-file>...]"
  print "  -d    Output diagnostic messages"
  print "  -l    Perform lexical analysis only. This is a diagnostic option."
  print "  -o    Specifies the base name of the output files generated from"
  print "        the input. The files produced are"
  print "          <output-base>Object.dat"
  print "          <output-base>Links.dat"
  print "          <output-base>ObjValScalar.dat"
  print "          <output-base>ObjValIndexList.dat"
  print "          <output-base>ObjValSelfDefining.dat"
  print "  -p    Display progress messages while processing."
  print "  -v    Display verbose status messages while processing."
}

while getopts b:o:pv c; do
  case $c in
  b)  batchid=$OPTARG;;
  o)  output_root=$(echo "$OPTARG" | sed 's?[\\]?/?g');;
  p)  progress=1;;
  v)  verbose=1;;
  *)  Usage; exit;;
  esac
done

#if [ -z "$batchid" ]; then
#  Usage>&2
#  exit 1
#fi

for f in Object ObjValScalar Link; do
  rm -f "$output_root$f.dat"
done

shift $OPTIND-1


awk -v batchid=$batchid -v progress=$progress -v output_root="$output_root" -v verbose=$verbose 'BEGIN {
  object_file=sprintf("%sObject.dat", output_root);
  objvalscalar_file=sprintf("%sObjValScalar.dat", output_root);
  link_file=sprintf("%sLink.dat", output_root);
  monitored_field["DisplayResourceName"]=1;
  monitored_field["states"]=1;
  monitored_field["1.2.124.360501.1.215"]=1; # circuitID
  monitored_field["1.3.18.0.0.2032"]=1; # snaNodeName
  monitored_field["1.3.18.0.0.2044"]=1; # transmissionGroupId
  monitored_field["1.3.18.0.0.2053"]=1; # virtualRoutingNodeName
  monitored_field["1.3.18.0.0.2082"]=1; # tgCircuitId
  monitored_field["1.3.18.0.0.2133"]=1; # linkName
  monitored_field["1.3.18.0.0.2142"]=1; # portId
  monitored_field["1.3.18.0.0.2216"]=1; # graphId
}
function indicate_progress()
{
  if (progress && (verbose || NR % 100 == 0)) {
    printf("\rLines=%d OPS=%d Objects=%d Links=%d Values=%d", NR, nops, nobjs, nlinks, nobjvals);
  }
}
function print_header(ntype)
{
#  printf("%d|%d|%d|%d|", batchid, NR, nops, ntype);
}
function set_goal(goal_name)
{
  goal=goal_name;
  if (verbose)
    printf("Line %d: Goal=%s\n", NR, goal);
}
/[ ][ ]*$/ {
  sub("[ ][ ]*$", "");
}
NF==0 {
  continue;
}
/^[-][-].*/ {
  continue;
}
wrap_lines==1 && $0 ~ /['\''][ ]*$/ { # wrapped line
  while ($0 ~ /['\''][ ]*$/) {
    indicate_progress();
    sub("^[ ]*", "");
    sub("^['\'']", "");
    sub("['\''][ ]*$", "");
    line=$0;
    getline;
    sub("^[ ]*", "");
    sub("^['\'']", "");
    $0=sprintf("%s%s", line, $0);
    if (verbose) print "Line " NR ": wrapped line=" $0;
  }
  wrap_lines=0;
}
/^OP/ {
  ++nops;
  indicate_progress();
  op_specifier=$0;
  gsub("['\'']", "");
  set_goal("op");
  wrap_lines=1;
}
/HAS_INSTANCE/ {
  set_goal("object");
  class=$2;
  name="";
}
goal=="object" && NF>3 {
  $0=$4;
}
goal=="object" && $0 ~ /[;][ ]*$/ {
  sub("['\'']*[;]$", "");
  sub("^[ ]*['\'']", "");
  name=$0;
  if (class ~ /^[0-9]/) { # Filter out non-numerical classes
    ++nobjs;
    print_header(nobjs);
    printf("%s|%s\n", class, name) >> object_file;
  }
  indicate_progress();
  set_goal("");
  wrap_lines=0;
  name="";
  continue;
}
goal=="op" && $2 ~ /[.][ ]*$/ {
  set_goal("link_or_value");
  sub("[.][ ]*$", "", $2);
  class=$2;
  if (NR > 2) {
    expr=".*" $2 "[ ]*";
    sub(expr, "");
    name=$0;
  }
  continue;
}
goal=="link_or_value" && $0 ~ /[.][ ]*$/ {
  sub("['\'']*[.][ ]*$", "", $0);
  sub("^[ ]*['\'']", "", $0);
  if (name == "")
    name = $0;
  else
    field = $0;
  if (verbose) {
    print "  name=" name;
    print "  field=" field;
  }
  continue;
}
goal=="link_or_value" && $0 ~ /['\''][^'\'']*['\'']/ {
  field=$0;
  sub("^[ ]*", "", field);
  gsub("['\'']", "", field);
}
goal=="link_or_value" && $1=="IS_LINKED_TO" {
  set_goal("link");
  srcClass=class;
  srcName=name;
  srcField=field;
  dstClass="";
  dstName="";
  dstField="";
  wrap_lines=1;
  continue;
}
goal=="link_or_value" && $1=="VALUE" {
  set_goal("value");
}
goal=="link" && $0 ~ /['\''][.][ ]*$/ {
  if (verbose) print "Line " NR ": Dst link";
  gsub("['\'']", "");
  sub("^[ ]*", "");
  sub("[.][ ]*$", "");
  if (dstClass == "") {
    dstClass=$0;
    if (verbose)
      printf("Line %s: dstClass=%s\n", NR, dstClass);
  }
  else if (dstName == "") {
    dstName=$0;
    if (verbose)
      printf("Line %s: dstName=%s\n", NR, dstName);
  }
}
goal=="link" && $0 ~ /['\''][;][ ]*$/ {
  gsub("['\'']", "");
  sub("^[ ]*", "");
  sub("[;][ ]*$", "");
  dstField=$0;
  if (srcClass ~ /^[0-9]/ && dstClass ~ /^[0-9]/) { # Filter out non-numerical classes
    ++nlinks;
    print_header(nlinks);
    printf("%s|%s|%s|%s|%s|%s\n", srcClass, srcName, srcField, dstClass, dstName, dstField) >> link_file;
  }
  indicate_progress();
  wrap_lines=0;
  set_goal("");
  name="";
  class="";
  field="";
}
goal=="value" && $1 ~ /[(]INTEGER[)]/ {
  BinVal="";
  TextVal=$2;
  sub("[;]$", "", TextVal);
#  print "value: " TextVal;
  have_value=1;
}
goal=="value" && $0 ~ /[(]CHARVAR[)]/ {
  BinVal="";
  TextVal=$2;
  sub("^['\'']", "", TextVal);
  sub("[;]$", "", TextVal);
  sub("['\'']$", "", TextVal);
#  print "value: " TextVal;
  have_value=1;
}
goal=="value" && $0 ~ /[(]ANONYMOUSVAR[)]/ {
  BinVal=$2;
  TextVal="";
  sub("^X['\'']", "", BinVal);
  sub("[;]$", "", BinVal);
  sub("['\'']$", "", BinVal);
#  print "value: " BinVal;
  have_value=1;
}
have_value {
  indicate_progress();
  if (class ~ /^[0-9]/ && monitored_field[field]==1) {
    ++nobjvals;
    print_header(nobjvals);
    printf("%s|%s|%s|%s|%s\n", class, name, field, TextVal, BinVal) >> objvalscalar_file;
  }
  have_value=0;
  set_goal("");
  continue;
}
END {
  progress=1;
  verbose=1;
  indicate_progress();
  if (verbose)
    printf("\n");
}' $(echo $* | sed 's?[\\]?/?g')
