# $Header:   //ARCHIVE/Archive/scripts/SendMVSRODMRegisterObjects.ksv   1.3   Jan 12 2001 13:10:30   BRENT  $
#
# SendMVSRODMRegisterObjects.ksh

# this korn shell script is called from the _RODMRegister stored
# procedure to register an object or group of objects, probablyin response to
# a request from the GUI client.

#
# set -x

delay_seconds=0;
bProcessBatchOnly=0;
bProcessMJOBsOnly=0;
os_id=NULL
reg_unreg_control_statments='';  # default to registering objects

# Name of program used to execute SQL commands
SQLCMD=${SQLCMD:-isql}

usage	()
{
  print ''
  print 'Usage: '$0' -c<cid> -i<id> -v RegistrationStatus=<status> -v RegistrationScope=<scope> -v ResponseType=<response> [-b] [-m] [-u] [-o<os_id>] [-d<delay_seconds>]'
  print ''
  print '  Required Param(s):'
  print '  -c<cid>                 Object''s ClassID'
  print '  -i<id>                  Object''s ID'
  print '  -n<name>                Object''s name'
  print '  -v RegistrationStatus=<status>    0=unregister, 1=register, 2=reregister'
  print '  -v RegistrationScope=<scope>      1=register root object only, 2=register descendants only, 3=register root object and descendants'
  print '  -v ResponseType=<response>        0=OFF (never), 1=EX (errors only), 2=DEF (always)'
  print ''
  print '  Optional Param(s):'
  print '  -b                      SysPlex batch jobs only'
  print '  -m                      MVS jobs only.  No SysPlex batch jobs.'
  print '  -u                      Unregister the object(s)'
  print '  -o<os_id>               OS Object''s ID to limit output to that OS'
  print '  -d<delay_seconds>       Number of seconds before command is executed (default='$delay_seconds')'
  print ''
  print '  Sample:'
  print '    '$0' -cSNNT -i1 -n"USIBMNT" -vRegistrationStatus=1 -vRegistrationScope=3 -vResponseType=1'
}


# Process input parameters
while getopts "v:c:i:bmuo:n:d:?" opt; do
	case $opt in
	c)  cid=$OPTARG;;
	i)  id=$OPTARG;;
	n)  obj_name=$OPTARG;;
	o)  os_id=$OPTARG;;
	b)  bProcessBatchOnly=1;;
  v)  eval $OPTARG;;
	m)  bProcessMJOBsOnly=1;;
	u)  reg_unreg_control_statments='47OFF';;
	d)  delay_seconds=$OPTARG;;
	\?) usage
		  exit 16
		  ;;
	esac
done
shift $(($OPTIND - 1))

# Check for required params
typeset -i bMissingParam=0;
if [ "$cid" = "" ];         then echo 'ERROR: Missing required param -c<cid>'; bMissingParam=1; fi
if [ "$id" = "" ];          then echo 'ERROR: Missing required param -i<id>'; bMissingParam=1; fi
if [ "$obj_name" = "" ];          then echo 'ERROR: Missing required param -n<name>'; bMissingParam=1; fi
if [ "$RegistrationStatus" = "" ]; then echo 'ERROR: Missing required param -v RegistrationStatus=<status>'; bMissingParam=1; fi
if [ "$RegistrationScope" = "" ]; then echo 'ERROR: Missing required param -v RegistrationScope=<scope>'; bMissingParam=1; fi
if [ "$ResponseType" = "" ]; then echo 'ERROR: Missing required param -v ResponseType=<response>'; bMissingParam=1; fi
if  [ $bMissingParam -eq 1 ]; then
	usage
	exit 17
fi

# check for conflicting parameters
if [ $bProcessBatchOnly -eq 1 -a $bProcessMJOBsOnly -eq 1 ]; then;
  echo 'ERROR: -m and -b options are mutually exclusive.'
  usage
  exit 18
fi

main()
{
  timestamp=`date +"%Y%m%d%H%M%S"`

  # Get Database Server, User, Password from Registry
  eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" | awk '/Server/ {printf("db_server=%s\n",$3);} /User/ {printf("db_user=%s\n",$3);} /Password/ {printf("db_password=%s\n",$3);}'`

  # determine the output directory
  eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\Log" | awk '/Directory/ {gsub(/\\\\/, "/", $3); printf("output_path=%s\n",$3);}'`
  if [ "$output_path" = "" ]; then;
    if [ "$TEMP" != "" ]; then; output_path=`echo $TEMP | awk '{gsub(/\\\\/, "/"); print;}'`;
    elif [ "$TMP" != "" ]; then; output_path=`echo $TMP | awk '{gsub(/\\\\/, "/"); print;}'`;
    elif [ "$TMPDIR" != "" ]; then; output_path=`echo $TMPDIR | awk '{gsub(/\\\\/, "/"); print;}'`;
    fi;
  fi

  output_path="$output_path/MVSRODMRegisterObjects.$timestamp"

  master_output="$output_path.master.txt"
  isql_cmd="set nocount on exec asisp_uploadGetRODMObjects '$cid', $id, $RegistrationStatus, $RegistrationScope, $ResponseType"
  $SQLCMD -S$db_server -U$db_user -P$db_password -Q"$isql_cmd" -w5000 -o$master_output

  awk -F'|' -v output_path=$output_path '
    NF > 0 && NR > 2 { 
      node = $1 + 0
      count[node] ++
      filename = output_path "." node ".txt"
      if (count[node] == 1) {
        system("rm -f " filename)
        close("rm -f " filename)
      }
      sub(" *$", "", $2)
      print $2 >> filename
    } 
    END { 
      for (node in count) print node 
    }' $master_output | \
  while read os_id
  do
    output_file="${output_path}.${os_id}.txt"

    # create audit_msg file
    audit_msg_file="$output_file".AUDIT.MSG
    echo $cid'|'$id'|ASI_05\\02|RODM Register Request (05\\02)'" os=$os_id name=$obj_name cid=$cid id=$id status=$RegistrationStatus scope=$RegistrationScope" > "$audit_msg_file";

    # perform the upload
    if [ $delay_seconds -eq 0 ]; then;
      Upload_WrapAndSendAsFTP -cOS -i$os_id -f"$output_file"
    else
      SOON $delay_seconds sh Upload_WrapAndSendAsFTP -cOS -i$os_id -f"${output_path}.${os_id}.txt"
    fi
  done

  if [ -f $master_output ]
  then
    rm -f $master_output
  fi
}

main
