-- $Header:   //ARCHIVE/Archive/DB/sql/CreateNotifyLinkInsertTrigger.sqv   1.0   Feb 04 2001 12:34:28   BRENT  $
--
-- CreateNotifyLinkInsertTrigger.sql
--

USE Object
GO


---------------------------------------------------------------------------------------------------
-- CreateNotifyLinkInsertTrigger
PRINT 'Creating procedure CreateNotifyLinkInsertTrigger'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'CreateNotifyLinkInsertTrigger' AND type = 'P')
  DROP PROCEDURE CreateNotifyLinkInsertTrigger
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/CreateNotifyLinkInsertTrigger.sqv  $
-- $Revision:   1.0  $
-- $Date:   Feb 04 2001 12:34:28  $
-- $Author:   BRENT  $
-- Creates an insert trigger for the link table with the specified relationship. This trigger is
-- the same as the standard implementation, but also adds an insert statement to put records into
-- the PendingNotification table for the new rows.
CREATE PROCEDURE CreateNotifyLinkInsertTrigger
  @src_cid    ClassID,
  @dst_cid    ClassID,
  @link_type  VARCHAR(4)
AS
  DECLARE @table sysname, @containment_insert_sql VARCHAR(255)
  IF EXISTS (SELECT * FROM link_type WHERE link_type = @link_type AND flag_containment = 1)
    SELECT @containment_insert_sql = "
  INSERT containment(src_cid, src_id, dst_cid, dst_id)
    SELECT '" + @src_cid + "', src_id, '" + @dst_cid + "', dst_id FROM inserted"

  SELECT @table = RTRIM(@src_cid) + '_' + RTRIM(@dst_cid) + '_' + @link_type
  PRINT 'Creating notification insert trigger for ' + @table

  EXEC("
-- $Archive:   //ARCHIVE/Archive/DB/sql/CreateNotifyLinkInsertTrigger.sqv  $
-- $Revision:   1.0  $
-- $Date:   Feb 04 2001 12:34:28  $
-- $Author:   BRENT  $
-- Insert trigger generated by CreateNotifyLinkInsertTrigger that puts records into the
-- PendingNotification table.
CREATE TRIGGER tI_" + @table + " ON " + @table + " FOR INSERT AS
  DECLARE @rc INT, @src_id ObjID, @dst_id ObjID
  SELECT @rc=@@rowcount " + @containment_insert_sql + "
  INSERT link(link_type, src_cid, src_id, dst_cid, dst_id)
    SELECT '" + @link_type + "', '" + @src_cid + "', src_id, '" + @dst_cid + "', dst_id FROM inserted

  -- Send LinkCreated notifications
  INSERT INTO PendingNotification(type, fields, detail)
    SELECT
      et.type,
      '" + @link_type + "' + '|' +
      CONVERT(VARCHAR(9), ISNULL(src.cno, 0))   + '|' +
      CONVERT(VARCHAR(9), ISNULL(i.src_id, 0))  + '|' +
      CONVERT(VARCHAR(9), ISNULL(dst.cno, 0))   + '|' +
      CONVERT(VARCHAR(9), ISNULL(i.dst_id, 0))  + '|' +
      CONVERT(VARCHAR(9), ISNULL(lt.flags, 0)),
      ''
    FROM
      EventType AS et,
      obj_class AS src,
      obj_class AS dst,
      link_type AS lt,
      inserted AS i
    WHERE
      et.name = 'LinkCreated'
      AND src.cid = '" + @src_cid + "'
      AND dst.cid = '" + @dst_cid + "'
      AND lt.link_type = '" + @link_type + "'
")
GO
