-- $Header:   P:/Archive/DB/SQL/RODMCreateTableProcs.sqv   1.4   Dec 04 2000 12:33:44   BRENT  $
--
-- RODMCreateTableProcs.sql
--
-- Defines the procedures for creating the RODMManagedObject and RODMManagedLink tables,
-- which get created in both the Object and the RODMLoad databases.

/*
$Log:   P:/Archive/DB/SQL/RODMCreateTableProcs.sqv  $
   
      Rev 1.4   Dec 04 2000 12:33:44   BRENT
   Fixed syntax error in last rev
   
      Rev 1.3   Dec 04 2000 11:26:34   BRENT
   Changed primary key of RODMManagedObject table to _RODMClass, _RODMMyName
   
      Rev 1.2   Nov 30 2000 16:05:46   BRENT
   Added userid column to RODMManagedObject table
   
      Rev 1.1   Nov 30 2000 14:05:12   BRENT
   Added a new column called DiscoverAction to the ManagedObject and ManagedLink 
   tables to aid in discovery
 */

use master
go


---------------------------------------------------------------------------------------------------
-- CreateRODMManagedObject
PRINT 'Creating procedure CreateRODMManagedObject'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'CreateRODMManagedObject' AND type = 'P')
  DROP PROCEDURE CreateRODMManagedObject
GO

CREATE PROCEDURE CreateRODMManagedObject
  @dbname sysname
AS
  IF @dbname = 'master'
  BEGIN
    PRINT 'Cannot run CreateRODMManagedObject on the master database'
    RETURN -30000
  END

  EXEC("
  USE " + @dbname + "
  
  -- Table RODMManagedObject
  IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMManagedObject' AND type = 'U')
  BEGIN
    PRINT 'Creating table RODMManagedObject'
    CREATE TABLE RODMManagedObject
    (
      -- Base TBSM ManagedObject columns
      cid                 ClassID           CONSTRAINT RODMMO_cid_id_UQ UNIQUE (cid, id)
                                            CONSTRAINT RODMMO_cid_DF DEFAULT '',
      id                  ObjID             IDENTITY(1,1),
      name                ObjName           CONSTRAINT RODMMO_name_DF DEFAULT '',
      revision            INT               CONSTRAINT RODMMO_revision_DF DEFAULT 0,
      nchild              INT               CONSTRAINT RODMMO_nchild_DF DEFAULT 0,
      description         VARCHAR(255) NULL CONSTRAINT RODMMO_description_DF DEFAULT NULL,
      ctime               DATETIME          CONSTRAINT RODMMO_ctime_DF DEFAULT GETDATE(),
      mtime               DATETIME          CONSTRAINT RODMMO_mtime_DF DEFAULT GETDATE(),
      userid              UserID            CONSTRAINT RODMMO_userid_DF DEFAULT 'SYSADM',
      deleted             TINYINT           CONSTRAINT RODMMO_deleted_DF DEFAULT 0,
      _AlertStateID       INT               CONSTRAINT RODMMO_AlertStateID_DF DEFAULT 1,
      _PriorityID         INT               CONSTRAINT RODMMO_PriorityID_DF DEFAULT 6,
      _CurrentStateID     INT               CONSTRAINT RODMMO_CurrentStateID_DF DEFAULT 0,
      _DesiredStateID     INT               CONSTRAINT RODMMO_DesiredStateID_DF DEFAULT 0,
      _HasOwnershipNote   TINYINT           CONSTRAINT RODMMO_HasOwnershipNoteID_DF DEFAULT 0,
      _ChildEventMatrixID INT               CONSTRAINT RODMMO_ChildEventMatrixID_DF DEFAULT 0,
      _ExceptionMatrixID  INT               CONSTRAINT RODMMO_ExceptionMatrixID_DF DEFAULT 0,
      DiscoverAction      VARCHAR(8) NULL   CONSTRAINT RODMMO_DiscoverAction_DF DEFAULT NULL,

      -- Columns that map to RODMObject table
      _RODMClass          VARCHAR(60)       CONSTRAINT RODMMO_PK PRIMARY KEY (_RODMClass, _RODMMyName),
      _RODMMyName         VARCHAR(255),
      _NetworkID          VARCHAR(40)       CONSTRAINT RODMMO_NetworkID_DF DEFAULT '',

      -- Columns for TBSM RODMStatesObject class
      _RegistrationStatus TINYINT           CONSTRAINT RODMMO_RegistrationStatus_DF DEFAULT 0,
      _DisplayResourceName VARCHAR(255) NULL CONSTRAINT RODMMO_DisplayResourceName_DF DEFAULT NULL,
      _DisplayStatus      INT             CONSTRAINT RODMMO_DisplayStatus_DF DEFAULT 132,

      _operationalState   TINYINT         CONSTRAINT RODMMO_operationalState_DF DEFAULT 0xff,
      _usageState         TINYINT         CONSTRAINT RODMMO_usageState_DF DEFAULT 0xff,
      _administrativeState TINYINT        CONSTRAINT RODMMO_adminstrativeState_DF DEFAULT 0xff,
      _availabilityStatus TINYINT         CONSTRAINT RODMMO_availabilityState_DF DEFAULT 0x00,
      _proceduralStatus   TINYINT         CONSTRAINT RODMMO_proceduralState_DF DEFAULT 0x00,
      _unknownStatus      TINYINT         CONSTRAINT RODMMO_unknownState_DF DEFAULT 0x01,
      _nativeStatus       TINYINT         CONSTRAINT RODMMO_nativeState_DF DEFAULT 0xff,

    )
  END
  ")
GO


---------------------------------------------------------------------------------------------------
-- CreateRODMManagedLink
PRINT 'Creating procedure CreateRODMManagedLink'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'CreateRODMManagedLink' AND type = 'P')
  DROP PROCEDURE CreateRODMManagedLink
GO

CREATE PROCEDURE CreateRODMManagedLink
  @dbname sysname
AS
  IF @dbname = 'master'
  BEGIN
    PRINT 'Cannot run CreateRODMManagedLink on the master database'
    RETURN -30000
  END

  EXEC("
  USE " + @dbname + "
  
  IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMManagedLink' AND type = 'U')
  BEGIN
    PRINT 'Creating table RODMManagedLink'
    CREATE TABLE RODMManagedLink
    (
      link_type    VARCHAR(4)     CONSTRAINT RODMML_PK PRIMARY KEY (link_type, SrcClass, SrcMyName, DstClass, DstMyName),
      SrcClass     VARCHAR(60),
      SrcMyName    VARCHAR(255),
      DstClass     VARCHAR(60),
      DstMyName    VARCHAR(255),
      SrcField     VARCHAR(60),
      DstField     VARCHAR(60),
      src_cid      ClassID,
      src_id       ObjID,
      dst_cid      ClassID,
      dst_id       ObjID,
      ctime        DATETIME       CONSTRAINT RODMML_ctime_DF DEFAULT GETDATE(),
      mtime        DATETIME       CONSTRAINT RODMML_mtime_DF DEFAULT GETDATE(),
      DiscoverAction      VARCHAR(8) NULL   CONSTRAINT RODMML_DiscoverAction_DF DEFAULT NULL
    )

    ALTER TABLE RODMManagedLink ADD
      CONSTRAINT RODMML_src_FK FOREIGN KEY (src_cid, src_id) REFERENCES RODMManagedObject (cid, id),
      CONSTRAINT RODMML_dst_FK FOREIGN KEY (dst_cid, dst_id) REFERENCES RODMManagedObject (cid, id),
      CONSTRAINT RODMML_src_dst_UQ UNIQUE (link_type, src_cid, src_id, dst_cid, dst_id)
  END
  ")
GO
