-- $Header:   //ARCHIVE/Archive/DB/sql/RODMLoad.sqv   1.12   Feb 04 2001 12:46:18   BRENT  $
--
-- RODMLoad.sql
--
-- Defines the RODM-related database objects that reside in the RODMLoad database

USE master
GO

IF NOT EXISTS (SELECT * FROM sysdatabases WHERE name = 'RODMLoad')
  CREATE DATABASE RODMLoad
GO

USE RODMLoad
GO

----------------------------------------------------------------------------------------------------
-- Check for views of DiscoveryBatch and DiscoveryBatchPStatVal

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'CreateObjectTableView' AND type = 'P')
  DROP PROCEDURE CreateObjectTableView
GO

CREATE PROCEDURE CreateObjectTableView
  @table  sysname
AS
  IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = @table AND type = 'U')
  BEGIN
    PRINT @table + ' table does not exist...'
    IF EXISTS (SELECT * FROM master..sysdatabases WHERE name = 'Object')
    BEGIN
      PRINT '...Object database found'
      IF EXISTS (SELECT * FROM Object..sysobjects WHERE name = @table)
      BEGIN
        PRINT '...Object..' + @table + ' table found, creating view'
        IF EXISTS (SELECT * FROM sysobjects WHERE name = @table AND type = 'V')
          EXEC("
          DROP VIEW " + @table
          )
        EXEC("
        CREATE VIEW " + @table + " AS SELECT * FROM Object.." + @table
        )
      END
      ELSE
        PRINT 'Object..' + @table + ' table does not exist!'
    END
    ELSE
      PRINT 'Object database does not exist!'
  END
  ELSE
    PRINT @table + ' already exists as a table in this database, nothing done!'
GO

EXEC CreateObjectTableView DiscoveryBatchPStatVal
EXEC CreateObjectTableView DiscoveryBatch
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'CreateObjectTableView' AND type = 'P')
  DROP PROCEDURE CreateObjectTableView
GO


----------------------------------------------------------------------------------------------------
-- RODM meta data tables


-- DROP TABLE RODMClassMap
-- Maps RODM classes to TBSM classes.
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMClassMap' AND type = 'U')
BEGIN
  PRINT 'Creating table RODMClassMap'
  EXEC("
  CREATE TABLE RODMClassMap
  (
    cid                 ClassID NULL,         -- TBSM class id. Null indicates class is not mapped
    cname               VARCHAR(24) NULL,     -- TBSM class name. Null indicates class not mapped
    Class               VARCHAR(40) NOT NULL, -- Human-readable RODM class name.
    Role                VARCHAR(40) NOT NULL, -- Specifies role of RODM classes with multiple rows.
                                              -- Blank if class has only one role
    Description         VARCHAR(2000) NULL,   -- Description of class
    RODMClass           VARCHAR(40) NOT NULL, -- Internal RODM class identifier. Sometimes is
                                              -- exactly the same as Class, but is sometimes a
                                              -- different value, such as an ASN.1 specification
                                              -- For Class=logicalUnit RODMClass=1.3.18.0.0.1829
    ParentClass         VARCHAR(40) NOT NULL  -- Internal RODM class identifier for superclass.
  )
  ")
  EXEC("
  ALTER TABLE RODMClassMap ADD
    CONSTRAINT RODMClassMap_PK PRIMARY KEY (Class, Role),
    CONSTRAINT RODMClassMap_cid_UQ UNIQUE (Class, Role, cid),
    CONSTRAINT RODMClassMap_cname_UQ UNIQUE (Class, Role, cname)
  ")
END


-- DROP TABLE RODMFieldMap
-- Maps RODM fields to TBSM attributes.
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMFieldMap' AND type = 'U')
BEGIN
  PRINT 'Creating table RODMFieldMap'
  EXEC("
  CREATE TABLE RODMFieldMap
  (
    cid                 ClassID NULL,           -- TBSM Class id
    name                VARCHAR(24) NULL,       -- TBSM attribute name (usually same as Field)
    IsMapped            TINYINT NOT NULL,       -- Is field mapped to TBSM attribute (0=no, 1=yes)?
    Class               VARCHAR(40) NOT NULL,   -- Human-readable class name
    Role                VARCHAR(40) NOT NULL,   -- Role for multi-roled classes or blank
    Field               VARCHAR(80) NOT NULL,   -- Human-readable field name
    RODMField           VARCHAR(40) NOT NULL,   -- Internal field name (may be same as Field)
    DataType            VARCHAR(16) NOT NULL,   -- Field data type name (e.g. 'CHARVAR')
    Type                CHAR(1) NOT NULL,       -- Field type indicator (from documentation)
    ValidValues         VARCHAR(2000) NULL,     -- Description of valid values for the field
    Mod                 VARCHAR(4) NOT NULL,    -- Indicates if and when field is user-modifiable
    O_R                 VARCHAR(3) NOT NULL,    -- Optional or required indicator   
    Default_Value       VARCHAR(100) NULL       -- String representation of default value
  )
  ")
  EXEC("
  ALTER TABLE RODMFieldMap ADD
    CONSTRAINT RODMFieldMap_PK PRIMARY KEY (Class, Role, Field),
    CONSTRAINT RODMFieldMap_Class_FK FOREIGN KEY (Class, Role) REFERENCES RODMClassMap (Class, Role),
    CONSTRAINT RODMFieldMap_IsMapped_CK CHECK (IsMapped IN (0, 1))
  ")
END


-- DROP Table RODMFieldLinkMap
-- Maps RODM links to TBSM links
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMFieldLinkMap' AND type = 'U')
BEGIN
  PRINT 'Creating table RODMFieldLinkMap'
  EXEC("
  CREATE TABLE RODMFieldLinkMap
  (
     SrcClass           VARCHAR(60) NOT NULL,
     SrcField           VARCHAR(40) NOT NULL,
     DstClass           VARCHAR(60) NOT NULL,
     DstField           VARCHAR(40) NOT NULL,
     link_type          VARCHAR(4) NOT NULL,
     direction          SMALLINT NOT NULL       -- Direction is used to determine whether the TBSM
                                                -- link is established in the same direction as the
                                                -- RODM link (direction=1) or whether it should be
                                                -- reversed (direction=-1)
  )
  ")
  EXEC("
  ALTER TABLE RODMFieldLinkMap ADD
    CONSTRAINT RODMFieldLinkMap_PK PRIMARY KEY (SrcClass, SrcField, DstClass, DstField),
    CONSTRAINT RODMFieldLinkMap_direction_CK CHECK (direction IN (-1, 1))
  ")
END


-- DROP Table RODMClassLinkMap
-- Used to determine TBSM class of a RODM object based on the presence of links
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMClassLinkMap' AND type = 'U')
BEGIN
  PRINT 'Creating table RODMClassLinkMap'
  EXEC("
  CREATE TABLE RODMClassLinkMap
  (
     SrcClass           VARCHAR(60) NOT NULL,
     SrcField           VARCHAR(40) NOT NULL,
     DstClass           VARCHAR(60) NOT NULL,
     DstField           VARCHAR(40) NOT NULL,
     cid                ClassID NOT NULL,
     direction          SMALLINT NOT NULL,      -- direction is used to specify whether the cid 
                                                -- specifies the SrcClass (-1) or DstClass (1)
     priority           SMALLINT NOT NULL CONSTRAINT RODMClassLinkMap_priority_DF DEFAULT 0
  )
  ALTER TABLE RODMClassLinkMap ADD
    CONSTRAINT RODMClassLinkMap_PK PRIMARY KEY (SrcClass, SrcField, DstClass, DstField),
    CONSTRAINT RODMClassLinkMap_direction_CK CHECK (direction IN (-1, 1))
  ")
END


-- DROP Table RODMNameField
-- Specifies which RODM field is used to populate the TBSM object name
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMNameField' AND type = 'U')
BEGIN
  PRINT 'Creating table RODMNameField'
  EXEC("
  CREATE TABLE RODMNameField
  (
    cid                 ClassID NOT NULL,
    Field               VARCHAR(40) NOT NULL,
    priority            INT NOT NULL  CONSTRAINT RODMNameField_priority_DF DEFAULT 0,
    num_matches         INT NOT NULL  CONSTRAINT RODMNameField_num_patches_DF DEFAULT 0
  )
  ")
  EXEC("
  ALTER TABLE RODMNameField ADD
    CONSTRAINT RODMNameField_PK PRIMARY KEY (cid, Field)
  ")
END


-- DROP TABLE RODMUnmanagedClass
-- Discovery configuration table that lists classes that are to be discarded solely on the
-- basis of the class name. 
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMUnmanagedClass' AND type = 'U')
BEGIN
  PRINT 'Creating table RODMUnmanagedClass'
  EXEC("
  CREATE TABLE RODMUnmanagedClass
  (
    Class               VARCHAR(60) NOT NULL
  )
  ")
  EXEC("
  ALTER TABLE RODMUnmanagedClass ADD
    CONSTRAINT RODMUMC_PK PRIMARY KEY(Class)
  ")
END


-- DROP TABLE RODMTopLevelClass
-- Discovery configuration table that lists classes that to be represented directly
-- under the containing network object 
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMTopLevelClass' AND type = 'U')
BEGIN
  PRINT 'Creating table RODMTopLevelClass'
  EXEC("
  CREATE TABLE RODMTopLevelClass
  (
    cid                 ClassID NOT NULL
  )
  ")
  EXEC("
  ALTER TABLE RODMTopLevelClass ADD
    CONSTRAINT RODMTLC_PK PRIMARY KEY(cid)
  ")
END


-- DROP TABLE RODMClassIdLookup
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMClassIdLookup' AND type = 'U')
BEGIN
  PRINT 'Creating table RODMClassIdLookup'
  EXEC("
  CREATE TABLE RODMClassIdLookup
  (
    id                     INT IDENTITY(1,1),
    cid_assignment_expr    VARCHAR(80) NOT NULL,
    where_clause           VARCHAR(900) NOT NULL,
    priority               SMALLINT NOT NULL  CONSTRAINT RODMClassIdLookup_priority_DF DEFAULT 0,
    enabled                TINYINT NOT NULL   CONSTRAINT RODMClassIdLookup_enabled_DF DEFAULT 1,
    description            VARCHAR(255) NULL,
    num_matches            INT NOT NULL       CONSTRAINT RODMClassIdLookup_num_matches_DF DEFAULT 0
  )
  ")
  EXEC("
  ALTER TABLE RODMClassIdLookup ADD
    CONSTRAINT RODMClassIdLookup_PK PRIMARY KEY (where_clause)
  ")
END


-- DROP TABLE RODMLinkLookup
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMLinkLookup' AND type = 'U')
BEGIN
  PRINT 'Creating table RODMLinkLookup'
  EXEC("
  CREATE TABLE RODMLinkLookup
  (
    id                     INT NOT NULL IDENTITY(1,1),
    link_type              VARCHAR(4) NOT NULL,
    src_cid                VARCHAR(4) NOT NULL,
    dst_cid                VARCHAR(4) NOT NULL,
    where_clause           VARCHAR(2000) NOT NULL,
    priority               SMALLINT NOT NULL  CONSTRAINT RODMLinkLookup_priority_DF DEFAULT 0,
    enabled                TINYINT NOT NULL   CONSTRAINT RODMLinkLookup_enabled_DF DEFAULT 1,
    description            VARCHAR(255) NOT NULL,
    num_matches            INT NOT NULL       CONSTRAINT RODMLinkLookup_num_matches_DF DEFAULT 0
  )
  ")
  EXEC("
  ALTER TABLE RODMLinkLookup ADD
    CONSTRAINT RODMLinkLookup_PK PRIMARY KEY (link_type, src_cid, dst_cid, description)
  ")
END


-- DROP Table RODM_isa
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODM_isa' AND type = 'U')
BEGIN
  PRINT 'Creating table RODM_isa'
  EXEC("
  CREATE TABLE RODM_isa
  (
    cid                 ClassID NOT NULL,
    base_cid            ClassID NOT NULL
  )
  ")
  EXEC("
  ALTER TABLE RODM_isa ADD
    CONSTRAINT RODM_isa_PK PRIMARY KEY (cid, base_cid)
  ")
END


-- DROP TABLE RODM_isa_chain
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODM_isa_chain' AND type = 'U')
BEGIN
  PRINT 'Creating table RODM_isa_chain'
  EXEC("
  CREATE TABLE RODM_isa_chain
  (
    cid                 ClassID NOT NULL,
    generation          INT NOT NULL,
    derived_cid         ClassID NOT NULL,
    base_cid            ClassID NOT NULL
  )
  ")
  EXEC("
  ALTER TABLE RODM_isa_chain ADD
    CONSTRAINT RODM_isa_chain_PK PRIMARY KEY (cid, generation, derived_cid, base_cid)
  ")
END


----------------------------------------------------------------------------------------------------
-- RODM instance data tables

-- DROP TABLE RODMObject
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMObject' AND type = 'U')
BEGIN
  PRINT 'Creating table RODMObject'
  EXEC("
  CREATE TABLE RODMObject
  (
    Class          VARCHAR(60) NOT NULL,
    MyName         VARCHAR(255) NOT NULL
  )
  ")
  EXEC("
  ALTER TABLE RODMObject ADD
    CONSTRAINT RODMObject_Class_MyName_UQ UNIQUE (Class, MyName)
  ")
END


-- DROP TABLE RODMLink
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMLink' AND type = 'U')
BEGIN
  PRINT 'Creating table RODMLink'
  EXEC("
  CREATE TABLE RODMLink
  (
    SrcClass            VARCHAR(60) NOT NULL,
    SrcMyName           VARCHAR(255) NOT NULL,
    SrcField            VARCHAR(40) NOT NULL,
    DstClass            VARCHAR(60) NOT NULL,
    DstMyName           VARCHAR(255) NOT NULL,
    DstField            VARCHAR(40) NOT NULL
  )
  ")
  EXEC("
  ALTER TABLE RODMLink ADD
    CONSTRAINT RODMLink_PK PRIMARY KEY (SrcClass, SrcMyName, SrcField, DstClass, DstMyName, DstField)
  ")
END


-- DROP TABLE RODMObjValScalar
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMObjValScalar' AND type = 'U')
BEGIN
  PRINT 'Creating table RODMObjValScalar'
  EXEC("
  CREATE TABLE RODMObjValScalar
  (
    Class               VARCHAR(60) NOT NULL,
    MyName              VARCHAR(255) NOT NULL,
    Field               VARCHAR(40) NOT NULL,
    TextVal             VARCHAR(500) NULL   CONSTRAINT RODMObjValScalar_TextVal_DF DEFAULT NULL,
    BinVal              VARBINARY(500) NULL CONSTRAINT RODMObjValScalar_BinVal_DF DEFAULT NULL
  )
  ")
  EXEC("
  ALTER TABLE RODMObjValScalar ADD
    CONSTRAINT RODMObjValScalar_PK PRIMARY KEY (Class, MyName, Field)
  ")
END


-- DROP TABLE RODMObjValSelfDefining
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMObjValSelfDefining' AND type = 'U')
BEGIN
  PRINT 'Creating table RODMObjValSelfDefining'
  EXEC("
  CREATE TABLE RODMObjValSelfDefining
  (
    Class               VARCHAR(60) NOT NULL,
    MyName              VARCHAR(255) NOT NULL,
    Field               VARCHAR(40) NOT NULL,
    RowNum              INT NOT NULL,
    Value               VARCHAR(255) NULL
  )
  ")
  EXEC("
  ALTER TABLE RODMObjValSelfDefining ADD
    CONSTRAINT RODMObjValSelfDefining_PK PRIMARY KEY (Class, MyName, Field, RowNum)
  ")
END


-- DROP TABLE RODMObjValIndexList
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMObjValIndexList' AND type = 'U')
BEGIN
  PRINT 'Creating table RODMObjValIndexList'
  EXEC("
  CREATE TABLE RODMObjValIndexList
  (
    Class               VARCHAR(60) NOT NULL,
    MyName              VARCHAR(255) NOT NULL,
    Field               VARCHAR(40) NOT NULL,
    RowNum              INT NOT NULL,
    Value               VARCHAR(255) NULL
  )
  ")
  EXEC("
  ALTER TABLE RODMObjValIndexList ADD
    CONSTRAINT RODMObjValIndexList_PK PRIMARY KEY (Class, MyName, Field, RowNum)
  ")
END

-- Check for existance of old view and get rid of it since RODMManagedObject table now
-- exists in the RODMLoad database.
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMManagedObject' AND type = 'V')
  DROP VIEW RODMManagedObject
GO

EXEC master..CreateRODMManagedObject RODMLoad

EXEC master..CreateRODMManagedLink RODMLoad


---------------------------------------------------------------------------------------------------
-- insert_RODM_isa
PRINT 'Creating procedure insert_RODM_isa'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'insert_RODM_isa' AND type='P')
  DROP PROCEDURE insert_RODM_isa
GO

CREATE PROCEDURE insert_RODM_isa
  @cid ClassID,
  @base_cid ClassID
AS
  IF NOT EXISTS (SELECT * FROM RODM_isa WHERE cid = @cid AND base_cid = @base_cid)
    INSERT RODM_isa SELECT @cid, @base_cid
GO


---------------------------------------------------------------------------------------------------
-- asisp_make_RODM_isa_chain
if exists (select * from sysobjects where name='asisp_make_RODM_isa_chain' and type='P')
  drop procedure asisp_make_RODM_isa_chain
go

create procedure asisp_make_RODM_isa_chain
  @verbose tinyint = 0
as
  truncate table RODM_isa_chain

  -- Everything is itself (sophistry I know!)
  insert RODM_isa_chain select cid, 0, cid, cid from RODMClassMap

  set nocount on
  declare @cid ClassID, @generation smallint

  declare cc insensitive cursor for select distinct cid from RODM_isa
  open cc
  fetch next from cc into @cid
  while @@fetch_status <> -1
  begin
    select @generation = 1
    insert RODM_isa_chain 
      select distinct @cid, @generation, cid, base_cid
      from RODM_isa
      where cid = @cid

    while @@rowcount > 0
    begin
      if @verbose = 1
        select cid=@cid, generation=@generation
      select * from RODM_isa_chain
      select @generation = @generation + 1
      insert RODM_isa_chain
        select distinct @cid, @generation, isa.cid, isa.base_cid
        from RODM_isa_chain ic, RODM_isa isa
        where
          ic.cid = @cid
          and generation = @generation - 1 
          and ic.base_cid = isa.cid
    end

    fetch next from cc into @cid
  end
  deallocate cc
go


---------------------------------------------------------------------------------------------------
-- _DiscoverRODMObjects
PRINT 'Creating procedure _DiscoverRODMObjects'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_DiscoverRODMObjects' AND type = 'P')
  DROP PROCEDURE _DiscoverRODMObjects
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMLoad.sqv  $
-- $Revision:   1.12  $
-- $Date:   Feb 04 2001 12:46:18  $
-- $Author:   BRENT  $
-- Procedure to perform delta discovery of RODM managed objects.
-- This procedure is called after the RODM database files have been downloaded.
-- Source tables:
--   RODMObject
--   RODMLink
--   RODMObjValScalar
--   RODMObjValIndexList and RODMObjValSelfDefining are not used
CREATE PROCEDURE _DiscoverRODMObjects
  @verbose TINYINT = 0
AS
  DECLARE @batchid INT
  SELECT @batchid = batchid 
  FROM DiscoveryBatch AS b JOIN DiscoveryBatchPStatVal AS v ON v.pstat = b.pstat
  WHERE
    b.FormatType = 5
    AND b.ActionType = 0
    AND v.name = 'LOADED'
  IF @batchid IS NULL
  BEGIN
    PRINT 'There are no loaded RODM discovery batches waiting to be processed'
    RETURN
  END

  SET NOCOUNT ON

  IF @verbose = 1
    PRINT '_DiscoverRODMObjects: Processing batch ' + CAST(@batchid AS VARCHAR)

  -- Update the RODMManagedObject/RODMManagedLink tables. 
  -- This assumes a single-dataserver configuration. For distributed configurations we want
  -- to do one of the following:
  --  a) use a pull subscription to get these tables from the RODM database on the central server.
  --  b) define a linked server that is the central server that has the RODM database and
  --     reference RODMManagedLink and RODMManagedObject through this server.
  IF @verbose = 1
    PRINT 'Deleting RODMLoad..RODMManagedLink'
  DELETE RODMManagedLink
  IF @@ERROR <> 0
  BEGIN
    PRINT 'Error deleting RODMLoad..RODMManagedLink'
    IF @verbose = 1
      SELECT 'RODMManagedLink AFTER DELETE' AS 'RODMManagedLink AFTER DELETE', *
      FROM RODMManagedLink
  END
  ELSE IF @verbose = 1
    PRINT 'Successfully deleted from RODMLoad..RODMManagedLink'

  IF @verbose = 1
    PRINT 'Deleting RODMLoad..RODMManagedObject'
  DELETE RODMManagedObject
  IF @@ERROR <> 0
  BEGIN
    PRINT 'Error deleting RODMLoad..RODMManagedObject'
  END

  SET IDENTITY_INSERT RODMManagedObject ON
  INSERT RODMManagedObject(cid,id,name,revision,nchild,description,ctime,mtime,userid,deleted,_AlertStateID,_PriorityID,_CurrentStateID,_DesiredStateID,_HasOwnershipNote,_ChildEventMatrixID,_ExceptionMatrixID,DiscoverAction,_RODMClass,_RODMMyName,_NetworkID,_RegistrationStatus,_DisplayResourceName,_DisplayStatus,_operationalState,_usageState,_administrativeState,_availabilityStatus,_proceduralStatus,_unknownStatus,_nativeStatus)
    SELECT cid,id,name,revision,nchild,description,ctime,mtime,userid,deleted,_AlertStateID,_PriorityID,_CurrentStateID,_DesiredStateID,_HasOwnershipNote,_ChildEventMatrixID,_ExceptionMatrixID,DiscoverAction,_RODMClass,_RODMMyName,_NetworkID,_RegistrationStatus,_DisplayResourceName,_DisplayStatus,_operationalState,_usageState,_administrativeState,_availabilityStatus,_proceduralStatus,_unknownStatus,_nativeStatus
    FROM RODM..RODMManagedObject
  SET IDENTITY_INSERT RODMManagedObject OFF
  INSERT RODMManagedLink SELECT * FROM RODM..RODMManagedLink

  DECLARE @err INT, @rc INT

  -- Mark all rows from the existing tables to be subject to removal
  UPDATE RODMManagedObject SET DiscoverAction = 'DELETE'
  UPDATE RODMManagedLink SET DiscoverAction = 'DELETE'

  -- Replace internal class identifiers with readable names
  UPDATE ro SET Class = rcm.Class FROM RODMObject AS ro
    JOIN RODMClassMap AS rcm ON rcm.RODMClass = ro.Class

  -- Replace the internal class ids with their names in RODMLink
  UPDATE rl SET SrcClass = map.Class FROM RODMLink AS rl 
    JOIN RODMClassMap AS map ON map.RODMClass = rl.SrcClass
  UPDATE rl SET DstClass = map.Class from RODMLink AS rl
    JOIN RODMClassMap AS map ON map.RODMClass = rl.DstClass

  -- Replace the internal class ids with their names in the ObjValScalar table
  UPDATE rv SET Class = map.Class FROM RODMObjValScalar AS rv
    JOIN RODMClassMap AS map ON map.RODMClass = rv.Class


  -- Delete the loaded objects where the classes are in the RODMUnmanagedClass table
  DELETE ro FROM RODMObject AS ro 
    JOIN RODMUnmanagedClass AS ruc ON ruc.Class = ro.Class

  -- Delete source links for unmanaged classes
  DELETE rl FROM RODMLink AS rl 
    JOIN RODMUnmanagedClass AS ruc ON ruc.Class = rl.SrcClass

  -- Delete destination links for unmanaged classes
  DELETE rl FROM RODMLink AS rl 
    JOIN RODMUnmanagedClass AS ruc ON ruc.Class = rl.DstClass

  -- Delete the ObjValScalar rows for unmanaged classes
  DELETE rv FROM RODMObjValScalar AS rv 
    JOIN RODMUnmanagedClass AS ruc ON ruc.Class = rv.Class


  -- Replace the internal fields ids with their names in RODMLink for
  -- source fields
  UPDATE rl SET SrcField = map.Field
  FROM RODMLink AS rl 
    JOIN RODMObject AS ro ON ro.Class = rl.SrcClass AND ro.MyName = rl.SrcMyName
    JOIN RODMFieldMap AS map ON map.Class = ro.Class AND rl.SrcField = map.RODMField
  WHERE
    rl.SrcField <> map.Field

  -- Replace the internal fields ids with their names in RODMLink for
  -- destination fields
  UPDATE rl SET DstField = map.Field
  FROM RODMLink AS rl 
    JOIN RODMObject AS ro ON ro.Class = rl.DstClass AND ro.MyName = rl.DstMyName
    JOIN RODMFieldMap AS map ON map.Class = ro.Class AND rl.DstField = map.RODMField
  WHERE
    rl.DstField <> map.Field

  -- Replace the internal fields with their names in the ObjValScalar table
  UPDATE rv SET Field = map.Field
  FROM RODMObjValScalar AS rv
    JOIN RODMFieldMap AS map ON map.Class = rv.Class and map.RODMField = rv.Field
  WHERE
    rv.Field <> map.Field

/*
  jcp: ObjValIndexList table is not used

  update v set v.Field = def.Field
  from RODMObjValIndexList as v
    join RODMObject as o on o.MyName = v.MyName
    join RODMFieldMap as def on def.Class = o.Class and v.Field = def.RODMField
  where
    v.Field <> def.Field
*/

/*
  jcp: ObjValSelfDefining table is not used

  update v set v.Field = def.Field
  from RODMObjValSelfDefining as v
    join RODMObject as o on o.MyName = v.MyName
    join RODMFieldMap as def on def.Class = o.Class and v.Field = def.RODMField
  where
    v.Field <> def.Field
*/

  --------------------------------
  -- process object information --
  --------------------------------

  -- We'll update the timestamp of previously found RODMManagedObject records

    -- the DisplayResourceName is always updated but it happens later on

  UPDATE rmo SET mtime=getdate(), DiscoverAction='UPDATE'
    from RODMManagedObject as rmo
    join RODMObject as ro on ro.MyName = rmo._RODMMyName and ro.Class = rmo._RODMClass

  -- We'll create new RODMManagedObject records based on newly loaded RODMObject table

    -- MyName appear in a form with one or more variable/value pairs.  An example is:
    -- "1.3.18.0.2.4.6=ADPPRU.USPRU001;1.3.18.0.0.2082=50.1.196".
    -- The "1.3.18.0.2.4.6=" pair identifies the name of the network so this objects network name is "ADPPRU.USPRU001"

  INSERT RODMManagedObject(DiscoverAction, _RODMClass, _RODMMyName, _NetworkID) 
    SELECT 'ADD', Class, new_obj.MyName,
      CASE 
        WHEN MyName LIKE '1.3.18.0.2.4.6=_%;%' THEN
          SUBSTRING(MyName, 16, CHARINDEX(';', MyName)-16) -- this expression extracts the network name
        ELSE
          ''
      END
    FROM RODMObject AS new_obj
    WHERE NOT EXISTS
    (
      SELECT * FROM RODMManagedObject AS old_obj 
      WHERE old_obj._RODMMyName = new_obj.MyName AND old_obj._RODMClass = new_obj.Class
    )

  -- Determine TBSM model class ids for objects based on various criteria

  -- For classes that have no roles defined the mapping is a straightforward join
  UPDATE rmo SET
    cid = map.cid
  FROM RODMManagedObject AS rmo 
    JOIN RODMClassMap AS map on map.Class = rmo._RODMClass 
  WHERE rmo.cid = '' AND map.Role = ''

  -- Lookup classes based on links, as specified in the RODMClassLinkMap table

  -- Source links
  DECLARE clmc CURSOR FOR 
    SELECT SrcClass, SrcField, DstClass, DstField, cid, direction
    FROM RODMClassLinkMap AS clm
    ORDER BY priority DESC
  OPEN clmc
  DECLARE @SrcClass VARCHAR(60), @SrcField VARCHAR(40), @DstClass VARCHAR(60), @DstField VARCHAR(40), @cid ClassID, @direction SMALLINT
  FETCH NEXT FROM clmc INTO @SrcClass, @SrcField, @DstClass, @DstField, @cid, @direction
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    IF @verbose = 1
      PRINT 'Processing RODMClassLinkMap row for ' + @cid + ' <- ' + @SrcClass + '.' + @SrcField + CASE @direction WHEN 1 THEN ' -> ' ELSE ' <- ' END + @DstClass + '.' + @DstField
    IF @direction = -1 -- reverse the lookup fields
    BEGIN
      DECLARE @tmp VARCHAR(60)
      SET @tmp = @SrcClass
      SET @SrcClass = @DstClass
      SET @DstClass = @tmp
      SET @tmp = @SrcField
      SET @SrcField = @DstField
      SET @DstField = @tmp
    END
    UPDATE rmo SET
      cid = @cid
    FROM
      RODMManagedObject AS rmo
      JOIN RODMLink AS rl ON rl.SrcClass = rmo._RODMClass AND rl.SrcMyName = rmo._RODMMyName 
    WHERE
      rmo.cid = ''
      AND rl.SrcClass = @SrcClass
      AND rl.SrcField = @SrcField
      AND rl.DstClass = @DstClass
      AND rl.DstField = @DstField
    FETCH NEXT FROM clmc INTO @SrcClass, @SrcField, @DstClass, @DstField, @cid, @direction
  END
  DEALLOCATE clmc

  -- Update the class ids by using the RODMClassIdLookup table to perform
  -- dynamic update queries. This is the most flexible but most complicated
  -- method for resolving the class ids and should only be used when 
  -- the logic for determining the ids is not straightforward, as in the
  -- case where multiple links must be followed.
  DECLARE cc CURSOR FOR
    SELECT id, cid_assignment_expr, where_clause, description FROM RODMClassIdLookup WHERE enabled = 1 ORDER BY priority DESC
  OPEN cc
  DECLARE @row_id INT, @cid_assignment_expr VARCHAR(80), @where_clause VARCHAR(1000), @description VARCHAR(255)
  FETCH NEXT FROM cc INTO @row_id, @cid_assignment_expr, @where_clause, @description
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    IF @verbose = 1
      PRINT 'Processing RODMClassIdLookup row ' + CAST(@row_id AS VARCHAR) + ': ' + ISNULL(@description, 'NO DESCRIPTION')
    EXEC("
    UPDATE rmo SET cid = " + @cid_assignment_expr + "
    FROM RODMManagedObject AS rmo " + @where_clause
    )
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      PRINT 'ERROR PROCESSING RODMClassIdLookup ROW ' + CAST(@row_id AS VARCHAR)
      PRINT '----------------------------------------------------------------------------------------'
      PRINT "
    UPDATE rmo SET cid = " + @cid_assignment_expr + "
    FROM RODMManagedObject AS rmo " + @where_clause
      PRINT '----------------------------------------------------------------------------------------'
    END
    IF @verbose = 1
      PRINT '  #rows=' + CAST(@rc AS VARCHAR)
    UPDATE RODMClassIdLookup SET num_matches = @rc WHERE CURRENT OF cc
    FETCH NEXT FROM cc INTO @row_id, @cid_assignment_expr, @where_clause, @description
  END
  DEALLOCATE cc

  -- Update names according to best fields as specified in the RODMNameField table
  DECLARE nc CURSOR FOR
    SELECT cid, Field FROM RODMNameField ORDER BY priority DESC
  OPEN nc
  DECLARE @Field VARCHAR(40)
  FETCH NEXT FROM nc INTO @cid, @Field
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    IF @verbose = 1
      PRINT 'Updating names for cid ' + @cid + ', Field ' + @Field
    -- Update all rows even when not actually overwriting the name column so that we get a
    -- count of how many rows matched this rule, which we use to update the num_matches column
    -- of the current RODMNameField row
    UPDATE rmo SET name = CASE rmo.name WHEN '' THEN rv.TextVal ELSE rmo.name END
    FROM
      RODMManagedObject AS rmo
      JOIN RODM_isa_chain AS ic ON ic.cid = rmo.cid
      JOIN RODMObjValScalar AS rv ON rv.Class = rmo._RODMClass AND rv.MyName = rmo._RODMMyName
    WHERE
      ic.base_cid = @cid AND rv.Field = @Field
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose = 1
      PRINT '  #rows=' + CAST(@rc AS VARCHAR)
    -- Update RODMNameField.num_matches to the number of rows just affected
    UPDATE RODMNameField SET num_matches = @rc WHERE CURRENT OF nc
    FETCH NEXT FROM nc INTO @cid, @Field
  END
  DEALLOCATE nc

  -- Special case for DisplayResourceName
  UPDATE rmo SET
    _DisplayResourceName = rv.TextVal
  FROM
    RODMManagedObject AS rmo
    JOIN RODMObjValScalar AS rv ON rv.Class = rmo._RODMClass AND rv.MyName = rmo._RODMMyName 
      AND rv.Field = 'DisplayResourceName'

  ------------------------------
  -- process link information --
  ------------------------------

  -- We'll update the mtime column on any previously existing links

  UPDATE rml SET mtime=getdate(), DiscoverAction='UPDATE' 
  FROM RODMManagedLink AS rml
    JOIN RODMLink AS rl ON 
      rl.SrcClass = rml.SrcClass
      AND rl.SrcMyName = rml.SrcMyName
      AND rl.SrcField = rml.SrcField
      AND rl.DstClass = rml.DstClass
      AND rl.DstMyName = rml.DstMyName
      AND rl.DstField = rml.DstField

  IF @verbose = 1
    PRINT 'Inserting new links based on RODMFieldLinkMap'
  INSERT RODMManagedLink(DiscoverAction, link_type, SrcClass, SrcMyName, DstClass, DstMyName, SrcField, DstField, 
                         src_cid, src_id, dst_cid, dst_id)
    SELECT 
      'ADD',
      flm.link_type,
      rl.SrcClass,
      rl.SrcMyName,
      rl.DstClass,
      rl.DstMyName,
      rl.SrcField,
      rl.DstField,
      srco.cid,
      srco.id,
      dsto.cid,
      dsto.id
    FROM RODMLink AS rl
    JOIN RODMFieldLinkMap AS flm ON 
      flm.SrcClass = rl.SrcClass 
      AND flm.SrcField = rl.SrcField 
      AND flm.DstClass = rl.DstClass 
      AND flm.DstField = rl.DstField
    JOIN RODMManagedObject AS srco ON srco._RODMClass = rl.SrcClass AND srco._RODMMyName = rl.SrcMyName
    JOIN RODMManagedObject AS dsto ON dsto._RODMClass = rl.DstClass AND dsto._RODMMyName = rl.DstMyName
    AND NOT EXISTS
    (
      SELECT * FROM RODMManagedLink AS prev
      WHERE
        prev.link_type = flm.link_type
        AND prev.SrcClass = rl.SrcClass
        AND prev.SrcMyName = rl.SrcMyName
        AND prev.DstClass = rl.DstClass
        AND prev.DstMyName = rl.DstMyName
    )
  IF @@ERROR <> 0
  BEGIN
    RAISERROR('_DiscoverRODMObjects: Error adding links based on RODMFieldLinkMap', 3, 1)
  END

  IF @verbose = 1
    PRINT 'Updating previous links based on RODMFieldLinkMap'
  UPDATE rml SET
    src_cid = srco.cid,
    src_id = srco.id,
    dst_cid = dsto.cid,
    dst_id = dsto.id,
    DiscoverAction = 'UPDATE'
  FROM
    RODMManagedLink AS rml
    JOIN RODMLink AS rl ON 
      rl.SrcClass = rml.SrcClass
      AND rl.SrcMyName = rml.SrcMyName
      AND rl.SrcField = rml.SrcField
      AND rl.DstMyName = rml.DstMyName
      AND rl.DstField = rml.DstField
    JOIN RODMFieldLinkMap AS flm ON 
      flm.SrcClass = rl.SrcClass 
      AND flm.SrcField = rl.SrcField 
      AND flm.DstClass = rl.DstClass 
      AND flm.DstField = rl.DstField
    JOIN RODMManagedObject AS srco ON srco._RODMClass = rl.SrcClass AND srco._RODMMyName = rl.SrcMyName
    JOIN RODMManagedObject AS dsto ON dsto._RODMClass = rl.DstClass AND dsto._RODMMyName = rl.DstMyName
    AND
    (
      rml.src_cid <> srco.cid
      OR
      rml.src_id <> srco.id
      OR
      rml.dst_cid <> dsto.cid
      OR
      rml.dst_id <> dsto.id
    )
  IF @@ERROR <> 0
  BEGIN
    RAISERROR('_DiscoverRODMObjects: Error updating links based on RODMFieldLinkMap', 3, 1)
  END

  IF @verbose = 1
    PRINT 'Inserting new links based on RODMLinkLookup table'
  DECLARE cc CURSOR FOR
    SELECT id, link_type, src_cid, dst_cid, where_clause, @description
    FROM RODMLinkLookup
    WHERE enabled = 1 ORDER BY priority DESC
  OPEN cc
  DECLARE @link_type VARCHAR(4), @src_cid VARCHAR(4), @dst_cid VARCHAR(4)
  FETCH NEXT FROM cc INTO @row_id, @link_type, @src_cid, @dst_cid, @where_clause, @description
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    IF @verbose = 1
      PRINT 'Processing RODMLinkLookup row ' + CAST(@row_id AS VARCHAR) + ': ' + ISNULL(@description, 'NO DESCRIPTION')
    EXEC("
    INSERT RODMManagedLink
    (
      DiscoverAction,
      link_type,
      SrcClass,
      SrcMyName,
      DstClass,
      DstMyName,
      SrcField,
      DstField, 
      src_cid,
      src_id,
      dst_cid,
      dst_id
    )
    SELECT 
      'ADD',
      '" + @link_type + "',
      src._RODMClass,
      src._RODMMyName,
      dst._RODMClass,
      dst._RODMMyName,
      '',
      '',
      src.cid,
      src.id,
      dst.cid,
      dst.id
    FROM
      RODMManagedObject AS src,
      RODMManagedObject AS dst
  " + @where_clause + "
      AND src.cid = '" + @src_cid + "'
      AND dst.cid = '" + @dst_cid + "'
      AND NOT EXISTS
      (
        SELECT * FROM RODMManagedLink AS prev
        WHERE
          prev.link_type = '" + @link_type + "'
          AND prev.SrcClass = src._RODMClass
          AND prev.SrcMyName = src._RODMMyName
          AND prev.DstClass = dst._RODMClass
          AND prev.DstMyName = dst._RODMMyName
      )
    ")
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      PRINT 'ERROR PROCESSING RODMLinkLookup ROW ' + CAST(@row_id AS VARCHAR)
    END
    IF @verbose = 1
      PRINT '  #rows=' + CAST(@rc AS VARCHAR)
    UPDATE RODMLinkLookup SET num_matches = @rc WHERE CURRENT OF cc
    FETCH NEXT FROM cc INTO @row_id, @link_type, @src_cid, @dst_cid, @where_clause, @description
  END
  DEALLOCATE cc

  -- update the discovery batch table to indicate that the loaded batch was completed

  DECLARE @loaded_pstat int
  SELECT @loaded_pstat = pstat from DiscoveryBatchPStatVal WHERE name = 'LOADED'

  DECLARE @completed_pstat int
  SELECT @completed_pstat = pstat from DiscoveryBatchPStatVal WHERE name = 'COMPLETED'

  UPDATE DiscoveryBatch SET pstat = @completed_pstat
    WHERE pstat = @loaded_pstat
      AND FormatType = 5
      AND ActionType = 0

  -- If we are on a single-dataserver configuration then we should call the
  -- Object..RODM_object_discovery_backend procedure to make the newly discovered
  -- objects get added/updated in the Object database.
  EXEC Object..RODM_object_discovery_backend
    @verbose = @verbose
GO


---------------------------------------------------------------------------------------------------
-- _LoadRODMDiscoveryBatch
PRINT 'Creating procedure _LoadRODMDiscoveryBatch'
GO

if exists (select * from sysobjects where name = '_LoadRODMDiscoveryBatch' and type = 'P')
  drop procedure _LoadRODMDiscoveryBatch
go


-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMLoad.sqv  $
-- $Revision:   1.12  $
-- $Date:   Feb 04 2001 12:46:18  $
-- $Author:   BRENT  $
-- Loads a RODMDiscovery batch from file path for the specified batch. 
-- The file is a combination of files dumped from the RODM cache on the
-- mainframe.
CREATE PROCEDURE _LoadRODMDiscoveryBatch
  @cid          ClassID,
  @id           ObjID,
  @batchid      INT = NULL
AS
  IF @batchid IS NULL
  BEGIN
    SELECT @batchid = MIN(batchid)
    FROM DiscoveryBatch AS b JOIN DiscoveryBatchPStatVal AS v ON v.pstat = b.pstat
    WHERE
      b.FormatType = 5
      AND b.ActionType = 0
      AND v.name = 'ENQUEUED'
    IF @batchid IS NULL
    BEGIN
      PRINT 'No RODM DiscoveryBatch is in a ENQUEUED state, nothing to do!'
      RETURN 0
    END
    SELECT * FROM DiscoveryBatch AS b JOIN DiscoveryBatchPStatVal AS v ON v.pstat = b.pstat
             WHERE 
               b.FormatType = 5
               AND b.ActionType = 0
               AND v.name <> 'ENQUEUED'
               AND v.name <> 'COMPLETED'
    IF @@ROWCOUNT > 0
    BEGIN
      PRINT 'There are RODM discovery batches in states preventing a load from taking place'
      RETURN 0
    END
  END
  ELSE
    UPDATE DiscoveryBatch SET pstat = v.pstat
    FROM DiscoveryBatchPStatVal AS v
    WHERE batchid = @batchid AND v.name = 'ENQUEUED' AND FormatType = 5

  UPDATE DiscoveryBatch SET pstat = v.pstat
  FROM DiscoveryBatchPStatVal AS v
  WHERE batchid = @batchid AND v.name = 'INPROGRESS'

  -- Lookup file and codepage of the batch
  DECLARE @FilePath NVARCHAR(500), @CodePage VARCHAR(8)
  SELECT @FilePath = FilePath, @CodePage = CodePage FROM DiscoveryBatch WHERE batchid = @batchid
  IF @@ROWCOUNT = 0
  BEGIN
    RAISERROR('DiscoveryBatch id %d is not valid', 3, 1, @batchid)
    UPDATE DiscoveryBatch SET pstat = v.pstat
    FROM DiscoveryBatchPStatVal AS v
    WHERE batchid = @batchid AND v.name = 'LOAD_ERROR'
    RETURN -30000
  END

  -- parse the data into something more useful to us
  DECLARE @cmd nvarchar(1250)
  DECLARE @ret int
  DECLARE @err int
  DECLARE @FilePathParsedPrefix nvarchar(128)
  SELECT @FilePathParsedPrefix = @FilePath + '.RODMparsed.'
  SELECT @cmd = 'ParseRODM -o ' + @FilePathParsedPrefix + ' ' + @FilePath

  print 'Executing: ' + @cmd
  EXEC @ret = master..xp_cmdshell @cmd
  SELECT @err = @@ERROR
  IF @err <> 0 OR @ret <> 0
  BEGIN
    RAISERROR(N'Could not parse %s (OS error %d), aborting', 3, 1, @FilePath, @err)
    UPDATE DiscoveryBatch SET pstat = v.pstat, error = @err
    FROM DiscoveryBatchPStatVal AS v
    WHERE batchid = @batchid AND v.name = 'LOAD_ERROR'
    RETURN -30000
  END

  DECLARE @rc INT

  -- Perform bulk load of link data
  DELETE FROM RODMLink

  EXEC("
  BULK INSERT RODMLink FROM '" + @FilePathParsedPrefix + "Link.dat'
  WITH
  (
    BATCHSIZE = 1000,
    FIELDTERMINATOR='|'
  )
  ")
  SELECT @err = @@ERROR
  IF @err <> 0
  BEGIN
    RAISERROR('Error loading discovery batch file %sLink.dat', 3, 1, @FilePathParsedPrefix)
    UPDATE DiscoveryBatch SET pstat = v.pstat, error = @err
    FROM DiscoveryBatchPStatVal AS v
    WHERE batchid = @batchid AND v.name = 'LOAD_ERROR'
    RETURN -30000
  END
  ELSE
    PRINT 'Loaded file ' + @FilePathParsedPrefix + 'Link.dat'

  -- Perform bulk load of attribute data
  DELETE FROM RODMObjValScalar

  EXEC("
  BULK INSERT RODMObjValScalar FROM '" + @FilePathParsedPrefix + "ObjValScalar.dat'
  WITH
  (
    BATCHSIZE = 1000,
    FIELDTERMINATOR='|'
  )
  ")
  SELECT @err = @@ERROR
  IF @err <> 0
  BEGIN
    RAISERROR('Error loading discovery batch file %sObjValScalar.dat', 3, 1, @FilePathParsedPrefix)
    UPDATE DiscoveryBatch SET pstat = v.pstat, error = @err
    FROM DiscoveryBatchPStatVal AS v
    WHERE batchid = @batchid AND v.name = 'LOAD_ERROR'
    RETURN -30000
  END
  ELSE
    PRINT 'Loaded file ' + @FilePathParsedPrefix + 'ObjValScalar.dat'

  -- Perform bulk load of Object data
  DELETE FROM RODMObject

  EXEC("
  BULK INSERT RODMObject FROM '" + @FilePathParsedPrefix + "Object.dat'
  WITH
  (
    BATCHSIZE = 1000,
    FIELDTERMINATOR='|'
  )
  ")
  SELECT @err = @@ERROR
  IF @err <> 0
  BEGIN
    RAISERROR('Error loading discovery batch file %sObject.dat', 3, 1, @FilePathParsedPrefix)
    UPDATE DiscoveryBatch SET pstat = v.pstat, error = @err
    FROM DiscoveryBatchPStatVal AS v
    WHERE batchid = @batchid AND v.name = 'LOAD_ERROR'
    RETURN -30000
  END
  ELSE
    PRINT 'Loaded file ' + @FilePathParsedPrefix + 'Object.dat'

  UPDATE DiscoveryBatch SET pstat = v.pstat, numrows = @rc
  FROM DiscoveryBatchPStatVal AS v
  WHERE batchid = @batchid AND v.name = 'LOADED'

  SELECT @cmd = 'DEL ' + @FilePathParsedPrefix + '*'
  EXEC @ret = master..xp_cmdshell @cmd
GO


---------------------------------------------------------------------------------------------------
-- _CreateRODMClassLinkMapEntry
PRINT 'Creating procedure _CreateRODMClassLinkMapEntry'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_CreateRODMClassLinkMapEntry' AND type = 'P')
  DROP PROCEDURE _CreateRODMClassLinkMapEntry
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMLoad.sqv  $
-- $Revision:   1.12  $
-- $Date:   Feb 04 2001 12:46:18  $
-- $Author:   BRENT  $
-- Creates or updates a RODMClassLinkMap entry.
CREATE PROCEDURE _CreateRODMClassLinkMapEntry
  @cid        ClassID = NULL,
  @id         ObjID = NULL,
  @ClassId    ClassID,
  @Direction  SMALLINT,
  @SrcClass   VARCHAR(60),
  @SrcField   VARCHAR(40),
  @DstClass   VARCHAR(60),
  @DstField   VARCHAR(40)
AS
  SET NOCOUNT ON
  IF NOT EXISTS
  (
    SELECT * 
    FROM RODMClassLinkMap 
    WHERE SrcClass = @SrcClass AND SrcField = @SrcField AND DstClass = @DstClass AND DstField = @DstField
  )
    INSERT RODMClassLinkMap(cid, direction, SrcClass, SrcField, DstClass, DstField)
      SELECT @ClassId, @Direction, @SrcClass, @SrcField, @DstClass, @DstField
  ELSE
    UPDATE RODMClassLinkMap SET
      cid = @ClassId,
      direction = @Direction
    WHERE SrcClass = @SrcClass AND SrcField = @SrcField AND DstClass = @DstClass AND DstField = @DstField
      AND (cid <> @ClassId OR direction <> @Direction)
GO


---------------------------------------------------------------------------------------------------
-- _CreateRODMFieldLinkMapEntry
PRINT 'Creating procedure _CreateRODMFieldLinkMapEntry'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_CreateRODMFieldLinkMapEntry' AND type = 'P')
  DROP PROCEDURE _CreateRODMFieldLinkMapEntry
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMLoad.sqv  $
-- $Revision:   1.12  $
-- $Date:   Feb 04 2001 12:46:18  $
-- $Author:   BRENT  $
-- Creates or updates a RODMFieldLinkMap entry.
CREATE PROCEDURE _CreateRODMFieldLinkMapEntry
  @cid        ClassID = NULL,
  @id         ObjID = NULL,
  @LinkType   VARCHAR(4),
  @Direction  SMALLINT,
  @SrcClass   VARCHAR(60),
  @SrcField   VARCHAR(40),
  @DstClass   VARCHAR(60),
  @DstField   VARCHAR(40)
AS
  SET NOCOUNT ON
  IF NOT EXISTS
  (
    SELECT * 
    FROM RODMFieldLinkMap 
    WHERE SrcClass = @SrcClass AND SrcField = @SrcField AND DstClass = @DstClass AND DstField = @DstField
  )
    INSERT RODMFieldLinkMap(link_type, direction, SrcClass, SrcField, DstClass, DstField)
      SELECT @LinkType, @Direction, @SrcClass, @SrcField, @DstClass, @DstField
  ELSE
    UPDATE RODMFieldLinkMap SET
      link_type = @LinkType,
      direction = @Direction
    WHERE SrcClass = @SrcClass AND SrcField = @SrcField AND DstClass = @DstClass AND DstField = @DstField
      AND (link_type <> @LinkType OR direction <> @Direction)
GO


---------------------------------------------------------------------------------------------------
-- _CreateRODMNameFieldEntry
PRINT 'Creating procedure _CreateRODMNameFieldEntry'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_CreateRODMNameFieldEntry' AND type = 'P')
  DROP PROCEDURE _CreateRODMNameFieldEntry
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMLoad.sqv  $
-- $Revision:   1.12  $
-- $Date:   Feb 04 2001 12:46:18  $
-- $Author:   BRENT  $
-- Creates or updates a RODMNameField entry.
CREATE PROCEDURE _CreateRODMNameFieldEntry
  @cid        ClassID = NULL,
  @id         ObjID = NULL,
  @ClassId    VARCHAR(60),
  @Field      VARCHAR(40),
  @Priority   INT = 0
AS
  SET NOCOUNT ON
  IF NOT EXISTS
  (
    SELECT * 
    FROM RODMNameField
    WHERE cid = @ClassId AND Field = @Field
  )
    INSERT RODMNameField (cid, Field, priority)
      SELECT @ClassId, @Field, @Priority
  ELSE
    UPDATE RODMNameField SET
      priority = @Priority
    WHERE cid = @ClassId AND Field = @Field AND priority <> @Priority
GO


---------------------------------------------------------------------------------------------------
-- _CreateRODMClassIdLookupEntry
PRINT 'Creating procedure _CreateRODMClassIdLookupEntry'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_CreateRODMClassIdLookupEntry' AND type = 'P')
  DROP PROCEDURE _CreateRODMClassIdLookupEntry
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMLoad.sqv  $
-- $Revision:   1.12  $
-- $Date:   Feb 04 2001 12:46:18  $
-- $Author:   BRENT  $
-- Creates or updates a RODMClassIdLookup entry.
CREATE PROCEDURE _CreateRODMClassIdLookupEntry
  @cid                 ClassID = NULL,
  @id                  ObjID = NULL,
  @cid_assignment_expr VARCHAR(80),
  @where_clause        VARCHAR(1000),
  @Priority            SMALLINT = 0,
  @Enabled             TINYINT = 1,
  @Description         VARCHAR(255) = NULL
AS
  DECLARE @rowid INT
  SELECT @rowid = id
  FROM RODMClassIdLookup
  WHERE
    cid_assignment_expr = @cid_assignment_expr
    AND where_clause = @where_clause
  IF @rowid IS NULL
    INSERT RODMClassIdLookup (cid_assignment_expr, where_clause, priority, enabled, description)
    SELECT @cid_assignment_expr, @where_clause, @Priority, @Enabled, @Description
  ELSE
    UPDATE RODMClassIdLookup SET
      priority = @Priority,
      enabled = @Enabled,
      description = @Description
    WHERE
      id = @rowid
      AND NOT
      (
        priority = @Priority
        AND enabled = @Enabled
        AND description = @Description
      )
GO


---------------------------------------------------------------------------------------------------
-- Create scheduled tasks for doing the discovery load and processing
USE msdb

-- Create the load job
DECLARE @job_name VARCHAR(100)
SET @job_name = 'RODM Discovery Load'
IF NOT EXISTS (SELECT * FROM sysjobs WHERE name = @job_name)
BEGIN
  PRINT 'Adding job ''' + @job_name + '''.'
  EXEC sp_add_job @job_name=@job_name, @enabled=1

  -- create a single step within the job
  EXEC sp_add_jobstep
    @job_name=@job_name,
    @step_name='Load',
    @command='_LoadRODMDiscoveryBatch ROOT, 0',
    @database_name='RODMLoad',
    @server=@@SERVERNAME

  -- associate the job with a server
  EXEC sp_add_jobserver
    @job_name=@job_name,
    @server_name=@@SERVERNAME

  -- set up the schedule for which to run the job
  EXEC sp_add_jobschedule
    @name='Daily 12am',
    @job_name=@job_name,
    @enabled=0,
    @freq_type=4,              -- run daily
    @freq_interval=1,          -- run every day
    @active_start_time=000000  -- execute at 3am
END


-- Create the processing job
SET @job_name = 'RODM Discovery Process'
IF NOT EXISTS (SELECT * FROM sysjobs WHERE name = @job_name)
BEGIN
  PRINT 'Adding job ''' + @job_name + '''.'
  EXEC sp_add_job @job_name=@job_name, @enabled=1

  -- create a single step within the job
  EXEC sp_add_jobstep
    @job_name=@job_name,
    @step_name='Process',
    @command='_DiscoverRODMObjects',
    @database_name='RODMLoad',
    @server=@@SERVERNAME

  -- associate the job with a server
  EXEC sp_add_jobserver
    @job_name=@job_name,
    @server_name=@@SERVERNAME

  -- set up the schedule for which to run the job
  EXEC sp_add_jobschedule
    @name='Daily 3am',
    @job_name=@job_name,
    @enabled=0,
    @freq_type=4,              -- run daily
    @freq_interval=1,          -- run every day
    @active_start_time=030000  -- execute at 3am
END

USE RODMLoad
