-- $Header:   //ARCHIVE/Archive/DB/sql/RODMSample.sqv   1.1   Feb 05 2001 18:27:22   BRENT  $
--
-- RODMSample.sql
--
-- This file is a series of steps to be run in order to demonstrate the functionality
-- of the TBSM RODM interace.

-- Step 0: If necessary, create the physical object hierarchy. Do this through the GUI
-- or use the SQL below. YOU ONLY HAVE TO DO THIS ONCE.
-- There must be a SNA/APPN Network called 'USIBMNT'. It may be located anywhere from 
-- an Enterprise down to an Operating System. The MonitoringOSID attribute must also
-- be set on the network. This can be done through the GUI once the network object
-- has been created by setting the 'Monitoring Operating System' field on the main
-- property page for the SNA/APPN Network.
USE Object
EXEC asisp_createENT  'Tivoli Systems', BUSC, 1
EXEC asisp_createCOMP 'RTP',            ENT,  1
EXEC asisp_createMACH 'IBM-1',          COMP, 1
EXEC asisp_createLPAR 'NMPIPL77',       MACH, 1
EXEC asisp_createOS   'NMPIPL77',       LPAR, 1
EXEC asisp_createSNNT 'USIBMNT',        ENT,  1
UPDATE SNANetwork_V SET _MonitoringOSID = 1 WHERE id = 1
GO


-- Step 1: Create the discovery batch using the sample data.
-- Modify the commands below as needed for your installation.
EXEC master..xp_cmdshell 'copy C:\Tivolimanager\SQL\SNA\USIBMNT.SAMPLE.TXT C:\Tivolimanager\Data\USIBMNT.TXT'
EXEC master..xp_cmdshell 'sh CreateDiscoveryBatch -F5 -A0 C:\Tivolimanager\Data\USIBMNT.TXT'
-- VERIFICATION:
-- 1. The commands completed without error
-- 2. The following SQL displays one record
SELECT v.name, b.* FROM DiscoveryBatch AS b JOIN DiscoveryBatchPStatVal AS v ON v.pstat = b.pstat
WHERE b.FormatType = 5 AND b.ActionType = 0 AND v.name = 'ENQUEUED'


-- Step 2: Test the loading of the discovery batch. Highlight and execute the commands below down to
-- the ROLLBACK statement. If this executes successfully then change the ROLLBACK to COMMIT and rerun,
-- then proceed to the verification.
USE RODMLoad
BEGIN TRAN xxx
SET NOCOUNT ON
EXEC _LoadRODMDiscoveryBatch ROOT, 0
SET NOCOUNT OFF
ROLLBACK TRAN xxx
-- VERIFICATION:
-- 1. The command batch completed without error
-- 2. From the output of the above step there should be a line that looks as follows:
--    Lines=140009 OPS=19053 Objects=335 Links=300 Values=1001
-- 3. Run the query batch below. It should produce the following output:
--    Objects     Links       Values      
--    ----------- ----------- ----------- 
--    335         300         1001
SET NOCOUNT ON
DECLARE @batchid INT, @Objects INT, @Links INT, @Values INT
SELECT @batchid = MIN(batchid) FROM DiscoveryBatch WHERE FormatType = 5 AND ActionType = 0 AND pstat = 0
SELECT @Objects = COUNT(*) FROM RODMObject --WHERE batchid = @batchid
SELECT @Links =   COUNT(*) FROM RODMLink --WHERE batchid = @batchid
SELECT @Values =  COUNT(*) FROM RODMObjValScalar --WHERE batchid = @batchid
SELECT @Objects AS Objects, @Links AS Links, @Values AS 'Values'
SET NOCOUNT OFF
GO


-- Step 4: Process the loaded discovery batch. First highlight and execute the commands down to and
-- including the ROLLBACK statement. If this executes successfully then comment out the ROLLBACK
-- statement, highlighting the statements down the 'GO' and executing. In order to verify that 
-- notifications are received on the GUI, it should be opened to a view of the USIBMNT network
-- prior to running the command batch.
USE RODMLoad
BEGIN TRAN xxx
SET NOCOUNT ON
DECLARE @start DATETIME
SELECT @start = CURRENT_TIMESTAMP
EXEC _DiscoverRODMObjects @verbose = 1
ROLLBACK TRAN xxx
-- VERIFICATION:
-- 
SET NOCOUNT OFF
SELECT * FROM RODMManagedObject
SELECT * FROM RODMManagedLink
SELECT * FROM ObjectQueues..PendingNotification WHERE ctime > @start
COMMIT TRAN xxx
GO

-- Step 5: Define the GenerateRODMEvents stored procedure. Highlight down to the second GO below and execute.
USE Object
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'GenerateRODMEvents' AND type = 'P')
  DROP PROCEDURE GenerateRODMEvents
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMSample.sqv  $
-- $Revision:   1.1  $
-- $Date:   Feb 05 2001 18:27:22  $
-- $Author:   BRENT  $
-- Generates sample RODM field update events
CREATE PROCEDURE GenerateRODMEvents
  @Field    VARCHAR(30),  -- RODM field. Currently 'DisplayStatus' and 'states' are supported
  @Value    VARCHAR(255), -- Value for field
  @BaseCID  ClassID,      -- Any valid RODM-managed object cid. Specify either 'RDSO' or 'RDMO' for base class messages.
  @WhereClause  VARCHAR(255) = NULL  -- Optional where-clause to filter RODMMangedObject rows (not including the WHERE keyword)
AS
  IF @WhereClause IS NULL
    SET @WhereClause = ''
  ELSE 
    SET @WhereClause = 'AND (' + @WhereClause + ')'

  SET NOCOUNT ON
  EXEC("
  DECLARE statec CURSOR FOR
  SELECT 
   'EXEC Process_RODM_StatesMessage @SourceCID=''ROOT'', @SourceID=0, @FormatType=5, @ActionType=1,
        @DATE_TIME='''
          + CAST(DATEPART(YEAR, CURRENT_TIMESTAMP) AS VARCHAR(4))
          + REPLICATE('0', 2-DATALENGTH(CAST(DATEPART(MONTH, CURRENT_TIMESTAMP) AS VARCHAR(4)))) + CAST(DATEPART(MONTH, CURRENT_TIMESTAMP) AS VARCHAR(4))
          + REPLICATE('0', 2-DATALENGTH(CAST(DATEPART(DAY, CURRENT_TIMESTAMP) AS VARCHAR(4)))) + CAST(DATEPART(DAY, CURRENT_TIMESTAMP) AS VARCHAR(4))
          + REPLICATE('0', 2-DATALENGTH(CAST(DATEPART(HOUR, CURRENT_TIMESTAMP) AS VARCHAR(4)))) + CAST(DATEPART(HOUR, CURRENT_TIMESTAMP) AS VARCHAR(4))
          + ':'
          + REPLICATE('0', 2-DATALENGTH(CAST(DATEPART(MINUTE, CURRENT_TIMESTAMP) AS VARCHAR(4)))) + CAST(DATEPART(MINUTE, CURRENT_TIMESTAMP) AS VARCHAR(4))
          + ':'
          + REPLICATE('0', 2-DATALENGTH(CAST(DATEPART(SECOND, CURRENT_TIMESTAMP) AS VARCHAR(4)))) + CAST(DATEPART(SECOND, CURRENT_TIMESTAMP) AS VARCHAR(4))
          + '.000000'
        + ''', @SEQ_NO=''0'', @OBJ_TYPE_ID=''RESP'',
        @PARM_1=''' + cm.RODMClass + ''', @PARM_2=''' + rmo._RODMMyName + ''', 
        @PARM_3=''" + @Field + "'', @PARM_4=''" + @Value + "'''
  FROM RODMManagedObject AS rmo
    JOIN RODMClassMap AS cm ON cm.Class = rmo._RODMClass
    JOIN isa_chain AS ic ON ic.cid = rmo.cid
  WHERE 
    ic.base_cid = '" + @BaseCID + "' " + @WhereClause
  )
  OPEN statec
  DECLARE @sql VARCHAR(500)
  FETCH NEXT FROM statec INTO @sql
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    PRINT @sql
    EXEC(@sql)
    FETCH NEXT FROM statec INTO @sql
  END
  DEALLOCATE statec
  SET NOCOUNT OFF
GO

-- Step 6: Generate some sample events. Wait after executing each block to see the effect on the GUI

-- Set the 'states' field. This will cause the following three exceptions to occur on all RODM managed objects
-- that have the states fields (all have AlertState=Green, Priority=Ignore):
--    Administrative State unlocked
--    Native Status active
--    Usage State idle
EXEC GenerateRODMEvents @Field = 'states', @Value = '01000100000000', @BaseCID='RDSO', @WhereClause='_NetworkID=''USIBMNT'''

-- Set DisplayStatus to 130 (Unsatisfactory) for all RODM managed objects in network USIBMNT.
-- This should turn all objects red.
EXEC GenerateRODMEvents @Field = 'DisplayStatus', @Value = '130', @BaseCID='RDMO', @WhereClause='_NetworkID=''USIBMNT'''

-- Set DisplayStatus to 129 (Satisfactory) for all RODM managed objects in network USIBMNT
-- This should clear all red alerts.
EXEC GenerateRODMEvents @Field = 'DisplayStatus', @Value = '129', @BaseCID='RDMO', @WhereClause='_NetworkID=''USIBMNT'''
