# $Header:   //ARCHIVE/Archive/DB/SQL/SNA/LoadSNAIcons.ksv   1.3   Jan 02 2001 11:28:14   BRENT  $

# LoadSNAIcons.ksh

# Script to load the SNA class icons that are stored in the database from BMP files

Usage()
{
  print "USAGE:"
  print "  LoadSNAIcons.ksh options... "
  print "  The options are:"
  print "    -U<user>         User name for the database login"
  print "    -P<password>     Password for the database login"
  print "    -S<server>       Database server"
  print ""
  print "Replaces the currently loaded SNA icon definitions with the bitmaps"
  print "in the current directory."
}

while getopts U:S:P:t: c; do
  case $c in
  U)  user_opt="-U$OPTARG";;
  P)  password_opt="-P$OPTARG";;
  S)  server_opt="-S$OPTARG";;
  t)  subtype=$OPTARG;;
  *)  Usage; exit;;
  esac
done

if [ -z "$subtype" ]; then
  Usage >&2
  exit 1
fi

for IconName in $(echo *.bmp | sed 's/.bmp//g' | tr '[a-z]' '[A-Z]'); do
  isql -n $user_opt $password_opt $server_opt -dObject <<- END
    DECLARE @blob_id ObjID
    SELECT @blob_id = id FROM BLOB_C WHERE name = '$IconName' AND _Type = 'BMP' AND _SubType = '$subtype'
    IF @blob_id IS NULL
      EXEC @blob_id = asisp_createBLOB '$IconName', @Type='BMP', @SubType='$subtype'
    UPDATE BLOB_C set _Data = 0x0 where id = @blob_id
END

  textcopy $user_opt $password_opt $server_opt -DObject -TBLOB_C -C_Data -W"where name='$IconName' and _Type = 'BMP' and _SubType = '$subtype'" -F$IconName.bmp -I
done


isql -n $user_opt $password_opt $server_opt -dObject <<- END_OF_QUERY
  SET NOCOUNT ON
  DECLARE @colname sysname
  SELECT @colname = CASE '$subtype' 
    WHEN 'LARGE' THEN '_LargeIconName'
    WHEN 'SMALL' THEN '_SmallIconName'
    WHEN 'LARGE_DELETED' THEN '_LargeDeletedIconName'
    WHEN 'SMALL_DELETED' THEN '_SmallDeletedIconName'
  END
  DECLARE cc INSENSITIVE CURSOR FOR
    SELECT
      cm.cname,
      icr.IconName
    FROM
      RODMClassMap AS cm
      JOIN RODMDisplayResourceType AS drt ON drt.Class = cm.Class AND drt.Role = cm.Role
      JOIN RODMIconCrossRef AS icr ON icr.DisplayResourceType = drt.Value
  OPEN cc
  DECLARE @cname VARCHAR(24), @blob_name ObjName, @blob_id_str VARCHAR(9)
  FETCH NEXT FROM cc INTO @cname, @blob_name
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SELECT @blob_id_str = CAST(id AS VARCHAR) FROM BLOB_V WHERE name = @blob_name AND _Type = 'BMP' AND _SubType = '$subtype'
    PRINT "UPDATE " + @cname + "_S SET " + @colname + " = '" + @blob_id_str + "'" 
    EXEC("
    UPDATE " + @cname + "_S SET " + @colname + " = '" + @blob_id_str + "'"
    )
    FETCH NEXT FROM cc INTO @cname, @blob_name
  END
  DEALLOCATE cc
END_OF_QUERY
