# $Header:   //ARCHIVE/Archive/DB/sql/SNADBGen.ksv   1.3   Feb 04 2001 13:42:44   BRENT  $

# snadbgen.ksh

# Wrapper script for calling dbgen on SNANetwork-related classes

Usage()
{
  print "USAGE:"
  print "  snadbgen.ksh options... "
  print "  The options are:"
  print "    -U<user>         User name for the database login"
  print "    -P<password>     Password for the database login"
  print "    -S<server>       Database server"
  print ""
  print "Runs dbgen for SNANetwork classes"
}

password_opt=-P

while getopts U:S:P: c; do
  case $c in
  U)  user_opt="-U$OPTARG";;
  P)  password_opt="-P$OPTARG";;
  S)  server_opt="-S$OPTARG";;
  *)  Usage; exit;;
  esac
done

dbgen $user_opt $password_opt $server_opt -oSNANetwork -cSNNT,NNDM,NNDN,NNDC,IDCT,ENLT,NNLT,LUG,STGC,BRNN,APEN,APNN,APTG,ATGC,TGCN,NTRI,IDNC,XDR,XDRM,DG,ICND,LENN,LL,LU,MDH,NVPT,SNAN,T21N,T4N,T4NG,T5N,VRN

# Post-process the DDL generated by dbgen.

awk '/if not exists.*_C. and type = .U./ {
  if ($0 ~ /SNANetwork/) {
    print $0;
    continue;
  }
  getline;
  getline;
  getline;
  getline;
  view=$3;
  getline;
  getline;
  sub(".*cid=..","");
  sub("..[)]","");
  cid=$0;
  printf("----------------------------------------------------------------------------------------------------\n");
  printf("PRINT '\''Creating view %s'\''\n", view);
  printf("GO\n");
  printf("\n");
  printf("IF EXISTS (SELECT * FROM sysobjects WHERE name = '\''%s'\'' AND type = '\''V'\'')\n", view);
  printf("  DROP VIEW %s\n", view);
  printf("GO\n");
  printf("\n");
  printf("CREATE VIEW %s AS SELECT *, mtime AS _LatestDiscoveryDate FROM RODMManagedObject WHERE cid = '\''%s'\''\n", view, cid);
  printf("GO\n");
  printf("\n");
  while ($1 != "go")
    getline;
  getline;
}
/if not exists.*_ID. and type = .U./ { # skip creation of ID tables
  if ($0 ~ /SNNT_ID/) {
    print $0;
    continue;
  }
  while ($1 != "go")
    getline;
  getline;
}
{
  print $0;
}' SNANetwork_tables.sql > SNANetwork_tables2.sql

mv -f SNANetwork_tables2.sql SNANetwork_tables.sql

awk '/^-- alloc_/ {
  if ($0 ~ /SNNT/) {
    print $0;
    continue;
  }
  num_goes = 0;
  while (num_goes < 3) {
    getline;
    if ($1 == "go")
      num_goes++;
  }
  getline;
}
/^-- asisp_create/ {
  if ($0 ~ /SNNT/) {
    print $0;
    continue;
  }

  # Remove the id allocation block
  while ($0 !~ /alloc_/) {
    print $0;
    getline;
  }
  sub(".*alloc_", "", $0);
  cid=$0;
  while (NF > 0)
    getline;

  # Set flag indicating we are in the create procedure
  in_create_proc = 1;

  # Reset variable(s) used for tracking progress through create procedure
  nlink_inserts = 0;
}
(in_create_proc && $1=="id,") {
  # In the class table insert column list, replace the id column with the cid
  print "    cid,"
  getline;
}  
in_create_proc && $1=="@id," {
  # In the class table insert parameter list, replace @id with @cid
  printf("    '\''%s'\'',\n", cid);
  getline;
}
in_create_proc && $1=="@RODMClass," {
  # In the class table insert, replace the @RODMClass parameter with the appropriate value from RODMClassMap
  print "    cdm.Class,"
  getline;
}  
in_create_proc && $1=="@RODMMyName," {
  # In the class table insert, conditionally replace the @RODMMyName parameter with the object name
  print "    ISNULL(@RODMMyName, @name),"
  in_param_list=1;
  getline;
}
in_create_proc && $0 ~ /_PRMX_A[CE]PX[(]src_id, dst_id[)] select @id/ {
  # Save insert statement for childevent/exception matrix for later output
  link_insert[++nlink_inserts] = $0;
  getline;
}
in_create_proc && $0 ~ /if @@rowcount <> 1/ {
  # This is the error check immediately after the insert
  # We put the id lookup block after this check
  while (NF > 0) {
    print $0;
    getline;
  }

  printf("\n");
  printf("  -- Lookup id of object just created\n");
  printf("  SELECT @id = id FROM %s_C WHERE name = @name\n", cname);
  printf("  IF @id IS NULL\n");
  printf("  BEGIN\n");
  printf("    RAISERROR('\''asisp_create%s: Could not find %s with name = '\'''\''%%s'\'''\'''\'', 3, 1, @name)\n", cid, cname);
  printf("    ROLLBACK TRANSACTION asisp_create%s\n", cid);
  printf("    RETURN -30000\n");
  printf("  END\n");
  printf("\n");

  # Print the link insert statements now that the id has been resolved
  for (i = 1; i <= nlink_inserts; i++)
    printf("%s\n", link_insert[i]);
}
in_create_proc && $0 ~ /-- Insert class record/ {
  # Add lookup for network name
  print "  -- Lookup network name if not specified"
  print "  IF @NetworkID IS NULL"
  print "  BEGIN"
  print "    CREATE TABLE #links ( src_cid ClassID, src_id ObjID, gen INT )"
  print "    DECLARE @src_cid ClassID, @src_id ObjID, @gen INT, @rc INT"
  print "    SELECT @src_cid = @parent_cid, @src_id = @parent_id, @gen = 0"
  print "    INSERT #links SELECT @src_cid, @src_id, @gen"
  print "    SELECT @rc = @@ROWCOUNT"
  print "    WHILE @rc > 0"
  print "    BEGIN"
  print "      IF EXISTS (SELECT * FROM #links WHERE gen = @gen AND src_cid = '\''SNNT'\'')"
  print "      BEGIN"
  print "        SELECT @src_id = src_id FROM #links WHERE src_cid = '\''SNNT'\''"
  print "        SELECT @NetworkID = name FROM SNANetwork_C WHERE id = @src_id"
  print "        BREAK"
  print "      END"
  print "      SET @gen = @gen + 1"
  print "      INSERT #links"
  print "        SELECT l.src_cid, l.src_id, @gen"
  print "        FROM link AS l"
  print "          JOIN #links AS p ON p.src_cid = l.dst_cid AND p.src_id = l.dst_id"
  print "          JOIN link_type AS lt ON lt.link_type = l.link_type"
  print "        WHERE"
  print "          (lt.flag_parent_child = 1 OR lt.flag_containment = 1)"
  print "          AND p.gen = @gen - 1"
  print "      SELECT @rc = @@ROWCOUNT"
  print "      DELETE #links WHERE gen = @gen - 1"
  print "    END"
  print "  END"
  print ""
}
in_create_proc && $0 ~ /insert .*_C/ {
  # Extract the class name from the "insert <cname>_C" statement
  cname = $2;
  sub("_C", "", cname);
}
in_param_list && NF==0 {
  in_param_list=0;
  print "  from"
  print "    RODMClassMap AS cdm"
  print "  where"
  printf("    cdm.cid = '\''%s'\''\n", cid);
}
in_create_proc && $1=="go" {
  in_create_proc = 0;
}
{
  print $0;
}' SNANetwork_procs.sql > SNANetwork_procs2.sql

mv -f SNANetwork_procs2.sql SNANetwork_procs.sql


echo "
-- Create custom link insert triggers for objects physically contained under the SNANetwork class (cid=SNNT)
SET NOCOUNT ON
DECLARE linkc INSENSITIVE CURSOR FOR
  SELECT dol.src_cid, dol.dst_cid, dol.link_type
  FROM derived_obj_link AS dol
    JOIN obj_class AS soc ON soc.cid = dol.src_cid
    JOIN obj_class AS doc ON doc.cid = dol.dst_cid
  WHERE
    dol.src_cid = 'SNNT'
    AND dol.link_type = 'PHYC'
    AND soc.flag_abstract = 0
    AND doc.flag_abstract = 0
OPEN linkc
DECLARE @src_cid ClassID, @dst_cid ClassID, @link_type VARCHAR(4)
FETCH NEXT FROM linkc INTO @src_cid, @dst_cid, @link_type
WHILE @@FETCH_STATUS <> -1
BEGIN
  EXEC CreateNotifyLinkInsertTrigger @src_cid, @dst_cid, @link_type
  FETCH NEXT FROM linkc INTO @src_cid, @dst_cid, @link_type
END
DEALLOCATE linkc
SET NOCOUNT OFF
GO
" >> SNANetwork_triggers.sql
