-- $Header:   //ARCHIVE/Archive/DB/sql/StagedOPCEventChanges.sqv   1.1   Feb 09 2001 13:16:34   BRENT  $
--
-- StagedOPCEventChanges.sql
--
-- Changes the StagedOPCEvent.MESSAGE field from VARCHAR(16) to VARCHAR(255) in order to
-- accomodate custom events. Corresponding changes are also made to the event handler
-- procedures for exceptions and messages.

USE Object
GO

---------------------------------------------------------------------------------------------------
-- UpgradeStagedOPCEvent
PRINT 'Creating procedure UpgradeStagedOPCEvent'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'UpgradeStagedOPCEvent' AND type = 'P')
  DROP PROCEDURE UpgradeStagedOPCEvent
GO

CREATE PROCEDURE UpgradeStagedOPCEvent
AS
  IF (SELECT length FROM syscolumns WHERE id = object_id('StagedOPCEvent') AND name = 'MESSAGE') = 255
  BEGIN
    PRINT 'StagedOPCEvent.MESSAGE is already 255 characters wide, nothing done'
    RETURN
  END

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION UpgradeStagedOPCEvent
  ELSE
    SAVE TRANSACTION UpgradeStagedOPCEvent

  PRINT 'Dropping old StagedOPCEvent table'
  EXEC("
  DROP TABLE StagedOPCEvent
  ")
  IF @@ERROR <> 0
  BEGIN
    PRINT 'UpgradeStagedOPCEvent: could not drop table StagedOPCEvent'
    ROLLBACK TRANSACTION UpgradeStagedOPCEvent
    RETURN -30000
  END

  PRINT 'Defining new StagedOPCEvent table'
  EXEC("
  CREATE TABLE StagedOPCEvent
  (
    timestamp                             CONSTRAINT StagedOPCEvent_PK PRIMARY KEY (timestamp),
    ctime           DATETIME              CONSTRAINT StagedOPCEvent_ctime_DF DEFAULT GETDATE(),
    src_cid         ClassID,
    src_id          ObjID,
    TIME            DATETIME NULL,
    SEQ_NO          VARCHAR(50) NULL,
    MESSAGE         VARCHAR(255) NULL,
    MESSAGE_TEXT    VARCHAR(255) NULL,
    APPID           VARCHAR(16) NULL      CONSTRAINT StagedOPCEvent_APPID_DF DEFAULT NULL,
    WKSTNID         VARCHAR(8) NULL       CONSTRAINT StagedOPCEvent_WKSTNID_DF DEFAULT NULL,
    OPID            VARCHAR(10) NULL      CONSTRAINT StagedOPCEvent_OPID_DF DEFAULT NULL,
    JOBNAME         VARCHAR(8) NULL       CONSTRAINT StagedOPCEvent_JOBNAME_DF DEFAULT NULL,
    PRTY            INT NULL              CONSTRAINT StagedOPCEvent_PRTY_DF DEFAULT NULL,
    IA              VARCHAR(16) NULL      CONSTRAINT StagedOPCEvent_IA_DF DEFAULT NULL,
    EC              VARCHAR(8) NULL       CONSTRAINT StagedOPCEvent_EC_DF DEFAULT NULL,
    RESOURCE        VARCHAR(16) NULL      CONSTRAINT StagedOPCEvent_RESOURCE_DF DEFAULT NULL,
    WAIT_MINUTES    INT NULL              CONSTRAINT StagedOPCEvent_WAIT_MINUTES_DF DEFAULT NULL
  )
  ")
  IF @@ERROR <> 0
  BEGIN
    PRINT 'UpgradeStagedOPCEvent: could not create table StagedOPCEvent'
    ROLLBACK TRANSACTION UpgradeStagedOPCEvent
    RETURN -30000
  END

  IF @tcount = 0
    COMMIT TRANSACTION UpgradeStagedOPCEvent
GO

EXEC UpgradeStagedOPCEvent

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'UpgradeStagedOPCEvent' AND type = 'P')
  DROP PROCEDURE UpgradeStagedOPCEvent
GO

-- Change the OPC Application Id DATA_TYPE name to 'APPID' for consistency with OPC format type
UPDATE DATA_TYPE SET NAME = 'APPID' WHERE FORMAT_TYPE IN (7, 9, 10) AND DATA_TYPE = 10 AND NAME <> 'APPID'

-- Add the EventProcMap entries for OPC events
EXEC asisp_defineEventProcMapping SA390,      OPCEXCEPTION, Process_SA390_OPCException
EXEC asisp_defineEventProcMapping OPSMVS,     OPCEXCEPTION, Process_SA390_OPCException
EXEC asisp_defineEventProcMapping AFOPERATOR, OPCEXCEPTION, Process_SA390_OPCException

EXEC asisp_defineEventProcMapping SA390,      OPCMESSAGE, Process_SA390_OPCMessage
EXEC asisp_defineEventProcMapping OPSMVS,     OPCMESSAGE, Process_SA390_OPCMessage
EXEC asisp_defineEventProcMapping AFOPERATOR, OPCMESSAGE, Process_SA390_OPCMessage
GO
