-- $Header:   //ARCHIVE/Archive/DB/sql/UpgradeMessageDescription.sqv   1.2   Jan 17 2001 18:06:58   BRENT  $
--
-- UpgradeMessageDescription.sql
--
-- Upgrades the MessageDescription_A, MessageDescription_T tables so that the MsgID columns can
-- accomodate 255 characters.

USE Object
GO

---------------------------------------------------------------------------------------------------
-- UpgradeMessageDescription
PRINT 'Creating procedure UpgradeMessageDescription'

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'UpgradeMessageDescription' AND type = 'P')
  DROP PROCEDURE UpgradeMessageDescription
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/UpgradeMessageDescription.sqv  $
-- $Revision:   1.2  $
-- $Date:   Jan 17 2001 18:06:58  $
-- $Author:   BRENT  $
CREATE PROCEDURE UpgradeMessageDescription
AS
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN UpgradeMessageDescription
  ELSE
    SAVE TRAN UpgradeMessageDescription

  IF (SELECT length FROM syscolumns WHERE id = OBJECT_ID('MessageDescription_A') AND name = '_MsgID') <> 255
  BEGIN
    PRINT 'Altering MessageDescription_A._MsgID to have a length of 255 characters'
    EXEC("
    ALTER TABLE MessageDescription_A ALTER COLUMN _MsgID VARCHAR(255)
    ")
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN UpgradeMessageDescription
      PRINT 'Error altering MessageDescription_A._MsgID'
      RETURN -30000
    END
  END

  IF (SELECT length FROM syscolumns WHERE id = OBJECT_ID('MessageDescription_T') AND name = 'MsgID') <> 255
  BEGIN
    PRINT 'Saving current contents of MessageDescription_T'
    SELECT * INTO MessageDescription_T_SAVE FROM MessageDescription_T
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN UpgradeMessageDescription
      RETURN -30000
    END
  
    PRINT 'Dropping old MessageDescription_T table'
    EXEC("
    DROP TABLE MessageDescription_T
    ")
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN UpgradeMessageDescription
      RETURN -30000
    END

    PRINT 'Creating new MessageDescription_T table'
    EXEC("
    CREATE TABLE MessageDescription_T
    (
      obj_cid                       ClassID NOT NULL,
      obj_id                        ObjID NOT NULL,
      msgd_id                       ObjID NOT NULL,
      MsgID                         varchar(255) NOT NULL,
      State                         varchar(24) NOT NULL,
      AlertState                    varchar(12) NOT NULL,
      Priority                      varchar(12) NOT NULL,
      StateID                       int NOT NULL,
      AlertStateID                  int NOT NULL,
      PriorityID                    int NOT NULL,
      Flags                         int NOT NULL,
      SearchExpression              varchar(255) NOT NULL,
      ChildLookupPattern            varchar(255) NOT NULL,
      ProcessingPattern             varchar(255) NOT NULL,
      Description                   varchar(255) NOT NULL
    )
    ")
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN UpgradeMessageDescription
      RETURN -30000
    END

    PRINT 'Adding indices to MessageDescription_T'
    EXEC("
    ALTER TABLE MessageDescription_T ADD
      CONSTRAINT MSGD_T_PK PRIMARY KEY (obj_cid, obj_id, MsgID),
      CONSTRAINT MSGD_T_msgd_id_UQ UNIQUE (msgd_id)
    ")
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN UpgradeMessageDescription
      RETURN -30000
    END

    PRINT 'Restoring contents of MessageDescription_T'
    EXEC("
    INSERT MessageDescription_T SELECT * FROM MessageDescription_T_SAVE
    ")
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN UpgradeMessageDescription
      RETURN -30000
    END

    PRINT 'Dropping backup table MessageDescription_T_SAVE'
    EXEC("
    DROP TABLE MessageDescription_T_SAVE
    ")
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRAN UpgradeMessageDescription
      RETURN -30000
    END
  END

  IF @tcount = 0
    COMMIT TRAN UpgradeMessageDescription
GO

EXEC sp_dbcmptlevel Object, 70
EXEC UpgradeMessageDescription
EXEC sp_dbcmptlevel Object, 65
GO


---------------------------------------------------------------------------------------------------
-- MessageDescription_V
PRINT 'Creating view MessageDescription_V'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'MessageDescription_V' AND type = 'V')
  DROP VIEW MessageDescription_V
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/UpgradeMessageDescription.sqv  $
-- $Revision:   1.2  $
-- $Date:   Jan 17 2001 18:06:58  $
-- $Author:   BRENT  $
CREATE VIEW MessageDescription_V AS
  select o.id, name, revision, nchild, description, o.ctime, mtime, userid, deleted,
    _ParentCID,
    _ParentID,
    _AlertState='GENC/STCR-1/ALRS-' + convert(varchar(9), _AlertStateID) + '/T/Desc',
    _AlertStateID,
    _ChildLookupPattern,
    _Desc,
    _Flags,
    _MsgID,
    _Priority='GENC/STCR-1/OPRI-' + convert(varchar(9), _PriorityID) + '/T/Priority',
    _PriorityID,
    _ProcessingPattern,
    _SearchExpression,
    _SourceID,
    _State=rtrim(_ParentCID)+'/STCR-1/STTP-' + convert(varchar(9), _StateID) + '/T/State',
    _StateID
from
  MessageDescription_C o,
  MessageDescription_A a
where
  o.id = a.id
GO


---------------------------------------------------------------------------------------------------
-- MessageDescription
PRINT 'Creating view MessageDescription'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'MessageDescription' AND type = 'V')
  DROP VIEW MessageDescription
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/UpgradeMessageDescription.sqv  $
-- $Revision:   1.2  $
-- $Date:   Jan 17 2001 18:06:58  $
-- $Author:   BRENT  $
-- This view creates a more convenient way of looking at MessageDescription information for message processing
create view MessageDescription
as
  select
    obj_cid = msgd._ParentCID,
    obj_id  = msgd._ParentID,
    msgd_id = msgd.id,
    MsgID = _MsgID,
    State = convert(varchar(24), sttp._State),
    AlertState = case _AlertStateID
                   when 1 then 'Green' 
                   when 2 then 'Yellow' 
                   when 3 then 'Red' 
                   else 'Unknown' 
                 end,
    Priority = convert(varchar(12),pri6._Priority),
    StateID = msgd._StateID,
    AlertStateID = msgd._AlertStateID,
    PriorityID =msgd. _PriorityID,
    Flags = msgd._Flags,
    SearchExpression = msgd._SearchExpression,
    ChildLookupPattern =  msgd._ChildLookupPattern,
    ProcessingPattern = msgd._ProcessingPattern,
    Description = msgd._Desc
  from
     MessageDescription_V msgd,
     StateType_V sttp,
     SixLevelPri_V pri6
  where 
    msgd.deleted = 0
    and sttp.id = _StateID
    and pri6.id = _PriorityID
GO


---------------------------------------------------------------------------------------------------
-- asisp_defineMessageDescription
PRINT 'Creating procedure asisp_defineMessageDescription'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_defineMessageDescription' AND type = 'P')
  DROP PROCEDURE asisp_defineMessageDescription
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/UpgradeMessageDescription.sqv  $
-- $Revision:   1.2  $
-- $Date:   Jan 17 2001 18:06:58  $
-- $Author:   BRENT  $
-- Creates or updates a MessageDescription entry. A MessageDescription is uniquely identified by @cid, @id, and @MsgID.
CREATE PROCEDURE asisp_defineMessageDescription
  @cid                ClassID,              -- Class of object receiving this type of message
  @MsgID              VARCHAR(255),         -- Message identifier
  @Description        VARCHAR(255),         -- Description of message
  @State              VARCHAR(40),          -- State associated with message (e.g. - "Active", "Inactive")
  @AlertState         VARCHAR(40),          -- Alert level associated with message (e.g. - "Green", "Yellow", "Red")
  @Priority           VARCHAR(40),          -- Priority level associated with message (e.g. - "Critical", "High", "Medium", "Low", "Ignore", "Eliminate")
  @Flags              INT = 0,              -- Flags controlling handling of message. Current defined flags are:
                                            --   1 - MSGD_FLAG_NO_TRAP_REGISTER
                                            --   2 - MSGD_FLAG_WILDCARD
  @id                 ObjID = 0,            -- Optional instance id. If 0 (the default) then the MessageDescription
                                            -- is a class-level setting. Otherwise the MessageDescription is only
                                            -- for the specified instance.
  @SearchExpression   VARCHAR(255) = NULL,  -- Optional regular expression for performing additional processing.
                                            -- This expression is applied to the message text in order to extract
                                            -- elements from the text that can be used for specialized handling.
  @ChildLookupPattern VARCHAR(255) = NULL,  -- Optional regular expression specifying elements in the message
                                            -- text that identify a child object of the object receiving the message
                                            -- that is affected by the message.
  @ProcessingPattern  VARCHAR(255) = NULL   -- Optional string that specifies a template of SQL commands to execute
                                            -- using fields from the message and elements extracted from the 
                                            -- message text using the SearchExpression.
AS
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_defineMessageDescription
  ELSE
    SAVE TRANSACTION asisp_defineMessageDescription

  DECLARE @msgd_name ObjName, @msgd_id ObjID, @StateID INT, @AlertStateID INT, @PriorityID INT

  SELECT @StateID = id FROM StateType_V WHERE name = RTRIM(@cid) + '_' + @State
  IF @StateID IS NULL SELECT @StateID = 0
  SELECT @AlertStateID = id FROM AlertState_V WHERE name = @AlertState
  IF @AlertStateID IS NULL SELECT @AlertStateID = 0
  SELECT @PriorityID = id FROM SixLevelPri_V WHERE name = @Priority
  IF @PriorityID IS NULL SELECT @PriorityID = 0

  IF @SearchExpression IS NULL SELECT @SearchExpression = ''
  IF @ChildLookupPattern IS NULL SELECT @ChildLookupPattern = ''
  IF @ProcessingPattern IS NULL SELECT @ProcessingPattern = ''

  -- Lookup previous MessageDescription
  SELECT @msgd_name = RTRIM(@cid) + '_' + @MsgID
  IF @id <> 0 SELECT @msgd_name = @msgd_name + '_' + CAST(@id AS VARCHAR(9))
  SELECT @msgd_id = id from MessageDescription_V WHERE name = @msgd_name AND deleted = 0

  IF @msgd_id IS NULL
  BEGIN
    EXEC @msgd_id = asisp_createMSGD 
      @name = @msgd_name,
      @parent_cid = @cid,
      @parent_id = @id,
      @MsgID = @MsgID,
      @Desc = @Description,
      @StateID = @StateID,
      @AlertStateID = @AlertStateID,
      @PriorityID = @PriorityID,
      @Flags = @Flags,
      @SearchExpression = @SearchExpression,
      @ChildLookupPattern = @ChildLookupPattern,
      @ProcessingPattern = @ProcessingPattern
    IF @msgd_id < 0
    BEGIN
      ROLLBACK TRANSACTION asisp_defineMessageDescription
      RETURN @msgd_id
    END
  END
  ELSE
  BEGIN
    UPDATE msgd SET
      _ParentCID = @cid,
      _ParentID = @id,
      _MsgID = @MsgID,
      _Desc = @Description,
      _StateID = @StateID,
      _AlertStateID = @AlertStateID,
      _PriorityID = @PriorityID,
      _Flags = @Flags,
      _SearchExpression = @SearchExpression,
      _ChildLookupPattern = @ChildLookupPattern,
      _ProcessingPattern = @ProcessingPattern
    FROM MessageDescription_V AS msgd
    WHERE id = @msgd_id
    AND
    (
       _Desc <> @Description
       OR
       _StateID <> @StateID
       OR
       _AlertStateID <> @AlertStateID
       OR
       _PriorityID <> @PriorityID
       OR
       _Flags <> @Flags
       OR
      _SearchExpression <> @SearchExpression
       OR
      _ChildLookupPattern <> @ChildLookupPattern
       OR
      _ProcessingPattern <> @ProcessingPattern
    )
  END

  IF @tcount = 0
    COMMIT TRANSACTION asisp_defineMessageDescription

  RETURN @msgd_id
GO
