USE Object
GO

---------------------------------------------------------------------------------------------------
-- Add reason column to UnhandledEvent table
IF NOT EXISTS (SELECT * FROM syscolumns WHERE id = OBJECT_ID('UnhandledEvent') AND name = 'reason')
BEGIN
  PRINT 'Adding column ''reason'' to UnhandledEvent table'
  EXEC("
  ALTER TABLE UnhandledEvent ADD
    reason VARCHAR(255) NULL CONSTRAINT UnhandledEvent_reason_DF DEFAULT NULL
  ")
END


---------------------------------------------------------------------------------------------------
-- asisp_createUnhandledEvent
PRINT 'Creating procedure asisp_createUnhandledEvent'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_createUnhandledEvent' AND type = 'P')
  DROP PROCEDURE asisp_createUnhandledEvent
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/asisp_decodeDatFileRecord.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jan 15 2001 12:38:44  $
-- $Author:   BRENT  $
-- Fall-through procedure for recording unhandled events processed by the MVS Event Handler.
-- asisp_decodeDatFileRecord
CREATE PROCEDURE asisp_createUnhandledEvent
  @SourceCID          ClassID,
  @SourceID           ObjID,
  @FormatType         SMALLINT,
  @ActionType         SMALLINT,
  @FormattedEventData VARCHAR(1000) = NULL,
  @RawEventData       VARBINARY(1000) = NULL,
  @Reason             VARCHAR(255) = NULL
AS
  -- If reason was not specified then try to decode and execute the record dynamically
  IF @Reason IS NULL
  BEGIN
    EXEC asisp_decodeDatFileRecord @SourceCID, @SourceID,
      @record = @FormattedEventData,
      @reason = @Reason OUTPUT,
      @execute = 1
  END

  -- If reason field is not null then insert into UnhandledEvent table
  IF @Reason IS NOT NULL
    INSERT UnhandledEvent(src_cid, src_id, format_type, action_type, formatted_data, raw_data, reason)
      SELECT @SourceCID, @SourceID, @FormatType, @ActionType, @FormattedEventData, @RawEventData, @Reason
GO


---------------------------------------------------------------------------------------------------
-- asisp_decodeDatFileRecord
PRINT 'Creating procedure asisp_decodeDatFileRecord'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_decodeDatFileRecord' AND type = 'P')
  DROP PROCEDURE asisp_decodeDatFileRecord
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/asisp_decodeDatFileRecord.sqv  $
-- $Revision:   1.0  $
-- $Author:   BRENT  $
-- $Date:   Jan 15 2001 12:38:44  $
-- Decodes a dat-file record into an exec statement that is equivalent to what the
-- event handler would execute for the given record. Can optionally execute the 
-- statement. 
CREATE PROCEDURE asisp_decodeDatFileRecord
  @cid ClassID,
  @id ObjID,
  @record VARCHAR(2000), -- Dat file record (may or may not start with 9 character delta timestamp)
  @exec_statement VARCHAR(2000) = NULL OUTPUT, -- Decoded exec statement from record
  @reason VARCHAR(255) = NULL OUTPUT, -- Reason for decode failure 
  @execute TINYINT = 0,  -- If 1 then the exec statement is actually executed
  @verbose TINYINT = 0   -- If 1 then diagnostic statements are printed during execution
AS
  SET NOCOUNT ON
  DECLARE @i INT, @end INT, @format_type INT, @action_type INT, @data_type INT, @data VARCHAR(2000),
    @data_type_name ObjName

  -- Skip delta timestamp if present
  IF PATINDEX('[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]\%', SUBSTRING(@record, 1, 12)) = 1
    SELECT @record = SUBSTRING(@record, 10, DATALENGTH(@record)-9)

  SELECT @format_type = CAST(SUBSTRING(@record, 1, 2) AS INT)
  SELECT @action_type = CAST(SUBSTRING(@record, 4, 2) AS INT)

  IF @verbose = 1
    SELECT @format_type AS FORMAT_TYPE, @action_type AS ACTION_TYPE

  SELECT @exec_statement = 'EXEC ' + proc_name +
    ' @SourceCID=''' + @cid + '''' + 
    ', @SourceID=' + CAST(@id AS VARCHAR) +
    ', @FormatType=' + CAST(@format_type AS VARCHAR) +
    ', @ActionType=' + CAST(@action_type AS VARCHAR)
  FROM EventProcMap
  WHERE format_type = @format_type AND action_type = @action_type

  SET @reason = 'Unknown format/action type'
  IF @exec_statement IS NULL
  BEGIN
UnhandledEvent:
    SELECT @exec_statement = 'EXEC asisp_createUnhandledEvent' + 
      ' @SourceCID=''' + @cid + '''' + 
      ', @SourceID=' + CAST(@id AS VARCHAR) +
      ', @FormatType=' + CAST(@format_type AS VARCHAR) +
      ', @ActionType=' + CAST(@action_type AS VARCHAR) +
      ', @FormattedEventData=''' + @record + '''' +
      ' -- ' + @reason
    RETURN
  END

  SELECT @record = SUBSTRING(@record, 7, DATALENGTH(@record)-6)
  SET @i = 0
  WHILE SUBSTRING(@record, 1, 1) <> '~'
  BEGIN
    SELECT @data_type = CAST(SUBSTRING(@record, 1, 2) AS INT)
    SELECT @data_type_name = NULL
    SELECT @data_type_name = NAME FROM DATA_TYPE WHERE FORMAT_TYPE = @format_type AND DATA_TYPE = @data_type
    SELECT @record = SUBSTRING(@record, 3, DATALENGTH(@record)-2)
    SELECT @end = PATINDEX('%[~\]%', @record)
    SELECT @data = SUBSTRING(@record, 1, @end-1)
    SELECT @record = SUBSTRING(@record, @end+1, DATALENGTH(@record)-@end)
    IF @verbose = 1
      SELECT @i AS i, @data_type AS data_type, @data_type_name AS name, @data AS data
    IF @data_type_name IS NULL
    BEGIN
      SELECT @reason = 'Data type ' + CAST(@data_type AS VARCHAR) + ' undefined for format type'
      GOTO UnhandledEvent
    END
    SELECT @exec_statement = @exec_statement + ', @' + @data_type_name + '=''' + @data + ''''
    SET @i = @i + 1
  END
GO

DECLARE @exec_statement VARCHAR(2000)
EXEC asisp_decodeDatFileRecord OS, 6,
  @record = '00000027005\04\002001010312:41:03.197069\01IPO16\02RODM\731.3.18.0.0.1829\741.3.18.0.2.4.6=USIBMNT;1.3.18.0.0.2032=NTF7MVS;1.3.18.0.0.1984=USIBMNT.NTF7L703\75states\7601010100000001~',
  @exec_statement = @exec_statement OUTPUT
SELECT @exec_statement

/*
00003425905\04\002001010312:41:35.029886\01IPO17\02RODM\731.3.18.0.0.1829\741.3.18.0.2.4.6=USIBMNT;1.3.18.0.0.2032=NTF7MVS;1.3.18.0.0.1984=USIBMNT.NTF7L703\75states\7601000100000000~
00002871105\04\002001010312:42:06.336000\01IPO18\02RODM\731.3.18.0.0.1829\741.3.18.0.2.4.6=USIBMNT;1.3.18.0.0.2032=NTF7MVS;1.3.18.0.0.1984=USIBMNT.NTF7L703\75states\7600000140000002~
00001276905\04\002001010312:42:19.123387\01IPO19\02RODM\731.3.18.0.0.1829\741.3.18.0.2.4.6=USIBMNT;1.3.18.0.0.2032=NTF7MVS;1.3.18.0.0.1984=USIBMNT.NTF7L703\75states\7601000100000000~
*/
