-- $Header:   //ARCHIVE/Archive/DB/sql/asisp_defineExceptionType.sqv   1.1   Jan 17 2001 15:24:46   BRENT  $
--
-- asisp_defineExceptionType.sql
--

USE Object
GO

---------------------------------------------------------------------------------------------------
-- asisp_defineExceptionType
PRINT 'Creating procedure asisp_defineExceptionType'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_defineExceptionType' AND type = 'P')
  DROP PROCEDURE asisp_defineExceptionType
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/asisp_defineExceptionType.sqv  $
-- $Revision:   1.1  $
-- $Date:   Jan 17 2001 15:24:46  $
-- $Author:   BRENT  $
-- Creates or updates an ExceptionType entry.
CREATE PROCEDURE asisp_defineExceptionType
  @cid                ClassID,      -- Class id of object on which the exception occurred
  @Category           ObjName,      -- Name of category for this exception
  @Exception          VARCHAR(255), -- Name of exception type
  @Description        VARCHAR(255), -- Description of exception type
  @AlertState         VARCHAR(40),  -- Alert level assigned to exception (e.g. - "Green", "Yellow", "Red")
  @Priority           VARCHAR(40),  -- Priority level assigned to exception (e.g. - "Critical", "High", "Medium", "Low", "Ignore", "Eliminate")
  @Salience           INT = 0       -- Optional Salience value used to disambiguate wildcard types. 
                                    -- Types with higher Salience will be matched before those with lower Salience.
AS
  RAISERROR('asisp_defineExceptionType @cid=''%s'', @Category=''%s'', @Exception=''%s'', @Description=''%s'', @AlertState=''%s'', @Priority=''%s''', 0, 1,
            @cid, @Category, @Exception, @Description, @AlertState, @Priority)

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_defineExceptionType
  ELSE
    SAVE TRANSACTION asisp_defineExceptionType

  DECLARE @exct_id ObjID, @extp_id ObjID

  SELECT @exct_id = id 
  FROM ExceptionCategory_C AS exct JOIN link AS l ON id = dst_id 
  WHERE l.link_type = RTRIM(@cid)
    AND l.src_cid = 'STCR' 
    AND l.dst_cid = 'EXCT' 
    AND l.deleted = 0 
    AND exct.name = @Category
  IF @exct_id IS NULL
  BEGIN
    RAISERROR('asisp_defineExceptionType: auto-creating exception category [%s] as does not exist', 0, 1, @Category)
    EXEC @exct_id=asisp_createEXCT @Category, STCR, 1, @Category, @Order=0, @link_type=@cid
  END
  IF @exct_id IS NULL OR @exct_id <= 0
  BEGIN
    RAISERROR('asisp_defineExceptionType: Could not find or create ExceptionCategory %s %s', 3, 1, @cid, @Category)
    ROLLBACK TRANSACTION asisp_defineExceptionType
    RETURN -30000
  END

  DECLARE @extp_name ObjName, @AlertStateStr VARCHAR(60), @PriorityStr VARCHAR(60)
  SELECT @extp_name = RTRIM(@cid) + '_' + @Exception
  SELECT @AlertStateStr = 'GENC/STCR-1/ALRS-' + CAST(id AS VARCHAR(9)) + '/T/Desc' FROM AlertState_V WHERE name = @AlertState
  SELECT @PriorityStr = 'GENC/STCR-1/PRI6-' + CAST(id AS VARCHAR(9)) + '/T/Priority' FROM SixLevelPri_V WHERE name = @Priority
  
  SELECT @extp_id = id FROM ExceptionType_V AS extp 
    JOIN link AS exct_extp ON exct_extp.src_cid='EXCT' AND exct_extp.dst_cid='EXTP' AND exct_extp.dst_id = extp.id
  WHERE
    src_id = @exct_id
    AND extp.name = @extp_name
    AND exct_extp.deleted = 0
  IF @extp_id IS NULL
  BEGIN
    EXEC @extp_id = asisp_createEXTP 
      @name       = @extp_name,
      @parent_cid = EXCT,
      @parent_id  = @exct_id,
      @Exception  = @Exception,
      @Desc       = @Description,
      @AlertState = @AlertStateStr,
      @Priority   = @PriorityStr,
      @Salience   = @Salience
    IF @extp_id < 0
    BEGIN
      ROLLBACK TRANSACTION asisp_defineExceptionType
      RETURN @extp_id
    END
  END
  ELSE
  BEGIN
    UPDATE extp SET
      _Desc       = @Description,
      _AlertState = @AlertStateStr,
      _Priority   = @PriorityStr,
      _Salience   = @Salience
    FROM
      ExceptionType_V AS extp
    WHERE
      id = @extp_id
      AND
      (
        _Desc <> @Description
        OR
        _AlertState <> @AlertStateStr
        OR
        _Priority <> @PriorityStr
        OR
        _Salience <> @Salience
      )
  END

  IF @tcount = 0
    COMMIT TRANSACTION asisp_defineExceptionType

  RETURN @extp_id
GO
