-- $Header:   P:/Archive/DB/SQL/asisp_dropclass.sqv   1.0   Dec 04 2000 12:40:56   BRENT  $
--
-- asisp_dropclass.sql
--
-- Defines the asisp_dropclass procedure.

USE Object
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_dropclass' AND type = 'P')
  DROP PROCEDURE asisp_dropclass
GO

-- $Archive:   P:/Archive/DB/SQL/asisp_dropclass.sqv  $
-- $Revision:   1.0  $
-- $Author:   BRENT  $
-- $Date:   Dec 04 2000 12:40:56  $
-- Drop all model tables for a specified class id
create procedure asisp_dropclass
  @cid ClassID,
  @rebuild TINYINT = 1  -- Specify 0 to avoid rebuilding of meta tables after drop
as
  declare @tcount int
  select @tcount = @@trancount
  if @tcount = 0
    begin tran asisp_dropclass
  else
    save tran asisp_dropclass

  declare @sql varchar(255)

  -- Make sure this class is not a base class for anything
  if exists (select * from isa where base_cid = @cid)
  begin
    select @sql='raiserror("The following classes are derived from ' + rtrim(@cid) + ' and must be dropped first", 11, 1)
           select cid from isa_chain where base_cid="'+rtrim(@cid)+'" and cid!="'+rtrim(@cid)+'"'
    exec(@sql)
    rollback tran asisp_dropclass
    return -30000
  end

  -- drop object tables/views
  declare @cname varchar(24), @table sysname, @type varchar(2)
  declare tablec insensitive cursor for
    select name, type 
    from sysobjects as so join obj_class as c on so.name like c.cname + '[_][ACSV]' 
    where c.cid = @cid and so.type in ('U', 'V')
  open tablec
  fetch next from tablec into @table, @type
  while @@fetch_status <> -1
  begin
    select @sql = 'DROP ' + case @type when 'U' then 'TABLE' else 'VIEW' end + ' ' + @table
    raiserror(@sql, 0, 1)
    exec(@sql)
    if @@error <> 0
    begin
      rollback tran asisp_dropclass
      deallocate tablec
      return -30000
    end
    fetch next from tablec into @table, @type
  end
  deallocate tablec

  -- drop <cid>_ID table
  select @table = rtrim(@cid) + '_ID'
  if exists (select * from sysobjects where name = @table and type = 'U')
  begin
    select @sql = 'DROP TABLE ' + @table
    raiserror(@sql, 0, 1)
    exec(@sql)
    if @@error <> 0
    begin
      rollback tran asisp_dropclass
      return -30000
    end
  end
  
  -- drop dbgen-genenerated stored procedures
  create table #dbgen_proc
  (
    procname sysname
  )
  insert #dbgen_proc select 'alloc_' + rtrim(@cid)
  insert #dbgen_proc select 'asisp_create'+rtrim(@cid)
  insert #dbgen_proc select 'delete_'+rtrim(@cid)
  insert #dbgen_proc select 'asisp_view'+rtrim(@cid)
  insert #dbgen_proc select 'asisp_joinedview'+rtrim(@cid)

  declare procc cursor for select procname from #dbgen_proc
  open procc
  declare @proc sysname
  fetch next from procc into @proc
  while @@fetch_status <> -1
  begin
    if exists (select * from sysobjects where name = @proc and type = 'P')
    begin
      select @sql = 'DROP PROCEDURE ' + @proc
      raiserror(@sql, 0, 1)
      exec(@sql)
      if @@error <> 0
      begin
        rollback tran asisp_dropclass
        return -30000
      end
    end
    fetch next from procc into @proc
  end
  deallocate procc

  -- Remove AttributePages, MessageDescriptions, and StateTypes
  select @sql = 'delete AttributePageSpec_C where name like ''' + rtrim(@cid) + '[_]%'''
  print @sql
  exec(@sql)
  if @@error <> 0
  begin
    rollback tran asisp_dropclass
    return -30000
  end

  select @sql = 'delete MessageDescription_C where name like ''' + rtrim(@cid) + '[_]%'''
  print @sql
  exec(@sql)
  if @@error <> 0
  begin
    rollback tran asisp_dropclass
    return -30000
  end

  select @sql = 'delete StateType_C where name like ''' + rtrim(@cid) + '[_]%'''
  print @sql
  exec(@sql)
  if @@error <> 0
  begin
    rollback tran asisp_dropclass
    return -30000
  end

  -- Remove Current and Desired State enumerations for class
  declare enumc insensitive cursor for
    select name, eid 
    from enumeration 
    where name = rtrim(@cid) + 'CurrentStates' or name = rtrim(@cid) + 'DesiredStates'
  open enumc
  declare @ename varchar(28), @eid int
  fetch next from enumc into @ename, @eid
  while @@fetch_status <> -1
  begin
    raiserror('Removing enumeration %s', 0, 1, @ename)
    delete obj_attr_range where eid = @eid
    if @@error <> 0
    begin
      rollback tran asisp_dropclass
      return -30000
    end

    delete obj_static_range where eid = @eid
    if @@error <> 0
    begin
      rollback tran asisp_dropclass
      return -30000
    end

    delete method_param_range where eid = @eid
    if @@error <> 0
    begin
      rollback tran asisp_dropclass
      return -30000
    end

    delete enum_value where eid = @eid
    if @@error <> 0
    begin
      rollback tran asisp_dropclass
      return -30000
    end

    delete enumeration where eid = @eid
    if @@error <> 0
    begin
      rollback tran asisp_dropclass
      return -30000
    end

    if exists (select * from sysobjects where name = @ename + '_E' and type = 'U')
    begin
      exec('drop table ' + @ename + '_E')
      if @@error <> 0
      begin
        rollback tran asisp_dropclass
        return -30000
      end
    end

    fetch next from enumc into @ename, @eid
  end
  deallocate enumc

  -- Drop link tables
  declare linkc insensitive cursor for
    select rtrim(src_cid) + '_' + rtrim(dst_cid) + '_' + rtrim(link_type) from derived_obj_link
    where link_type = @cid or src_cid = @cid or dst_cid = @cid
  open linkc
  fetch next from linkc into @table
  while @@fetch_status <> -1
  begin
    raiserror('Drop table %s', 0, 1, @table)
    if exists (select * from sysobjects where name = @table and type = 'U')
    begin
      exec('drop table ' + @table)
      if @@error <> 0
      begin
        rollback tran asisp_dropclass
        return -30000
      end
    end
    fetch next from linkc into @table
  end
  deallocate linkc
  
  -- Get rid of containment/link records
  raiserror('Deleting %s records from containment', 0, 1, @cid)
  delete containment where src_cid = @cid or dst_cid = @cid
  raiserror('Deleting %s records from link', 0, 1, @cid)
  delete link where link_type = @cid or src_cid = @cid or dst_cid = @cid
  if @@error <> 0
  begin
    rollback tran asisp_dropclass
    return -30000
  end

  -- Get rid of MenuItems
  raiserror('Deleting %s menu items from the MenuItem table', 0, 1, @cid)
  delete MenuItem where obj_cid = @cid
  if @@error <> 0
  begin
    rollback tran asisp_dropclass
    return -30000
  end

  -- Get rid of meta data

  raiserror('Deleting %s meta data', 0, 1, @cid)

  delete package_obj_method where cid = @cid
  if @@error <> 0
  begin
    rollback tran asisp_dropclass
    return -30000
  end

  delete package_obj_link where link_type = @cid or src_cid = @cid or dst_cid = @cid
  if @@error <> 0
  begin
    rollback tran asisp_dropclass
    return -30000
  end

  delete package_link_type where link_type = @cid
  if @@error <> 0
  begin
    rollback tran asisp_dropclass
    return -30000
  end

  delete package_class where cid = @cid
  if @@error <> 0
  begin
    rollback tran asisp_dropclass
    return -30000
  end

  delete obj_attr_range where cid = @cid
  if @@error <> 0
  begin
    rollback tran asisp_dropclass
    return -30000
  end

  delete obj_link where link_type = @cid or src_cid = @cid or dst_cid = @cid
  if @@error <> 0
  begin
    rollback tran asisp_dropclass
    return -30000
  end

  delete obj_method where cid = @cid
  if @@error <> 0
  begin
    rollback tran asisp_dropclass
    return -30000
  end

  delete obj_attr where cid = @cid
  if @@error <> 0
  begin
    rollback tran asisp_dropclass
    return -30000
  end

  delete obj_static where cid = @cid
  if @@error <> 0
  begin
    rollback tran asisp_dropclass
    return -30000
  end

  delete isa_chain where cid = @cid
  if @@error <> 0
  begin
    rollback tran asisp_dropclass
    return -30000
  end

  delete isa where cid = @cid
  if @@error <> 0
  begin
    rollback tran asisp_dropclass
    return -30000
  end

  delete obj_class where cid = @cid
  if @@error <> 0
  begin
    rollback tran asisp_dropclass
    return -30000
  end

  delete link_type where link_type = @cid
  if @@error <> 0
  begin
    rollback tran asisp_dropclass
    return -30000
  end

  print 'Updating Meta Database Version..'
  update Meta..DatabaseVersion set verno=verno+1

  if @rebuild = 1
  BEGIN
    print 'Rebuilding isa_chain'
    exec Meta..asisp_make_isa_chain

    print 'Rebuilding link_chain'
    exec Meta..asisp_makelinkchain

    print 'Rebuilding link_path and link_path_detail'
    exec Meta..asisp_makelinkpath
  END

  if @tcount = 0
    commit tran asisp_dropclass
GO
