-- $Header:   //ARCHIVE/Archive/DB/sql/asisp_generatechildevent.sqv   1.1   Feb 04 2001 20:49:18   BRENT  $
--
-- asisp_generatechildevent.sql

USE Object
GO


---------------------------------------------------------------------------------------------------
-- asisp_generatechildevent
PRINT 'Creating procedure asisp_generatechildevent'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_generatechildevent' AND type = 'P')
  DROP PROCEDURE asisp_generatechildevent
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/asisp_generatechildevent.sqv  $
-- $Revision:   1.1  $
-- $Date:   Feb 04 2001 20:49:18  $
-- $Author:   BRENT  $
-- Generates child events from the child object specified by the @dst_cid, @dst_id parameters. The
-- child event will be sent to the object specified by the @src_cid, @src_id parameters, plus any
-- appropriate object that has a 'propagatable' link to the child object. Propagatable links are
-- those whose link_type's have the flag_containment or flag_parent_child_link columns set.
CREATE PROCEDURE asisp_generatechildevent
  @src_cid      ClassID,
  @src_id       ObjID,
  @dst_cid      ClassID,
  @dst_id       ObjID,
  @EvtCID       ClassID,
  @EvtDetail    VARCHAR(255),
  @EvtCount     INT,
  @Direction    INT,
  @Priority     INT,
  @AlertState   INT,
  @SumDirection INT = null
AS
--  RAISERROR('asisp_generatechildevent "%s", %d, "%s", %d, "%s", "%s", %d, %d, %d, %d:', 2, 1,
--            @src_cid, @src_id, @dst_cid, @dst_id, @EvtCID, @EvtDetail, @EvtCount, @Direction,
--            @Priority, @AlertState)
  DECLARE @tcount int
  SELECT @tcount = @@trancount
  IF @tcount > 0
    SAVE TRANSACTION generatechildevent
  ELSE
    BEGIN TRANSACTION generatechildevent

  -- Create table of parent links for the child object
  SELECT CAST('XXXX' AS VARCHAR(4)) AS cid, CAST(0 AS INTEGER) AS id INTO #parent
  DELETE #parent
  INSERT INTO #parent
  SELECT src_cid AS cid, src_id AS id
    FROM link AS l
      JOIN link_type AS lt ON lt.link_type = l.link_type
      JOIN isa_chain AS i ON i.cid = l.src_cid
    WHERE
      dst_cid = @dst_cid AND dst_id = @dst_id
      AND deleted = 0
      AND lt.flags & 3 <> 0
      AND i.base_cid = 'MNO2'
  IF @src_cid IS NOT NULL
  BEGIN
    IF NOT EXISTS (SELECT * FROM #parent WHERE cid = @src_cid AND id = @src_id)
      INSERT #parent SELECT @src_cid, @src_id
  END
  ELSE
  BEGIN
    -- See if the object is directly contained. If so, then lookup its parent info
    DECLARE @cname VARCHAR(24), @flag_direct_containment TINYINT
    SELECT @cname = cname, @flag_direct_containment = flag_direct_containment FROM obj_class WHERE cid = @dst_cid
    IF @flag_direct_containment = 1
    BEGIN
      DECLARE @dst_id_str VARCHAR(9)
      SET @dst_id_str = CAST(@dst_id AS VARCHAR(9))
      EXEC("
      INSERT #parent SELECT _ParentCID, _ParentID FROM " + @cname + "_V WHERE id=" + @dst_id_str + "
      AND NOT EXISTS (SELECT * FROM #parent WHERE cid = _ParentCID AND id = _ParentID)
      ")
    END
  END

  -- Create the child event(s)
  SELECT CAST(0 AS INTEGER) AS src_id INTO #lobl_linked_lobs
  DELETE #lobl_linked_lobs

  DECLARE @chev_name ObjName, @chev_id ObjID
  DECLARE @parent_cid ClassID, @parent_id ObjID
  DECLARE parentc CURSOR FOR SELECT cid, id FROM #parent
  OPEN parentc
  FETCH NEXT FROM parentc INTO @parent_cid, @parent_id
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SELECT @chev_name = RTRIM(@parent_cid) + '-' + CONVERT(VARCHAR(9), @parent_id)
    EXEC @chev_id = asisp_createCHEV 
      @name=@chev_name,
      @parent_cid=@parent_cid,
      @parent_id=@parent_id,
      @ChildCID=@dst_cid,
      @ChildID=@dst_id,
      @EventCID=@EvtCID,
      @EventDetail=@EvtDetail,
      @EventCount=@EvtCount,
      @Direction=@Direction,
      @Priority=@Priority,
      @AlertState=@AlertState,
      @SumDirection=@SumDirection
    IF @chev_id < 0
    BEGIN
      ROLLBACK TRANSACTION generatechildevent
      DEALLOCATE parentc
      RETURN -30000
    END
    -- special processing for LOB's linked via LOBL links to other LOB's
    INSERT INTO #lobl_linked_lobs
    SELECT src_id 
      FROM link 
      WHERE 
        @parent_cid='LOB'
        AND dst_cid='LOB' 
        AND dst_id=@parent_id 
        AND src_cid='LOB' 
        AND link_type='LOBL' 
        AND deleted=0
    IF @@rowcount > 0
    BEGIN
      -- cursor over the LOB's linked via LOBL links to other LOB's
      DECLARE @stat int
      DECLARE @linked_src_id ObjID
      DECLARE @linked_chev_id ObjID
      EXEC @stat = asisp_declarecursor lobl_linked_lobs_c, "SELECT src_id FROM #lobl_linked_lobs", 1
      IF @stat < 0
        RETURN
      OPEN lobl_linked_lobs_c
      FETCH NEXT FROM lobl_linked_lobs_c INTO @linked_src_id
      WHILE @@fetch_status <> -1
      BEGIN
        EXEC @linked_chev_id = asisp_createCHEV 
          @name=@chev_name,
          @parent_cid=@parent_cid,
          @parent_id=@linked_src_id,
          @ChildCID=@dst_cid,
          @ChildID=@dst_id,
          @EventCID=@EvtCID,
          @EventDetail=@EvtDetail,
          @EventCount=@EvtCount,
          @Direction=@Direction,
          @Priority=@Priority,
          @AlertState=@AlertState,
          @SumDirection=@SumDirection
        IF @linked_chev_id >  0
          RAISERROR('asisp_generatechildevent for LOB linked via LOBL link "%s", %d, "%s", %d, "%s", "%s", %d, %d, %d, %d', 0, 1,
                    @parent_cid, @linked_src_id, @dst_cid, @dst_id, @EvtCID, @EvtDetail, @EvtCount, @Direction,
                    @Priority, @AlertState)
        ELSE
        BEGIN
          RAISERROR('asisp_generatechildevent FAILED to CREATE CHEV for LOB linked via LOBL link "%s", %d, "%s", %d, "%s", "%s", %d, %d, %d, %d', 0, 1,
                    @parent_cid, @linked_src_id, @dst_cid, @dst_id, @EvtCID, @EvtDetail, @EvtCount, @Direction,
                    @Priority, @AlertState)
/*
          ROLLBACK TRANSACTION generatechildevent
          DEALLOCATE lobl_linked_lobs_c
          DEALLOCATE parentc
          RETURN -30000
*/
        END
        FETCH NEXT FROM lobl_linked_lobs_c INTO @linked_src_id
      END
      DEALLOCATE lobl_linked_lobs_c
    END
    FETCH NEXT FROM parentc INTO @parent_cid, @parent_id
  END
  DEALLOCATE parentc

  IF @chev_id IS NULL
    SELECT @chev_id = 0

  IF @tcount = 0
    COMMIT TRANSACTION generatechildevent

  RETURN @chev_id
GO
