-- $Header:   //ARCHIVE/Archive/DB/sql/asisp_getchildlist.sqv   1.0   Feb 14 2001 06:39:18   BRENT  $
--
-- asisp_getchildlist.sql

USE Object
GO

---------------------------------------------------------------------------------------------------
-- asisp_getchildlist
PRINT 'Creating procedure asisp_getchildlist'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_getchildlist' AND type = 'P')
  DROP PROCEDURE asisp_getchildlist
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/asisp_getchildlist.sqv  $
-- $Revision:   1.0  $
-- $Date:   Feb 14 2001 06:39:18  $
-- $Author:   BRENT  $
-- Gets the children of the specified object. 'Children' for the purposes of this procedure
-- are objects that are on the destination side of links where the link_type has flag_parent_child
-- set to 1.
CREATE PROCEDURE asisp_getchildlist
  @cid  ClassID,
  @id   ObjID
as
  declare @var_set varchar(80)
  select @var_set = 'select @cid = ''' + rtrim(@cid) + ''', @id = ' + convert(varchar(9), @id)
  exec("
  -- Standard transaction logic
  declare @tcount int
  select @tcount = @@trancount
  if @tcount = 0
    begin tran asisp_getchildlist
  else
    save tran asisp_getchildlist
  declare @cid ClassID, @id ObjID " + @var_set + "
  create table #childlist
  (
    link_type   varchar(4),
    src_cid     ClassID,
    src_cno     smallint,
    src_id      ObjID,
    dst_cid     ClassID,
    dst_cno     smallint,
    dst_id      ObjID PRIMARY KEY (link_type, dst_cid, dst_id, src_cid, src_id),
    flags       int,
    generation  int
  )
  
  declare @generation int, @rc int, @err int
  select @generation = 1
  
  -- Insert the children, as determined by link_type.flag_parent_child
  insert #childlist
    select 
      l.link_type,
      l.src_cid,
      src.cno,
      l.src_id,
      l.dst_cid,
      dst.cno,
      l.dst_id,
      t.cls_flags,
      @generation
    from
      link l,
      obj_link_type t,
      obj_class src,
      obj_class dst
    where
      -- the specified object is on the source side of a nondeleted link
      l.src_cid=@cid and src_id=@id and deleted=0
      -- join the link to the link_type table
      and l.link_type=t.link_type and l.src_cid=t.src_cid and l.dst_cid=t.dst_cid
      -- look for only the parent-child link types
      and t.flag_parent_child=1
      -- lookup class numbers
      and src.cid = l.src_cid
      and dst.cid = l.dst_cid
  select @rc = @@rowcount, @err = @@error
  if @err <> 0
  begin
    raiserror('asisp_getchildlist: insert #childlist failed: error=%d', 4, 1, @err)
    rollback tran asisp_getchildlist
  end
  -- For LOB children we also get their linked objects.  This loop traverses all the way from the
  -- LOB to its ultimate physical object
  if exists (select * from #childlist where dst_cid = 'LOB')

  begin
    while @rc > 0
    begin
      raiserror('Getting generation %d for %s %d', 0, 1, @generation, @cid, @id)
      select @generation = @generation + 1
      insert #childlist 
        select 
          l.link_type,
          l.src_cid,
          src.cno,
          l.src_id,
          l.dst_cid,
          dst.cno,
          l.dst_id,
          t.cls_flags,
          @generation
        from 
          link l /*(INDEX=src_link)*/,
          #childlist o,
          obj_link_type t,
          obj_class src,
          obj_class dst
        where 
          -- Look for objects that are linked to objects in #childlist from the previous generation
          l.src_cid=o.dst_cid and l.src_id=o.dst_id
          and not (l.src_cid=o.src_cid and l.src_id=o.src_id)
          and deleted=0 
          and generation=@generation-1
          -- join link to obj_link_type
          and l.src_cid=t.src_cid and l.dst_cid=t.dst_cid
          and l.link_type=t.link_type 
          -- look only for the logical links
          and flag_logical=1
          -- lookup class numbers
          and src.cid = l.src_cid
          and dst.cid = l.dst_cid
      select @rc = @@rowcount, @err = @@error
      if @err <> 0
      begin
        raiserror('asisp_getchildlist: insert #childlist failed at generation %d: error=%d', 4, 1, @generation, @err)
        rollback tran asisp_getchildlist
      end
    end
  end
  -- Select the grandchildrens source links.  These are needed by the current model implementation
  select 
    l.link_type,
    l.src_cid,
    src_cno=src.cno,
    l.src_id,
    l.dst_cid,
    dst_cno=dst.cno,
    dst_id=min(l.dst_id),
    flags=t.cls_flags,
    generation=-1  -- this generation value denotes the results as link info only
  from
    link l /*(INDEX=src_link)*/,
    #childlist o,
    obj_link_type t,
    obj_class src,
    obj_class dst
  where
    -- the specified object is on the source side of a nondeleted link
    l.src_cid=o.dst_cid and l.src_id=o.dst_id and deleted=0 and generation=1
    -- join the link to the link_type table
    and l.link_type=t.link_type and l.src_cid=t.src_cid and l.dst_cid=t.dst_cid
    -- look for only the parent-child link types
    and t.flag_parent_child=1
    -- lookup class numbers
    and src.cid = l.src_cid
    and dst.cid = l.dst_cid
  group by
    l.link_type,
    l.src_cid,
    src.cno,
    l.src_id,
    l.dst_cid,
    dst.cno,
    t.cls_flags
  select @rc = @@rowcount, @err = @@error
  if @err <> 0
  begin
    raiserror('asisp_getchildlist: failed getting grandchildren links: error=%d', 4, 1, @err)
    rollback tran asisp_getchildlist
  end
  -- Special LOB logic: If the object has LOB children that are linked to LOBs we need for each one of these

  -- children to select one of the parent-child links where the linked LOB is on the source side.  Again, this
  -- is a requirement for the current model implementation to display the object correctly.
  select
    l.link_type,
    l.src_cid,
    src_cno=src.cno,
    l.src_id,
    l.dst_cid,
    dst_cno=dst.cno,
    dst_id=min(l.dst_id),
    flags=t.flags,
    generation=-1
  from
    #childlist o,
    link l /*(INDEX=src_link)*/,
    link_type t,
    obj_class src,
    obj_class dst
  where
    -- looking for only LOB->LOB links
    o.src_cid = 'LOB' and o.dst_cid = 'LOB'
    -- the object on the target side of the link is the object we want to get a parent-child link for
    and l.src_cid = 'LOB'
    and l.src_id = o.dst_id
    and l.deleted = 0
    and l.link_type = t.link_type
    and flag_parent_child = 1
    -- lookup class numbers
    and src.cid = l.src_cid
    and dst.cid = l.dst_cid
  group by
    l.link_type,
    l.src_cid,
    src.cno,
    l.src_id,
    l.dst_cid,
    dst.cno,
    t.flags
  select @rc = @@rowcount, @err = @@error
  if @err <> 0
  begin
    raiserror('asisp_getchildlist: LOB logic failed: error=%d', 4, 1, @err)
    rollback tran asisp_getchildlist
  end
  -- For each distinct class selected, we join #childlist to the appropriate class view
  declare childlistc cursor for select distinct dst_cid from #childlist
  open childlistc
  declare @dst_cid ClassID, @select_links tinyint
  declare @proc varchar(80), @initial_columns varchar(255), @joined_tables varchar(255), @where_clause varchar(255)
  select @initial_columns = 'cl.link_type, cl.src_cid, cl.src_cno, cl.src_id, cl.dst_cid, cl.dst_cno, cl.dst_id, cl.flags, cl.generation,'
  select @err = 0, @select_links = 1
--  raiserror('@initial_columns=''%s''', 0, 1, @initial_columns)
  fetch next from childlistc into @dst_cid
  while @@fetch_status <> -1 and @err = 0
  begin
    select @proc = 'exec asisp_joinedview' + @dst_cid + ' '
    select @joined_tables = ',#childlist cl'
--    select @where_clause = 'cl.dst_cid = ''''' + rtrim(@dst_cid) + ''''' and v.id=cl.dst_id and generation<>-1'
    select @where_clause = 'cl.dst_cid = ''''' + rtrim(@dst_cid) + ''''' and v.id=cl.dst_id'
--    raiserror('@proc=''%s''', 0, 1, @proc)
--    raiserror('@joined_tables=''%s''', 0, 1, @joined_tables)
--    raiserror('@where_clause=''%s''', 0, 1, @where_clause)
    exec(@proc
         + 
         ' ''' + @initial_columns + ''','
         +
         ' ''' + @joined_tables + ''','
         +
         ' ''' + @where_clause + ''''
        )
    select @err = @@error
    if @err <> 0
    begin
      select @select_links = 0
      close childlistc
      break
    end
    fetch next from childlistc into @dst_cid
  end
  deallocate childlistc
  if @err <> 0
  begin
    raiserror('asisp_getchildlist: failed making class view: error=%d', 4, 1, @err)
    rollback tran asisp_getchildlist
  end
  if @tcount = 0
    commit tran asisp_getchildlist
--  raiserror('#childlist:', 0, 1)
--  select * from #childlist
  ")
GO
