-- $Header:   //ARCHIVE/Archive/DB/sql/asisp_lookup_pa_by_cno.sqv   1.0   Jan 31 2001 12:14:34   BRENT  $
--
-- asisp_lookup_pa_by_cno

USE Object
GO

----------------------------------------------------------------------------------------------------
-- asisp_lookup_pa_by_cno
PRINT 'Creating procedure asisp_lookup_pa_by_cno'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_lookup_pa_by_cno' AND type = 'P')
  DROP PROCEDURE asisp_lookup_pa_by_cno
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/asisp_lookup_pa_by_cno.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jan 31 2001 12:14:34  $
-- $Author:   BRENT  $
-- Selects a row that contains the id of the propagation agent that handles the events for
-- the specified object.
CREATE PROCEDURE asisp_lookup_pa_by_cno
  @cno     SMALLINT,    -- Object's class number
  @id      ObjID,       -- Object's instance id
  @verbose TINYINT = 0  -- Option to display messages while processing. Set to 1 to turn on.
AS
  IF @verbose = 1
    RAISERROR('asisp_lookup_pa_by_cno %d, %d', 0, 1, @cno, @id)
  SET NOCOUNT ON

  DECLARE @cid ClassID, @dst_id ObjID/*, @msg varchar(255)*/
  SELECT @cid = cid FROM obj_class WHERE cno = @cno
  SELECT @dst_id = dst_id FROM link
    WHERE dst_cid='PA' AND src_cid=@cid AND src_id=@id AND deleted = 0
  IF @@ROWCOUNT != 0
  BEGIN
    SELECT agent_id=@dst_id FROM PropagationAgent_A WHERE id = @dst_id AND _StartMode <> 'disabled'
    IF @@ROWCOUNT = 1
      RETURN
  END

  -- Look through link table for the first ancestor that has a nondisabled PA
  CREATE TABLE #parent ( cid ClassID, id ObjID )
  WHILE @@ROWCOUNT = 0
  BEGIN
    IF @verbose = 1
      RAISERROR('asisp_lookup_pa_by_cno: looking immediate ancestor of %s %d', 1, 1, @cid, @id)
    DECLARE @cname varchar(24)
    SELECT @cname = NULL
    SELECT @cname = cname FROM obj_class WHERE cid = @cid AND flag_direct_containment = 1
    IF @cname IS NOT NULL
    BEGIN
      DECLARE @id_str varchar(9)
      SELECT @id_str = convert(varchar(9), @id)
      delete #parent
      EXEC("
      INSERT #parent SELECT _ParentCID, _ParentID FROM " + @cname + "_V WHERE id = " + @id_str
      )
      SELECT @cid = cid, @id = id FROM #parent
      IF @@ROWCOUNT = 0
        RETURN -- we've reached the end of the object's ancestory, so no PA  
    END
    ELSE -- object class is not directly contained
    BEGIN
      SET ROWCOUNT 1
      SELECT DISTINCT @cid = src_cid, @id = src_id
        FROM link AS l 
        JOIN link_type AS lt ON lt.link_type = l.link_type
        WHERE
          l.dst_cid = @cid
          AND l.dst_id = @id
          AND l.deleted = 0 
          AND (lt.flag_containment = 1 OR lt.flag_parent_child = 1)
      IF @@ROWCOUNT = 0 -- not in containment table, so check for direct containment of class
      BEGIN
        RETURN -- we've reached the end of the object's ancestory, so no PA  
      END
      SET ROWCOUNT 0
    END

--    RAISERROR('asisp_lookup_pa_by_cno: looking for PA attached to %s %d', 1, 1, @cid, @id)
    SELECT @dst_id = dst_id
    FROM link, PropagationAgent_A
    WHERE
      dst_cid='PA'
      AND src_cid = @cid
      AND src_id = @id
      AND deleted = 0
      AND _StartMode<>'disabled'
    IF @@ROWCOUNT != 0  -- found it!
    BEGIN
      SELECT agent_id = @dst_id
      RETURN
    END
  END
GO
