-- $Header:   //ARCHIVE/Archive/DB/sql/asisp_processStagedEXCPs.sqv   1.1   Jan 12 2001 13:15:54   BRENT  $

USE Object
GO

-- We redo the primary key on the worktable StagedEXCPInProgress to include the id column. This
-- is needed since we now support wildcarded clearing exceptions, which can match against multiple
-- Exception records.
DECLARE @keyname sysname
SELECT @keyname = name from sysobjects where parent_obj = object_id('StagedEXCPInProgress') AND type = 'K' AND name <> 'StagedEXCPInProgress_PK'
IF @keyname IS NOT NULL
BEGIN
  PRINT 'Dropping old primary key ' + @keyname + ' from StagedEXCPInProgress'
  EXEC("
  ALTER TABLE StagedEXCPInProgress DROP CONSTRAINT " + @keyname
  )
  PRINT 'Adding new primrary key to StagedEXCPInProgress'
  EXEC("
  ALTER TABLE StagedEXCPInProgress ADD CONSTRAINT StagedEXCPInProgress_PK PRIMARY KEY CLUSTERED(timestamp, id)
  ")
END
GO


---------------------------------------------------------------------------------------------------
-- asisp_processStagedEXCPs
PRINT 'Creating procedure asisp_processStagedEXCPs'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_processStagedEXCPs' AND type = 'P')
  DROP PROCEDURE asisp_processStagedEXCPs
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/asisp_processStagedEXCPs.sqv  $
-- $Revision:   1.1  $
-- $Date:   Jan 12 2001 13:15:54  $
-- $Author:   BRENT  $
-- Processes the records in the StagedEXCP table, up to the specified @rowlimit
CREATE PROCEDURE asisp_processStagedEXCPs
  @rowlimit INT = 100,    -- Maximum number of rows to process
  @verbose  TINYINT = 0   -- If 1 then verbose progress messages are printed during execution
AS
  -- Standard TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_processStagedEXCPs
  ELSE
    SAVE TRANSACTION asisp_processStagedEXCPs

  SET ROWCOUNT @rowlimit

  DECLARE @rc INT, @err INT, @msg VARCHAR(255), @chkpt INT
  SET @chkpt = 1

  -- Repopulate work table with ids of all EXCP events that are available for processing, 'available' being defined
  -- as a EXCP whose parent object does NOT have any earlier MESG events that have NOT been processed.
  TRUNCATE TABLE StagedEXCPInProgress
  INSERT StagedEXCPInProgress
    SELECT
      0,
      timestamp,
      ctime,
      parent_cid,
      parent_id,
      TIME,
      ISNULL(SEQ_NO,''),
      ISNULL(OBJECT_NAME,''),
      ISNULL(NATIVE_KEY,''),
      ISNULL(EXCP_OBJ_TYPE,''),
      ISNULL(EXCP_OBJ_NAME,''),
      EXCP_NAME,
      EXCP_CD,
      nEXCH,
      AlertState,
      Priority
    FROM
      StagedEXCP e
    WHERE
      timestamp NOT IN (SELECT evt_timestamp FROM ProcessedStagedEvent)
    ORDER BY timestamp
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  EXEC master..xp_asisprintf @msg output, 'asisp_processStagedEXCPS: @@ERROR=%d, @@ROWCOUNT=%d', @err, @rc
  EXEC asisp_checkpointmessage @msg, 1, @chkpt
  SET @chkpt = @chkpt + 1
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  IF @rc = 0
  BEGIN
    IF @tcount = 0
      COMMIT TRANSACTION asisp_processStagedEXCPs
    RETURN 0
  END

  SET ROWCOUNT 0

  -- Prune all records for which there are earlier MESG events that have NOT been processed.
  DELETE StagedEXCPInProgress
  WHERE
    EXISTS
    (
      SELECT * FROM StagedMESG m
      WHERE 
        m.parent_cid = StagedEXCPInProgress.parent_cid AND m.parent_id = StagedEXCPInProgress.parent_id
        AND m.timestamp < StagedEXCPInProgress.timestamp
        AND NOT EXISTS (SELECT * FROM ProcessedStagedEvent WHERE evt_timestamp = m.timestamp)
    )
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    raiserror('asisp_processStagedEXCPs: error %d pruning records for unprocessed messages', 3, 1, @err)
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  EXEC master..xp_asisprintf @msg output, 'asisp_processStagedEXCPs: Removed %d row(s) for which there are earlier unprocessed StagedMESG records', @rc
  EXEC asisp_checkpointmessage @msg, 1, @chkpt
  SET @chkpt = @chkpt + 1

  DECLARE @num_events_processed INT
  SELECT @num_events_processed = count(*) FROM StagedEXCPInProgress
  IF @num_events_processed = 0
  BEGIN
    IF @tcount = 0
      COMMIT TRANSACTION asisp_processStagedEXCPs
    RETURN 0
  END

  IF @verbose = 1
  BEGIN
    SET NOCOUNT OFF
    SELECT 'INITIAL' AS 'INITIAL', * FROM StagedEXCPInProgress
    SET NOCOUNT ON
  END

  -- All remaining records in StagedEXCPInProgress are to be processed, so record this in ProcessedStagedEvent
  INSERT ProcessedStagedEvent(evt_timestamp, ptime) 
    SELECT timestamp, GETDATE() FROM StagedEXCPInProgress
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: inserted ProcessedStagedEvent', 1, @chkpt
  SET @chkpt = @chkpt + 1

  -- HACK: 02/04/1999-dwp.  The following block of code is a hack to get around
  -- bad data FROM MVS feed for TAPE and DASD objects that were
  -- to be assigned to the OS.  The rexx code that was included in the AOP
  -- release had not been updated to correctly assign TAPE and DASD exceptions
  -- to the OS.  Rather, assigned to an incorrect OS as specified by the oid
  -- portion of the native key (H7X0=4 FROM 0000040002) instead of to E2J0
  --
  -- Hack may be removed when the rexx code for AOP is re-released
  --
  -- 00000027002\01\001999020411:23:23.488567\01E2J010660\02OS\03SRSC\040000040002\57TAPE\58......\59TRDY\60+ TRDY Warning: Tape NOT ready at 2005            Volser =   ......             ~
  -- 00000023102\01\001999020411:27:23.471403\01E2J010707\02OS\03SRSC\040000040002\57TAPE\58......\59TRDY\60+ TRDY Warning: Tape NOT ready at 2005            Volser =   ......             ~
  -- 00000015002\01\001999020411:33:23.556908\01E2J010941\02OS\03SRSC\040000040002\57TAPE\58......\59TRDY\60OK~
  --
  UPDATE StagedEXCPInProgress SET 
    parent_cid = 'OS',
    parent_id = os.id
  FROM
    OS_C os
  WHERE
    parent_cid IN ('DASD', 'TAPE')
    AND name = SUBSTRING(SEQ_NO, 1, 4)
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: performed DASD/TAPE -> OS replacement', 0, @chkpt
  SET @chkpt = @chkpt + 1

  -- Delete any events belonging to objects marked as deleted
  DELETE StagedEXCPInProgress FROM containment c WHERE parent_cid = c.dst_cid AND parent_id = c.dst_id AND c.deleted = 1
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: deleted events belonging to deleted objects', 0, @chkpt
  SET @chkpt = @chkpt + 1

  -- Delete any events with 'Elminate' priority
  DELETE StagedEXCPInProgress WHERE Priority = 6

  -- Update the AlertState/Priority values of all rows with NULL values
  UPDATE s SET 
    AlertState = ISNULL(s.AlertState, t.AlertStateID),
    Priority = ISNULL(s.Priority, t.PriorityID)
  FROM StagedEXCPInProgress AS s 
    JOIN ExceptionType_T AS t ON t.cid = s.parent_cid AND s.EXCP_NAME LIKE t.Exception 
  WHERE
    (s.AlertState IS NULL OR s.Priority IS NULL)
    AND NOT EXISTS
    (
      SELECT * FROM ExceptionType_T AS tplus 
      WHERE
        tplus.cid = s.parent_cid
        AND s.EXCP_NAME LIKE tplus.Exception
        AND tplus.Salience > t.Salience
    )
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: Looked up AlertState/Priority values from ExceptionType', 0, @chkpt
  SET @chkpt = @chkpt + 1

  -- Replace nulls with 0s for AlertState/Priority
  UPDATE StagedEXCPInProgress SET
    AlertState = ISNULL(AlertState, 0),
    Priority = ISNULL(Priority, 0)
  WHERE AlertState IS NULL OR Priority IS NULL
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: replaced NULL AlertState/Priority values', 0, @chkpt
  SET @chkpt = @chkpt + 1

  -- Remove all rows from StagedEXCPInProgress where there is a subsequent 'clear' event
  DELETE StagedEXCPInProgress WHERE EXISTS
  (
    SELECT * FROM StagedEXCPInProgress ce WHERE
      ce.parent_cid = StagedEXCPInProgress.parent_cid
      AND ce.parent_id = StagedEXCPInProgress.parent_id
      AND ce.EXCP_NAME = StagedEXCPInProgress.EXCP_NAME
      AND (ce.AlertState = StagedEXCPInProgress.AlertState OR ce.AlertState = -1)
      AND (ce.Priority = StagedEXCPInProgress.Priority OR ce.Priority = -1)
      AND ce.timestamp > StagedEXCPInProgress.timestamp
      AND (ce.EXCP_CD = 'OK' OR ce.EXCP_CD = 'SC' OR ce.EXCP_CD like 'SC[ ]%')
  )
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: removed records with subsequent clear events', 0, @chkpt
  SET @chkpt = @chkpt + 1

  IF @verbose = 1
  BEGIN
    SET NOCOUNT OFF
    SELECT 'CLEARED_REMOVED' AS 'CLEARED_REMOVED', * FROM StagedEXCPInProgress ORDER BY timestamp
    SET NOCOUNT ON
  END

  -- The staged exceptions which have an AlertState or Priority of -1 and EXCP_CD = 'OK' indicate
  -- clears where we want to do a wildcard match on the column(s) with a -1 value.
  INSERT StagedEXCPInProgress(id,timestamp,ctime,parent_cid,parent_id,TIME,SEQ_NO,OBJECT_NAME,NATIVE_KEY,EXCP_OBJ_TYPE,EXCP_OBJ_NAME,EXCP_NAME,EXCP_CD,nEXCH,AlertState,Priority)
    SELECT e.id,p.timestamp,p.ctime,p.parent_cid,p.parent_id,p.TIME,p.SEQ_NO,p.OBJECT_NAME,p.NATIVE_KEY,p.EXCP_OBJ_TYPE,p.EXCP_OBJ_NAME,p.EXCP_NAME,p.EXCP_CD,p.nEXCH,e._AlertState,e._Priority
    FROM StagedEXCPInProgress AS p JOIN Exception_V AS e ON
      e._ParentCID = p.parent_cid
      AND e._ParentID = p.parent_id
      AND e._EXCP_NAME = p.EXCP_NAME
      AND (e._AlertState = p.AlertState OR p.AlertState = -1)
      AND (e._Priority = p.Priority OR p.Priority = -1)
    WHERE
      (p.AlertState = -1 OR p.Priority = -1)
      AND e.deleted = 0
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: performed wildcard clear lookups', 0, @chkpt
  SET @chkpt = @chkpt + 1

  -- The wildcard rows have now been replaced with 'clear' rows for the matching exceptions,
  -- so we can get rid of them
  DELETE StagedEXCPInProgress WHERE AlertState = -1 OR Priority = -1
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: removed wildcard rows', 0, @chkpt
  SET @chkpt = @chkpt + 1

  IF @verbose = 1
  BEGIN
    SET NOCOUNT OFF
    SELECT 'CLEARED_REMOVED' AS 'CLEARED_REMOVED', * FROM StagedEXCPInProgress ORDER BY timestamp
    SET NOCOUNT ON
  END

  -- At this point for each object there may one 'clear' event followed by 0 or more observations
  -- We find these by looking up the ids of the previously generated nondeleted exceptions that
  -- match them
  UPDATE p SET id = e.id 
  FROM StagedEXCPInProgress AS p JOIN Exception_V AS e ON
    e._ParentCID = p.parent_cid
    AND e._ParentID = p.parent_id
    AND e._EXCP_NAME = p.EXCP_NAME
    AND e._AlertState = p.AlertState
    AND e._Priority = p.Priority
  WHERE
    (p.EXCP_CD = 'OK' OR p.EXCP_CD = 'SC' OR p.EXCP_CD like 'SC[ ]%')
    AND e.deleted = 0
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: looked up ids for cleared exceptions', 0, @chkpt
  SET @chkpt = @chkpt + 1

  -- Check for owned exceptions
  UPDATE p SET id = e.id 
  FROM 
    StagedEXCPInProgress AS p
    JOIN Exception_V AS e ON 
      e._ParentCID = p.parent_cid
      AND e._ParentID = p.parent_id
      AND e._EXCP_NAME = p.EXCP_NAME
      AND e._AlertState = p.AlertState
      AND e._Priority = p.Priority
    JOIN OwnedEvent AS o ON
      o.eventcid = 'EXCP'
      AND o.eventid = e.id
  WHERE
    e.deleted = 1
    AND p.EXCP_CD like 'SC[ ]%'
    AND o.status = 0
    AND p.id = 0
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: updated owned exceptions', 0, @chkpt
  SET @chkpt = @chkpt + 1

  IF @verbose = 1
  BEGIN
    SET NOCOUNT OFF
    SELECT 'AFTER_OWNED_CHECKED' AS 'AFTER_OWNED_CHECKED', * FROM StagedEXCPInProgress ORDER BY timestamp
    SET NOCOUNT ON
  END

  -- Check for clears of owned exceptions. For these we just clear the OwnedEvent records
  TRUNCATE TABLE StagedEXCPClearedOwned
  INSERT StagedEXCPClearedOwned
    SELECT e.id
    FROM
      StagedEXCPInProgress AS p
      JOIN Exception_A AS e ON
        e._ParentCID = p.parent_cid
        AND e._ParentID = p.parent_id
        AND e._EXCP_NAME = p.EXCP_NAME
        AND e._AlertState = p.AlertState
        AND e._Priority = p.Priority
      JOIN OwnedEvent AS o ON
        o.eventcid = 'EXCP'
        AND o.eventid = e.id
    WHERE
      o.status = 0
      AND p.EXCP_CD = 'OK'
  UPDATE OwnedEvent SET status = 1 FROM StagedEXCPClearedOwned WHERE eventcid='EXCP' AND eventid=id

  -- Remove all 'false' clears. These are rows that have a EXCP_CD indicating a cleared exception
  -- but a zero id which tells us that there is no corresponding Exception_V record.
  DELETE StagedEXCPInProgress WHERE id = 0 AND (EXCP_CD = 'OK' OR EXCP_CD = 'SC' OR EXCP_CD like 'SC[ ]%')
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: removed false clears', 0, @chkpt
  SET @chkpt = @chkpt + 1
  
  IF @verbose = 1
  BEGIN
    SET NOCOUNT OFF
    SELECT 'AFTER_FALSE_CLEARS' AS 'AFTER_FALSE_CLEARS', * FROM StagedEXCPInProgress ORDER BY timestamp
    SET NOCOUNT ON
  END

  -- Delete all cleared exceptions
  DECLARE @deleted_count INT
  TRUNCATE TABLE StagedEXCPDeletedHistory
  INSERT StagedEXCPDeletedHistory(exch_id, excp_id) 
    SELECT 0, id FROM StagedEXCPInProgress
    WHERE EXCP_CD = 'OK' OR EXCP_CD = 'SC' OR EXCP_CD like 'SC[ ]%'
  SELECT @deleted_count = @@ROWCOUNT
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: deleting cleared exceptions', 0, @chkpt
  SET @chkpt = @chkpt + 1
  IF @deleted_count > 0
  BEGIN
    -- Create history records
    DECLARE @exch_base_id ObjID, @date datetime
    SELECT @date = GETDATE()
    EXEC @exch_base_id = alloc_EXCH @deleted_count
    UPDATE StagedEXCPDeletedHistory SET exch_id = row_id + @exch_base_id
    INSERT ExceptionHistory_C(id, name, ctime) SELECT DISTINCT exch_id, CONVERT(VARCHAR(54), EXCP_NAME), @date
      FROM StagedEXCPDeletedHistory, StagedEXCPInProgress
      WHERE
        id = excp_id
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: created ExceptionHistory_C records for cleared exceptions', 0, @chkpt
    SET @chkpt = @chkpt + 1
    INSERT ExceptionHistory_A(id, ctime, _DATE, _TIME, _MSEC, _SEQ_NO, _EXCP_NAME, _EXCP_CD, _ParentID)
      SELECT
        exch_id, @date, TIME, TIME, DATEPART(ms,TIME), SEQ_NO, EXCP_NAME, EXCP_CD, excp_id
        FROM StagedEXCPDeletedHistory, StagedEXCPInProgress a
        WHERE
          id = excp_id
          AND NOT EXISTS (SELECT * FROM StagedEXCPInProgress a2 WHERE a.id = a2.id AND a2.timestamp > a.timestamp)
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: created ExceptionHistory_A records for cleared exceptions', 0, @chkpt
    SET @chkpt = @chkpt + 1
    -- Mark exception instances as deleted
    UPDATE Exception_C SET deleted = 1 FROM StagedEXCPDeletedHistory WHERE id = excp_id
    -- Send notifications of exception deletions
    EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: marked Exception_C records as deleted', 0, @chkpt
    SET @chkpt = @chkpt + 1
    INSERT PendingEvent(cid, id, type, fields, detail)
      SELECT 
        parent_cid,
        parent_id,
        type,
          CONVERT(VARCHAR(5), ISNULL(parent.cno, 0))          + '|' +
          CONVERT(VARCHAR(9), ISNULL(parent_id, 0))           + '|' +
          CONVERT(VARCHAR(9), ISNULL(excp_id, 0))             + '|' +
          CONVERT(VARCHAR(10), ISNULL(TIME, '1/1/1900'), 111) + '|' +
          CONVERT(VARCHAR(12), ISNULL(TIME, '1/1/1900'), 108) + '|' +
          CONVERT(VARCHAR(6), ISNULL(DATEPART(ms, TIME), 0))  + '|' +
          ISNULL(SEQ_NO, '')                                  + '|' +
          ISNULL(parent_cid, '')                              + '|' +
          ISNULL(OBJECT_NAME, '')   + '|' +
          ISNULL(NATIVE_KEY, '')                              + '|' +
          ISNULL(EXCP_OBJ_TYPE, '')                           + '|' +
          ISNULL(EXCP_OBJ_NAME, '')                           + '|' +
          ISNULL(EXCP_NAME, '')                               + '|' +
          CONVERT(VARCHAR(9), ISNULL(_nEXCH+1, 1))            + '|' +
          CONVERT(VARCHAR(9), ISNULL(AlertState, 0))          + '|' +
          CONVERT(VARCHAR(9), ISNULL(Priority, 0))            + '|' +
          '0', -- NumOwned, which is now a dummy value
        EXCP_CD
      FROM
        StagedEXCPDeletedHistory, StagedEXCPInProgress a, obj_class parent, EventType t, Exception_A e
      WHERE
        a.id = excp_id
        AND parent.cid = parent_cid
        AND t.name = 'ExceptionDeleted'
        AND e.id = excp_id
      ORDER BY a.timestamp
    EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: sent delete notifications', 0, @chkpt
    SET @chkpt = @chkpt + 1
   
    -- Remove the StagedEXCPInProgress records for the exceptions just deleted
    DELETE StagedEXCPInProgress FROM StagedEXCPDeletedHistory WHERE id = excp_id
    EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: removed StagedEXCPInProgress records for deleted exceptions', 0, @chkpt
    SET @chkpt = @chkpt + 1
  END

  IF @verbose = 1
  BEGIN
    SET NOCOUNT OFF
    SELECT 'AFTER_CLEARED_DELETED' AS 'AFTER_CLEARED_DELETED', * FROM StagedEXCPInProgress
    SET NOCOUNT ON
  END

  -- Update the ids of the StagedEXCPInProgress rows to the ids of previously generated
  -- exceptions that match them
  UPDATE StagedEXCPInProgress SET id = e.id FROM Exception_V e
  WHERE
    e._ParentCID = parent_cid
    AND e._ParentID = parent_id
    AND e._EXCP_NAME = EXCP_NAME
    AND e._AlertState = AlertState
    AND e._Priority = Priority
    AND e.deleted = 0
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: looked up ids of previous exceptions to be updated', 0, @chkpt
  SET @chkpt = @chkpt + 1

  IF @verbose = 1
  BEGIN
    SET NOCOUNT OFF
    SELECT 'AFTER_PREV_IDS_LOOKUP' AS 'AFTER_PREV_IDS_LOOKUP', * FROM StagedEXCPInProgress
    SET NOCOUNT ON
  END

  -- Any rows in StagedEXCPInProgress that have nonzero ids are updated to existing exceptions
  DECLARE @updated_count INT
  TRUNCATE TABLE StagedEXCPUpdatedHistory
  INSERT StagedEXCPUpdatedHistory(exch_id, excp_id) 
    SELECT 0, id FROM StagedEXCPInProgress WHERE id > 0
  SELECT @updated_count = @@ROWCOUNT
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: created table of updates to previous exceptions', 0, @chkpt
  SET @chkpt = @chkpt + 1
  IF @updated_count > 0
  BEGIN
    -- Create history records
    SELECT @date = GETDATE()
    EXEC @exch_base_id = alloc_EXCH @updated_count
    UPDATE StagedEXCPUpdatedHistory SET exch_id = row_id + @exch_base_id
/*--DEBUG
SELECT DISTINCT exch_id, count(*) FROM StagedEXCPUpdatedHistory group BY exch_id having count(*)>1
SELECT * FROM StagedEXCPUpdatedHistory WHERE exch_id IN (SELECT id FROM ExceptionHistory_C)
SELECT * FROM StagedEXCPUpdatedHistory
SELECT max(id) FROM ExceptionHistory_C
SELECT last_id FROM EXCH_ID
--DEBUG*/
    INSERT ExceptionHistory_C(id, name, ctime) SELECT DISTINCT exch_id, CONVERT(VARCHAR(54), EXCP_NAME), @date
      FROM StagedEXCPUpdatedHistory, StagedEXCPInProgress
      WHERE
        id = excp_id
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: inserted ExceptionHistory_C records for updated exceptions', 0, @chkpt
    SET @chkpt = @chkpt + 1

    INSERT ExceptionHistory_A(id, ctime, _DATE, _TIME, _MSEC, _SEQ_NO, _EXCP_NAME, _EXCP_CD, _ParentID)
      SELECT DISTINCT
        exch_id, @date, TIME, TIME, DATEPART(ms,TIME), SEQ_NO, EXCP_NAME, EXCP_CD, excp_id
        FROM StagedEXCPUpdatedHistory, StagedEXCPInProgress a
        WHERE
          id = excp_id
          AND NOT EXISTS (SELECT * FROM StagedEXCPInProgress a2 WHERE a.id = a2.id AND a2.timestamp > a.timestamp)
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: Inserted ExceptionHistory_A records for updated Exceptions', 0, @chkpt
    SET @chkpt = @chkpt + 1

    -- Update temporal attributes of exception
    UPDATE Exception_A SET
      _DATE    = TIME,
      _TIME    = TIME,
      _MSEC    = DATEPART(ms, TIME),
      _SEQ_NO  = SEQ_NO,
      _EXCP_CD = EXCP_CD,
      _nEXCH   = _nEXCH + (SELECT SUM(nEXCH) FROM StagedEXCPInProgress a2 WHERE a2.id = a.id)
    FROM
      StagedEXCPInProgress a
    WHERE 
      Exception_A.id = a.id
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: updated temporal attributes of previous exceptions', 0, @chkpt
    SET @chkpt = @chkpt + 1

    -- Send notifications of exception updates
    INSERT PendingEvent(cid, id, type, fields, detail)
      SELECT 
        parent_cid,
        parent_id,
        type,
          CONVERT(VARCHAR(5), ISNULL(parent.cno, 0))          + '|' +
          CONVERT(VARCHAR(9), ISNULL(parent_id, 0))           + '|' +
          CONVERT(VARCHAR(9), ISNULL(a.id, 0))                + '|' +
          CONVERT(VARCHAR(10), ISNULL(TIME, '1/1/1900'), 111) + '|' +
          CONVERT(VARCHAR(12), ISNULL(TIME, '1/1/1900'), 108) + '|' +
          CONVERT(VARCHAR(6), ISNULL(DATEPART(ms, TIME), 0))  + '|' +
          ISNULL(SEQ_NO, '')                                  + '|' +
          ISNULL(parent_cid, '')                              + '|' +
          ISNULL(OBJECT_NAME, '')                             + '|' +
          ISNULL(NATIVE_KEY, '')                              + '|' +
          ISNULL(EXCP_OBJ_TYPE, '')                           + '|' +
          ISNULL(EXCP_OBJ_NAME, '')                           + '|' +
          ISNULL(EXCP_NAME, '')                               + '|' +
          '1' /* nEXCH: always 1 for updates */               + '|' +
          CONVERT(VARCHAR(9), ISNULL(AlertState, 0))          + '|' +
          CONVERT(VARCHAR(9), ISNULL(Priority, 0))            + '|' +
          '0', -- NumOwned, which is now a dummy value
        EXCP_CD
      FROM
        StagedEXCPInProgress a, obj_class parent, EventType t, Exception_A e
      WHERE
        e.id = a.id
        AND parent.cid = parent_cid
        AND t.name = 'ExceptionUpdated'
      ORDER BY a.timestamp
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRANSACTION asisp_processStagedEXCPs
      RETURN -30000
    END
    EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: sent update notifications', 0, @chkpt
    SET @chkpt = @chkpt + 1

    -- Remove the StagedEXCPInProgress records for the exceptions just updated
    DELETE StagedEXCPInProgress WHERE id > 0
    EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: removed StagedEXCPInProgress records for updated exceptions', 0, @chkpt
    SET @chkpt = @chkpt + 1
  END

  IF @verbose = 1
  BEGIN
    SET NOCOUNT OFF
    SELECT 'AFTER_UPDATED_REMOVED' AS 'AFTER_UPDATED_REMOVED', * FROM StagedEXCPInProgress
    SET NOCOUNT ON
  END

  -- Remove any exceptions with 'Eliminate' priority
  DELETE StagedEXCPInProgress WHERE Priority=6
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: removed records for ''eliminate'' Priority exceptions ', 0, @chkpt
  SET @chkpt = @chkpt + 1

  -- We now only want to retain the most recent observation of an exception, but its nEXCH column
  -- has to reflect the SUM of all observations

  UPDATE StagedEXCPInProgress SET nEXCH = 
    (
      SELECT SUM(nEXCH) 
      FROM StagedEXCPInProgress e2
      WHERE 
        e2.parent_cid = StagedEXCPInProgress.parent_cid
        AND e2.parent_id = StagedEXCPInProgress.parent_id
        AND e2.EXCP_NAME = StagedEXCPInProgress.EXCP_NAME
        AND e2.AlertState = StagedEXCPInProgress.AlertState
        AND e2.Priority = StagedEXCPInProgress.Priority
        AND e2.timestamp <= StagedEXCPInProgress.timestamp
    )
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: updated count of exceptions', 0, @chkpt
  SET @chkpt = @chkpt + 1

  -- Remove duplicates, leaving us with only the exceptions that should be inserted
  DELETE StagedEXCPInProgress WHERE EXISTS
  (
    SELECT *
    FROM StagedEXCPInProgress e2
    WHERE 
      e2.parent_cid = StagedEXCPInProgress.parent_cid
      AND e2.parent_id = StagedEXCPInProgress.parent_id
      AND e2.EXCP_NAME = StagedEXCPInProgress.EXCP_NAME
      AND e2.AlertState = StagedEXCPInProgress.AlertState
      AND e2.Priority = StagedEXCPInProgress.Priority
      AND e2.timestamp > StagedEXCPInProgress.timestamp
  )
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: removed duplicate records', 0, @chkpt
  SET @chkpt = @chkpt + 1

  DECLARE @excp_base_id ObjID, @inserted_count INT
  SELECT @inserted_count = count(*) FROM StagedEXCPInProgress
  IF @inserted_count = 0
  BEGIN
    IF @tcount = 0
      COMMIT TRANSACTION asisp_processStagedEXCPs
    RETURN @num_events_processed
  END
  SELECT @msg = 'asisp_processStagedEXCPs: allocating ' + CAST(@inserted_count AS VARCHAR) + ' exception id' +
    CASE @inserted_count WHEN 1 THEN '' ELSE 's' END
  EXEC asisp_checkpointmessage @msg, 0, @chkpt
  SET @chkpt = @chkpt + 1

  -- Allocate ids for new exceptions    
  EXEC @excp_base_id = alloc_EXCP @inserted_count
  TRUNCATE TABLE StagedEXCPInserted
  INSERT StagedEXCPInserted(excp_timestamp) SELECT timestamp FROM StagedEXCPInProgress
  UPDATE StagedEXCPInProgress SET id = @excp_base_id + row_id FROM StagedEXCPInserted
    WHERE excp_timestamp = timestamp
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: allocated ids', 0, @chkpt
  SET @chkpt = @chkpt + 1

  IF @verbose = 1
  BEGIN
    SET NOCOUNT OFF
    SELECT 'TO_BE_INSERTED' AS 'TO_BE_INSERTED', * FROM StagedEXCPInProgress
    SET NOCOUNT ON
  END

  -- Insert Exception class records
  INSERT Exception_C(id, name) 
    SELECT id, CONVERT(VARCHAR(54), EXCP_NAME)
    FROM StagedEXCPInProgress
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: inserted Exception_C records', 0, @chkpt
  SET @chkpt = @chkpt + 1
  -- Insert Exception attribute records
  INSERT Exception_A
  (
    id, 
    _DATE, 
    _TIME, 
    _MSEC, 
    _SEQ_NO, 
    _OBJECT_NAME, 
    _NATIVE_KEY, 
    _EXCP_OBJ_TYPE, 
    _EXCP_OBJ_NAME,
    _EXCP_NAME,
    _EXCP_CD,
    _nEXCH,
    _AlertState,
    _Priority,
    _ParentCID,
    _ParentID
  )
    SELECT
      id,
      TIME,
      TIME,
      DATEPART(ms,TIME),
      SEQ_NO,
      OBJECT_NAME,
      NATIVE_KEY,
      EXCP_OBJ_TYPE,
      EXCP_OBJ_NAME,
      EXCP_NAME,
      EXCP_CD,
      nEXCH,
      AlertState,
      Priority,
      parent_cid,
      parent_id
    FROM
      StagedEXCPInProgress
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: inserted Exception_A records', 0, @chkpt
  SET @chkpt = @chkpt + 1

  -- Create history records
  SELECT @date = GETDATE()
  TRUNCATE TABLE StagedEXCPInsertedHistory
  INSERT StagedEXCPInsertedHistory(exch_id, excp_id) 
    SELECT 0, id FROM StagedEXCPInProgress
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: inserted StagedEXCPInsertedHistory records', 0, @chkpt
  SET @chkpt = @chkpt + 1
  EXEC @exch_base_id = alloc_EXCH @inserted_count
  UPDATE StagedEXCPInsertedHistory SET exch_id = row_id + @exch_base_id
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: updated ids in StagedEXCPInsertedHistory', 0, @chkpt
  SET @chkpt = @chkpt + 1
  INSERT ExceptionHistory_C(id, name, ctime) SELECT exch_id, CONVERT(VARCHAR(54), EXCP_NAME), @date
    FROM StagedEXCPInsertedHistory, StagedEXCPInProgress
    WHERE
      id = excp_id
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: inserted ExceptionHistory_C records for new exceptions', 0, @chkpt
  SET @chkpt = @chkpt + 1
  INSERT ExceptionHistory_A(id, ctime, _DATE, _TIME, _MSEC, _SEQ_NO, _EXCP_NAME, _EXCP_CD, _ParentID)
    SELECT
      exch_id, @date, TIME, TIME, DATEPART(ms,TIME), SEQ_NO, EXCP_NAME, EXCP_CD, excp_id
      FROM StagedEXCPInsertedHistory, StagedEXCPInProgress
      WHERE
        id = excp_id
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: inserted ExceptionHistory_A records for new exceptions', 0, @chkpt
  SET @chkpt = @chkpt + 1

  -- Send notifications of exception inserts
  INSERT PendingEvent(cid, id, type, fields, detail)
    SELECT 
      parent_cid,
      parent_id,
      type,
        CONVERT(VARCHAR(5), ISNULL(parent.cno, 0))          + '|' +
        CONVERT(VARCHAR(9), ISNULL(parent_id, 0))           + '|' +
        CONVERT(VARCHAR(9), ISNULL(a.id, 0))                + '|' +
        CONVERT(VARCHAR(10), ISNULL(TIME, '1/1/1900'), 111) + '|' +
        CONVERT(VARCHAR(12), ISNULL(TIME, '1/1/1900'), 108) + '|' +
        CONVERT(VARCHAR(6), ISNULL(DATEPART(ms, TIME), 0))  + '|' +
        ISNULL(SEQ_NO, '')                                  + '|' +
        ISNULL(parent_cid, '')                              + '|' +
        ISNULL(OBJECT_NAME, '')                             + '|' +
        ISNULL(NATIVE_KEY, '')                              + '|' +
        ISNULL(EXCP_OBJ_TYPE, '')                           + '|' +
        ISNULL(EXCP_OBJ_NAME, '')                           + '|' +

        ISNULL(EXCP_NAME, '')                               + '|' +
        CONVERT(VARCHAR(9), ISNULL(nEXCH, 0))               + '|' +
        CONVERT(VARCHAR(9), ISNULL(AlertState, 0))          + '|' +
        CONVERT(VARCHAR(9), ISNULL(Priority, 0))            + '|' +
        '0', -- NumOwned, which is now a dummy value
      EXCP_CD
    FROM
      StagedEXCPInProgress a, obj_class parent, EventType t
    WHERE
      parent.cid = parent_cid
      AND t.name = 'ExceptionInserted'
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedEXCPs
    RETURN -30000
  END
  EXEC asisp_checkpointmessage 'asisp_processStagedEXCPs: sent insert notifications', 0, @chkpt
  SET @chkpt = @chkpt + 1

  IF @tcount = 0
    COMMIT TRANSACTION asisp_processStagedEXCPs

  RETURN @num_events_processed
GO


/*

-- Exceptions can now be cleared by name by generating a clearing exception where the AlertState and/or Priority
-- is set to -1. Whichever field is set to -1 will be treated as a wildcard when looking up the exception to be
-- cleared. The text of the clearing exception must be 'OK'.

-- Example:
--
-- The following block of code will create two schedule violations on CICS 1. One is 
-- created with Yellow/Critical AlertState/Priority, and one with Red/Critical.

DECLARE @d DATETIME 
-- Yellow Critical schedule violation
SELECT @d = CURRENT_TIMESTAMP
EXEC asisp_createEXCP SCHV, CICS, 1, @d, @d, 0, 'MVSA1', 'CICS', 'CICSA', '0000010008', @EXCP_NAME='SCHV', @EXCP_CD='Yellow Critical schedule violation', @AlertState=2, @Priority=1
-- Red Critical schedule violation
SELECT @d = CURRENT_TIMESTAMP
EXEC asisp_createEXCP SCHV, CICS, 1, @d, @d, 0, 'MVSA1', 'CICS', 'CICSA', '0000010008', @EXCP_NAME='SCHV', @EXCP_CD='Red Critical schedule violation', @AlertState=3, @Priority=1


-- Allow the two exceptions above to be processed before proceeding

-- The following block of code will generate a new style clear for SCHV
-- Note that the AlertState and Priority are both specified as -1.
-- This will clear both of the schedule violations previously generated.

DECLARE @d DATETIME
SELECT @d = CURRENT_TIMESTAMP
EXEC asisp_createEXCP SCHV, CICS, 1, @d, @d, 0, 'MVSA1', 'CICS', 'CICSA', '0000010008', @EXCP_NAME='SCHV', @EXCP_CD='OK', @AlertState=-1, @Priority=-1

*/
