-- $Header:   //ARCHIVE/Archive/DB/sql/asisp_processStagedOPCEvents.sqv   1.1   Feb 05 2001 18:20:10   BRENT  $
--
-- asisp_processStagedOPCEvents.sql

USE Object
GO

---------------------------------------------------------------------------------------------------
-- asisp_processStagedOPCEvents
PRINT 'Creating procedure asisp_processStagedOPCEvents'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_processStagedOPCEvents' AND type = 'P')
  DROP PROCEDURE asisp_processStagedOPCEvents
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/asisp_processStagedOPCEvents.sqv  $
-- $Revision:   1.1  $
-- $Date:   Feb 05 2001 18:20:10  $
-- $Author:   BRENT  $
-- Processes the records in the StagedOPCEvent table
CREATE PROCEDURE asisp_processStagedOPCEvents
  @maxrows INT = 0,      -- maximum number of rows to process. 0=process all rows
  @verbose TINYINT = 0   -- generate progress output during execution
AS
  DECLARE @err INT, @rc INT, @step INT
  SET @step = 1

  SET ROWCOUNT @maxrows

  IF @verbose = 1
    PRINT 'asisp_processStagedOPCEvents: Step ' + CAST(@step AS VARCHAR(9)) + ': creating #StagedOPCInProgress'
  SET @step = @step + 1

  CREATE TABLE #StagedOPCInProgress
  (
    timestamp       BINARY(8) PRIMARY KEY,
    ctime           DATETIME,
    src_cid         ClassID,
    src_id          ObjID,
    TIME            DATETIME NULL,
    SEQ_NO          VARCHAR(50) NULL,
    MESSAGE         VARCHAR(255) NULL,
    MESSAGE_TEXT    VARCHAR(255) NULL,
    APPID           VARCHAR(16) NULL,
    WKSTNID         VARCHAR(8) NULL,
    OPID            VARCHAR(10) NULL,
    JOBNAME         VARCHAR(8) NULL,
    PRTY            INT NULL,
    IA              VARCHAR(16) NULL,
    EC              VARCHAR(8) NULL,
    RESOURCE        VARCHAR(40) NULL,
    WAIT_MINUTES    INT NULL,
    app_id          ObjID NULL,
    job_cid         ClassID NULL,
    job_id          ObjID NULL,
    EventCID        ClassID NULL,
    EventText       VARCHAR(255) NULL,
    AlertStateID    INT NULL,
    PriorityID      INT NULL,
    StateID         INT NULL,
    op_id           INT IDENTITY(1,1)
  )

  DECLARE @nEvents INT
  INSERT #StagedOPCInProgress SELECT *, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL FROM StagedOPCEvent
  SELECT @nEvents = @@ROWCOUNT

  IF @verbose = 1
  BEGIN
    PRINT 'asisp_processStagedOPCEvents: Step ' + CAST(@step AS VARCHAR(9)) + ': Inserted ' + CAST(@nEvents AS VARCHAR(9)) + ' row(s)' 
    SELECT @step AS Step, * FROM #StagedOPCInProgress ORDER BY timestamp
  END
  SET @step = @step + 1

  SET ROWCOUNT 0

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_processStagedOPCEvents
  ELSE
    SAVE TRANSACTION asisp_processStagedOPCEvents

  IF @verbose = 1
    PRINT 'asisp_processStagedOPCEvents: Step ' + CAST(@step AS VARCHAR(9)) + ': looking up batch schedule sets'
  SET @step = @step + 1

  -- Find the application by looking back through the ancestory of the source object (usually OS) 
  -- for a batch schedule set that contains a batch schedule with the specified name
  DECLARE appc INSENSITIVE CURSOR FOR SELECT DISTINCT src_cid, src_id, APPID FROM #StagedOPCInProgress
  OPEN appc
  DECLARE @src_cid ClassID, @src_id ObjID, @APPID VARCHAR(16), @app_id ObjID
  FETCH NEXT FROM appc INTO @src_cid, @src_id, @APPID
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    IF @verbose = 1
      PRINT 'asisp_processStagedOPCEvents: Looking up ' + @src_cid + ' ' + CAST(@src_id AS VARCHAR(9)) + ' ' + @APPID
    SELECT @app_id = NULL
    WHILE @app_id IS NULL AND @src_cid <> 'ROOT'
    BEGIN
      SELECT @app_id = bcys_btcy.dst_id
      FROM
        containment AS bcys_btcy
          JOIN BatchCycle_C AS btcy ON btcy.id = bcys_btcy.dst_id AND bcys_btcy.src_cid = 'BCYS' AND bcys_btcy.dst_cid = 'BTCY'
          JOIN containment AS src_bcys ON src_bcys.dst_cid = 'BCYS' AND src_bcys.dst_id = bcys_btcy.src_id
      WHERE
        src_bcys.src_cid = @src_cid
        AND src_bcys.src_id = @src_id
        AND btcy.name = @APPID
        AND bcys_btcy.deleted = 0
        AND src_bcys.deleted = 0
      -- If we didn't find the application then go up to the next level
      IF @app_id IS NULL
      BEGIN
        SELECT @src_cid = src_cid, @src_id = src_id 
        FROM containment 
        WHERE dst_cid = @src_cid AND dst_id = @src_id
        IF @@ROWCOUNT = 0
        BEGIN
          RAISERROR('Invalid object: cid=%s id=%d. Discarding all such records', 3, 1, @src_cid, @src_id)
          DELETE #StagedOPCInProgress WHERE src_cid = @src_cid AND src_id = @src_id
          BREAK
      END
      END
      ELSE
        UPDATE #StagedOPCInProgress 
        SET app_id = @app_id 
        WHERE src_cid = @src_cid AND src_id = @src_id AND APPID = @APPID
      IF @verbose = 1
        PRINT '  src_cid=' + @src_cid + ' src_id=' + CAST(@src_id AS VARCHAR(9)) + ' app_id=' + CAST(@app_id AS VARCHAR(9))
    END
    FETCH NEXT FROM appc INTO @src_cid, @src_id, @APPID
  END
  DEALLOCATE appc

  IF @verbose = 1
    PRINT 'asisp_processStagedOPCEvents: Step ' + CAST(@step AS VARCHAR(9)) + ': checking for new applications'
  SET @step = @step + 1

  -- If we get to this point and there are still applications that have not been found
  -- then we create them under the OS.
  IF EXISTS (SELECT * FROM #StagedOPCInProgress WHERE app_id IS NULL)
  BEGIN
    IF @verbose = 1
      PRINT 'asisp_processStagedOPCEvents: Creating Batch Schedule Sets for OSes'
    -- There has to be a Batch Schedule Set under each OS with the name of the OS
    CREATE TABLE #bcys
    (
      os_id   ObjID,
      bcys_id ObjID NULL
    )
    INSERT #bcys(os_id, bcys_id)
      SELECT DISTINCT s.src_id, NULL
      FROM #StagedOPCInProgress AS s
    IF @verbose = 1
      SELECT 'BCYS PRE-UPDATE', * FROM #bcys
    UPDATE b SET bcys_id = dst_id 
      FROM #bcys AS b JOIN containment AS c ON c.dst_cid = 'BCYS' AND c.src_cid = 'OS' AND c.src_id = os_id
        JOIN BatchCycles_C as bcys ON bcys.id = c.dst_id
        JOIN OS_C as os ON os.id = os_id
      WHERE
        c.deleted = 0
        AND bcys.name = os.name
    IF @verbose = 1
      SELECT 'BCYS POST-UPDATE', * FROM #bcys
    DECLARE @nNewBCYS INT
    SELECT @nNewBCYS = COUNT(*) FROM #bcys WHERE bcys_id IS NULL
    IF @nNewBCYS > 0
    BEGIN
      IF @verbose = 1
        PRINT 'asisp_processStagedOPCEvents: Creating ' + CAST(@nNewBCYS AS VARCHAR(9)) + ' Batch schedule sets'
      DECLARE @base_id ObjID
      EXEC @base_id = alloc_BCYS @nNewBCYS
      CREATE TABLE #new_bcys ( os_id ObjID, rownum INT IDENTITY(0,1) )
      IF @verbose = 1
        PRINT 'asisp_processStagedOPCEvents: Creating #new_bycs'
      INSERT #new_bcys(os_id) SELECT DISTINCT os_id FROM #bcys WHERE bcys_id IS NULL
      IF @@ERROR<>0
      BEGIN
        RETURN -30000
      END
      IF @verbose = 1
        SELECT 'NEW BCYS' AS 'NEW BCYS', * FROM #new_bcys
      IF @verbose = 1
        PRINT 'asisp_processStagedOPCEvents: INSERT BatchCycles_C'
      INSERT BatchCycles_C(id,name) SELECT @base_id+rownum, os.name FROM #new_bcys JOIN OS_C AS os ON os.id = os_id
      IF @verbose = 1
        PRINT 'asisp_processStagedOPCEvents: INSERT BatchCycles_A'
      INSERT BatchCycles_A(id,_ChildEventMatrixID,_ExceptionMatrixID) SELECT @base_id+rownum, 0, 0 FROM #new_bcys
      IF @verbose = 1
        PRINT 'asisp_processStagedOPCEvents: INSERT OS_BCYS_PHYC'
      INSERT OS_BCYS_PHYC(src_id, dst_id) SELECT nb.os_id, @base_id+rownum FROM #new_bcys AS nb
      IF @verbose = 1
        PRINT 'asisp_processStagedOPCEvents: INSERT PendingNotification'
      INSERT PendingNotification(type,fields,detail)
        SELECT 
          et.type,
          CONVERT(VARCHAR(5), c.cno) + '|' +
          CONVERT(VARCHAR(9), @base_id+rownum) + '|' +
          'BCYS' + '|' +
          '|' +
          'PHYC' + '|' +
          CONVERT(VARCHAR(5), p.cno) + '|' +
          CONVERT(VARCHAR(9), nb.os_id),
          os.name
        FROM
          EventType AS et,
          #new_bcys AS nb
            JOIN obj_class AS p ON p.cid = 'OS'
            JOIN obj_class AS c ON c.cid = 'BCYS'
            JOIN OS_C AS os ON os.id = nb.os_id
        WHERE
          et.name = 'ObjectCreated'
      IF @verbose = 1
      BEGIN
        PRINT '  UPDATE b SET b.bcys_id = @base_id + nb.rownum'
        PRINT '    FROM #bcys AS b JOIN #new_bcys AS nb ON nb.os_id = b.os_id'
      END     
      UPDATE b SET b.bcys_id = @base_id + nb.rownum
        FROM #bcys AS b JOIN #new_bcys AS nb ON nb.os_id = b.os_id
    END

    -- Now create the Batch Schedules for the applications
    CREATE TABLE #btcy
    (
      src_cid    ClassID,
      src_id     ObjID,
      APPID      VARCHAR(16),
      bcys_id    ObjID,
      rownum     INT IDENTITY(0,1)
    )
    INSERT #btcy(src_cid, src_id, APPID, bcys_id)
      SELECT DISTINCT src_cid, src_id, APPID, bcys_id
      FROM #StagedOPCInProgress AS s JOIN #bcys AS b ON b.os_id = s.src_id
      WHERE app_id IS NULL
    SET @rc = @@ROWCOUNT
    IF @verbose = 1
      SELECT 'BTCY', * FROM #btcy
    IF @rc > 0
    BEGIN
      DECLARE @nNewBTCY INT
      SELECT @nNewBTCY = COUNT(*) FROM #btcy
      EXEC @base_id = alloc_BTCY @nNewBTCY
      INSERT BatchCycle_C(id, name) SELECT @base_id + rownum, APPID FROM #btcy
      INSERT BatchCycle_A(id, _ChildEventMatrixID, _ExceptionMatrixID) SELECT @base_id + rownum, 0, 0 FROM #btcy
      INSERT BCYS_BTCY_AGGC(src_id, dst_id) SELECT bcys_id, @base_id+rownum FROM #btcy
      INSERT PendingNotification(type,fields,detail)
        SELECT 
          et.type,
          CONVERT(VARCHAR(5), c.cno) + '|' +
          CONVERT(VARCHAR(9), @base_id+rownum) + '|' +
          'BTCY' + '|' +
          '|' +
          'PHYC' + '|' +
          CONVERT(VARCHAR(5), p.cno) + '|' +
          CONVERT(VARCHAR(9), nb.bcys_id),
          nb.APPID
        FROM
          EventType AS et,
          #btcy AS nb
            JOIN obj_class AS p ON p.cid = 'BCYS'
            JOIN obj_class AS c ON c.cid = 'BTCY'
        WHERE
          et.name = 'ObjectCreated'
      UPDATE s SET app_id = @base_id+rownum
        FROM #StagedOPCInProgress AS s 
          JOIN #btcy AS b ON s.src_cid = b.src_cid AND s.src_id = b.src_id AND s.APPID = b.APPID
    END
  END

  IF EXISTS (SELECT * FROM #StagedOPCInProgress WHERE app_id IS NULL)
  BEGIN
    RAISERROR('asisp_processStagedOPCEvents: Still have staged events with null app_id, aborting!', 3, 1)
    IF @verbose = 1
      SELECT 'NULL APPID' AS 'NULL APPID', * FROM #StagedOPCInProgress WHERE app_id IS NULL
    ROLLBACK TRAN asisp_processStagedOPCEvents
    RETURN -30000
  END

  IF @verbose = 1
    PRINT 'asisp_processStagedOPCEvents: Step ' + CAST(@step AS VARCHAR(9)) + ': looking up job ids'
  SET @step = @step + 1
--SELECT * FROM #StagedOPCInProgress

  -- For all applications that have been successfully looked up we now do the same with the jobs
  UPDATE s SET 
    job_cid = 'BATC',
    job_id  = b.id
  FROM
    #StagedOPCInProgress AS s
      JOIN BATCH_V AS b ON 
        b._ParentCID = 'BTCY' 
        AND b._ParentID = s.app_id 
        AND b.name = s.JOBNAME
        AND DATEPART(YEAR, b._SchedStartTime) % 100 = CAST(SUBSTRING(s.IA, 1, 2) AS INT)
        AND DATEPART(MONTH, b._SchedStartTime)      = CAST(SUBSTRING(s.IA, 3, 2) AS INT)
        AND DATEPART(DAY, b._SchedStartTime)        = CAST(SUBSTRING(s.IA, 5, 2) AS INT)
        AND DATEPART(HOUR, b._SchedStartTime)       = CAST(SUBSTRING(s.IA, 7, 2) AS INT)
        AND DATEPART(MINUTE, b._SchedStartTime)     = CAST(SUBSTRING(s.IA, 9, 2) AS INT)
  WHERE
    s.app_id IS NOT NULL

  IF @verbose = 1
    PRINT 'asisp_processStagedOPCEvents: Step ' + CAST(@step AS VARCHAR(9)) + ': creating any new jobs required'
  SET @step = @step + 1

  -- Create any new batch jobs required.
  DECLARE @nNewBATC INT  
  SELECT @nNewBATC = COUNT(*) FROM #StagedOPCInProgress WHERE job_cid IS NULL
  IF @nNewBATC > 0
  BEGIN
    CREATE TABLE #batc
    (
      btcy_id   ObjID,
      jobname   VARCHAR(8),
      IA        VARCHAR(16) NULL,
      rownum    INT IDENTITY(0,1)
    )
    INSERT #batc(btcy_id, jobname, IA) SELECT DISTINCT app_id, JOBNAME, IA FROM #StagedOPCInProgress 
      WHERE JOBNAME IS NOT NULL AND job_cid IS NULL
--SELECT 'BATC', * FROM #batc
    EXEC @base_id = alloc_BATC @nNewBATC
    INSERT BATCH_C(id, name, description, _ChildEventMatrixID, _ExceptionMatrixID, _ConsoleTrapStatus, _SysplexResource, _ParentCID, _ParentID, _SchedStartTime)
      SELECT
        @base_id+rownum,
        jobname,
        'AUTO-CREATED FROM MESSAGE AT ' + CONVERT(VARCHAR(40), GETDATE(), 120),
        0 AS _ChildEventMatrixID,
        0 AS _ExceptionMatrixID,
        0 AS _ConsoleTrapStatus,
        0 AS _SysplexResource,
        'BTCY' AS _ParentCID,
        btcy_id AS _ParentID,
        CAST(
         '20' + SUBSTRING(IA, 1, 2) + '-' + 
          SUBSTRING(IA, 3, 2) + '-' +
          SUBSTRING(IA, 5, 2) + ' ' +
          SUBSTRING(IA, 7, 2) + ':' +
          SUBSTRING(IA, 9, 2)
        AS DATETIME) AS _SchedStartTime
      FROM #batc
    INSERT BTCY_BATC_PHYC(src_id, dst_id)
      SELECT btcy_id, @base_id+rownum FROM #batc
    INSERT PendingNotification(type,fields,detail)
      SELECT 
        et.type,
        CONVERT(VARCHAR(5), c.cno) + '|' +
        CONVERT(VARCHAR(9), @base_id+rownum) + '|' +
        'BATC' + '|' +
        '|' +
        'PHYC' + '|' +
        CONVERT(VARCHAR(5), p.cno) + '|' +
        CONVERT(VARCHAR(9), nb.btcy_id),
        nb.jobname
      FROM
        EventType AS et,
        #batc AS nb
          JOIN obj_class AS p ON p.cid = 'BTCY'
          JOIN obj_class AS c ON c.cid = 'BATC'
      WHERE
        et.name = 'ObjectCreated'
    UPDATE s SET job_cid = 'BATC', job_id = @base_id+rownum
      FROM #StagedOPCInProgress AS s 
        JOIN #batc AS b ON s.app_id = b.btcy_id AND b.jobname = s.JOBNAME 
      WHERE s.job_cid IS NULL
  END

  IF @verbose = 1
    PRINT 'asisp_processStagedOPCEvents: Step ' + CAST(@step AS VARCHAR(9)) + ': Creating events'
  SELECT @step AS step, * FROM #StagedOPCInProgress
  SET @step = @step + 1

  -- Now that all objects have been found/created we use the OPCEventTypeMap table to generate
  -- events of the appropriate type.
  DECLARE etmc INSENSITIVE CURSOR FOR
    SELECT DISTINCT CAST(m.id AS VARCHAR(9)), m.EventName, m.WhereClause, m.MapPriority
    FROM OPCEventTypeMap AS m JOIN #StagedOPCInProgress AS s ON s.MESSAGE LIKE m.EventName
    WHERE m.Enabled = 1
    ORDER BY m.MapPriority DESC
  OPEN etmc
  DECLARE @MapId VARCHAR(9), @MapEventName VARCHAR(16), @MapWhereClause VARCHAR(500), @MapPriority INT
  FETCH NEXT FROM etmc INTO @MapId, @MapEventName, @MapWhereClause, @MapPriority
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    PRINT 'asisp_processStagedOPCEvents: Checking OPCEventTypeMap ' + CAST(@MapId AS VARCHAR) + ': EventName=''' + @MapEventName + ''''
    IF @MapWhereClause <> '' SET @MapWhereClause = ' AND (' + @MapWhereClause + ')'
    EXEC("
    UPDATE s SET 
      s.EventCID = m.EventCID,
      s.EventText = CASE WHEN m.EventTextPat IS NULL THEN s.MESSAGE_TEXT ELSE m.EventTextPat END,
      s.AlertStateID = m.AlertStateID,
      s.PriorityID = m.PriorityID,
      s.StateID = m.StateID
    FROM
      #StagedOPCInProgress AS s,
      OPCEventTypeMap as m
    WHERE
      s.MESSAGE LIKE '" + @MapEventName + "' AND s.EventCID IS NULL AND m.id = " + @MapId + @MapWhereClause
    )
    FETCH NEXT FROM etmc INTO @MapId, @MapEventName, @MapWhereClause, @MapPriority
  END
  DEALLOCATE etmc

  IF @verbose = 1
    PRINT 'asisp_processStagedOPCEvents: Step ' + CAST(@step AS VARCHAR(9))
  SET @step = @step + 1
--SELECT * FROM #StagedOPCInProgress

  PRINT 'asisp_processStagedOPCEvents: Inserting StagedEXCP records'
  INSERT StagedEXCP(parent_cid, parent_id, TIME, SEQ_NO, OBJECT_NAME, NATIVE_KEY, EXCP_NAME, EXCP_CD, AlertState, Priority, nEXCH)
    SELECT job_cid, job_id, TIME, SEQ_NO, JOBNAME, '', MESSAGE, ISNULL(EventText,''), AlertStateID, PriorityID, 1
    FROM #StagedOPCInProgress
    WHERE EventCID = 'EXCP'
    ORDER BY timestamp
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedOPCEvents
    RETURN -30000
  END
  PRINT 'asisp_processStagedOPCEvents: Inserted ' + CAST(@rc AS VARCHAR(9)) + ' StagedEXCP records'

  IF @verbose = 1
    PRINT 'Step ' + CAST(@step AS VARCHAR(9))
  SET @step = @step + 1

  PRINT 'asisp_processStagedOPCEvents: Inserting StagedMESG records'
  INSERT StagedMESG(parent_cid, parent_id, TIME, SEQ_NO, OBJECT_NAME, NATIVE_KEY, MESSAGE, STATE, REASON_CD, AlertState, Priority, StateID, DISCOVERY_IND)
    SELECT job_cid, job_id, TIME, SEQ_NO, JOBNAME, '', MESSAGE, '', ISNULL(EventText,''), AlertStateID, PriorityID, StateID, 'MON'
    FROM #StagedOPCInProgress
    WHERE EventCID = 'MESG' AND job_id IS NOT NULL
    ORDER BY timestamp
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedOPCEvents
    RETURN -30000
  END
  PRINT 'asisp_processStagedOPCEvents: Inserted ' + CAST(@rc AS VARCHAR(9)) + ' StagedMESG records'

  IF @verbose = 1
    PRINT 'asisp_processStagedOPCEvents: Step ' + CAST(@step AS VARCHAR(9))
  SET @step = @step + 1

  IF @tcount = 0
    COMMIT TRANSACTION asisp_processStagedOPCEvents

  DELETE s FROM StagedOPCEvent AS s JOIN #StagedOPCInProgress AS p ON s.timestamp = p.timestamp

  RETURN @nEvents
GO
