-- $Header:   P:/Archive/DB/SQL/asisp_resetclass.sqv   1.0   Dec 04 2000 13:59:06   BRENT  $
--
-- asisp_resetclass.sql
--
-- Defines the asisp_resetclass procedure. 
-- 
-- This revision checks for the existence of the <cid>_ID table before attempting to reset it.
--
-- This archive is temporary, and should not be maintained beyond acceptance.

USE Object
GO

---------------------------------------------------------------------------------------------------
-- asisp_resetclass
PRINT 'Creating procedure asisp_resetclass'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_resetclass' AND type = 'P')
  DROP PROCEDURE asisp_resetclass
GO

-- $Archive:   P:/Archive/DB/SQL/asisp_resetclass.sqv  $
-- $Revision:   1.0  $
-- $Date:   Dec 04 2000 13:59:06  $
-- $Author:   BRENT  $
-- Removes all instances of a specified class.
create procedure asisp_resetclass
  @cid    ClassID,
  @cname  sysname = NULL
as
  set nocount on

  IF @cname IS NULL
    SELECT @cname = cname FROM obj_class WHERE cid = @cid
  IF @cname IS NULL
  BEGIN
    RAISERROR('asisp_resetclass: %s is not a valid class id', 3, 1, @cid)
    RETURN -30000
  END

  declare @sql varchar(255), @time_str varchar(32),
          @link_type varchar(4), @src_cid ClassID, @dst_cid ClassID,
          @err int, @rc int

  -- See if the class has its own reset proc
  select @sql = name from sysobjects where type = 'P' and name = 'asisp_reset' + @cid
  if @@rowcount = 1
  begin
    select @time_str = convert(varchar(32), getdate(), 14)
    exec(@sql)
  end

  -- Update the deletion attributes for LOBs that are linked to objects of this class
  select @sql = 'update lob set _DeletedCID = ''' + @cid + ''', _DeletedID = o.id, _DeletedName = o.name, _DeletedTime = getdate() ' +
                'from lob_link as l ' + 
                '  join LineOfBusiness_V as lob on lob.id = l.dst_id ' +
                '  join ' + @cname + '_V as o on l.phy_cid = ''' + @cid + ''' and o.id = l.phy_id '
  select @time_str = convert(varchar(32), getdate(), 14)
  raiserror('%s: %s', 0, 1, @time_str, @sql)
  exec(@sql)
  select @err = @@error, @rc = @@rowcount
  if @err = 0
  begin
    select @time_str = convert(varchar(32), getdate(), 14)
    raiserror('%s: updated %d row(s)', 0, 1, @time_str, @rc)
  end

  -- Delete the link table rows
  exec asisp_deletebycid 'containment', 'src_cid', @cid
  exec asisp_deletebycid 'containment', 'dst_cid', @cid
  exec asisp_deletebycid 'link', 'src_cid', @cid
  exec asisp_deletebycid 'link', 'dst_cid', @cid
  checkpoint
  dump tran Object with no_log

  -- Truncate the class and attribute tables
  select @sql = 'delete ' + @cname + '_C'
  select @time_str = convert(varchar(32), getdate(), 14)
  raiserror('%s: %s', 0, 1, @time_str, @sql)
  exec(@sql)
  select @sql = 'delete ' + @cname + '_A'
  select @time_str = convert(varchar(32), getdate(), 14)
  raiserror('%s: %s', 0, 1, @time_str, @sql)
  exec(@sql)

  -- Reset the id table
  IF EXISTS (SELECT * FROM sysobjects WHERE name = RTRIM(@cid) + '_ID' AND type = 'U')
  BEGIN
    select @sql = 'update ' + RTRIM(@cid) + '_ID set last_id=0'
    select @time_str = convert(varchar(32), getdate(), 14)
    raiserror('%s: %s', 0, 1, @time_str, @sql)
    exec(@sql)
  END

  -- Truncate the link tables
  declare lc cursor for 
    select link_type, src_cid, dst_cid from derived_obj_link 
      join obj_class as src on src.cid = src_cid
      join obj_class as dst on dst.cid = dst_cid
    where 
      (src_cid=@cid or dst_cid=@cid)
      and src.flag_abstract = 0 
      and dst.flag_abstract = 0
  open lc
  fetch next from lc into @link_type, @src_cid, @dst_cid
  while @@fetch_status <> -1
  begin
    select @sql = 'delete ' + rtrim(@src_cid) + '_' + rtrim(@dst_cid) + '_' + rtrim(@link_type)
    select @time_str = convert(varchar(32), getdate(), 14)
    raiserror('%s: %s', 0, 1, @time_str, @sql)
    exec(@sql)
    fetch next from lc into @link_type, @src_cid, @dst_cid
  end
  deallocate lc
    
  checkpoint
  dump tran Object with no_log
GO
