-- $Header:   //ARCHIVE/Archive/DB/sql/sp_tabledef.sqv   1.1   Jan 08 2001 14:51:56   BRENT  $
-- 
-- sp_tabledef.sql
--
--

USE master
GO

---------------------------------------------------------------------------------------------------
-- sp_tabledef
PRINT 'Creating procedure sp_tabledef'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'sp_tabledef' AND type = 'P')
  DROP PROCEDURE sp_tabledef
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/sp_tabledef.sqv  $
-- $Revision:   1.1  $
-- $Date:   Jan 08 2001 14:51:56  $
-- $Author:   BRENT  $
-- Outputs the definition of the specified table, along with any constraints defined.
create procedure sp_tabledef
  @table sysname
as
  set nocount on

  create table #coldef
  (
    colid  int PRIMARY KEY,
    name   sysname,
    type   sysname,
    length int,
    nullability varchar(8),
    pk     varchar(255) null,
    fk     varchar(255) null,
    ck     varchar(255) null,
    df     varchar(255) null
  )

  insert #coldef
  select 
    c.colid, c.name, t.name,
    case 
      when c.usertype < 5 then c.length
      else 0
    end,
    CASE c.isnullable when 0 then 'NOT NULL' ELSE 'NULL' END,
    null, null, null, null
  from
    syscolumns c,
    systypes t
  where
    c.id = object_id(@table)
    and t.usertype = c.usertype

  declare  @objid      int           -- the object id of the table
    ,@cnstdes    nvarchar(4000)-- string to build up index desc
    ,@cnstname    sysname       -- name of const. currently under consideration
    ,@i        int
    ,@cnstid    int
    ,@cnsttype    character(2)
    ,@keys      nvarchar(2078)  --Length (16*max_identifierLength)+(15*2)
    ,@dbname    sysname

  select @objid = object_id(@table)
  
  -- Create temp table for constraints
  create table #spcnsttab
  (
    cnst_id      int      NOT NULL
    ,cnst_type      nvarchar(146)NOT NULL   -- 128 for name + text for DEFAULT
    ,cnst_column    sysname NULL
    ,cnst_name      sysname    NOT NULL
    ,cnst_nonblank_name  sysname    NOT NULL
    ,cnst_2type      character(2)  NULL
    ,cnst_disabled    bit        NULL
    ,cnst_notrepl    bit        NULL
    ,cnst_keys      nvarchar(2078)  NULL  -- see @keys above for length descr
  )

  -- Check to see that the object names are local to the current database.
  select @dbname = parsename(@table,3) 

  if @dbname is not null and @dbname <> db_name()
  begin
    raiserror(15250,-1,-1)
    return (1)
  end

  -- Check to see if the table exists and initialize @objid.
  select @objid = object_id(@table)
  if @objid is NULL
  begin
    select @dbname=db_name()
    raiserror(15009,-1,-1,@table,@dbname)
    return (1)
  end

  -- STATIC CURSOR OVER THE TABLE'S CONSTRAINTS
  declare cnst_csr insensitive cursor for
    select id, xtype, name from sysobjects where parent_obj = @objid
      and xtype in ('C ','PK','UQ','F ', 'D ')  -- ONLY 6.5 sysconstraints objects
    for read only

  -- Now check out each constraint, figure out its type and keys and
  -- save the info in a temporary table that we'll print out at the end.
  open cnst_csr
  fetch cnst_csr into @cnstid ,@cnsttype ,@cnstname
  while @@fetch_status >= 0
  begin

    if @cnsttype in ('PK','UQ')
    begin
      -- get indid and index description
      declare @indid smallint
      select  @indid = indid,
          @cnstdes = case when @cnsttype = 'PK'
                then 'PRIMARY KEY' else 'UNIQUE' end
               + case when (status & 16)=16
                then ' CLUSTERED' else ' NONCLUSTERED' end
      from  sysindexes
      where  name = object_name(@cnstid)
          and id = @objid

      -- Format keys string
      declare @thiskey sysname
      select @keys = index_col(@table, @indid, 1), @i = 2,
          @thiskey = index_col(@table, @indid, 2)
      while (@thiskey is not null )
      begin
        select @keys = @keys + ', ' + @thiskey, @i = @i + 1
        select @thiskey = index_col(@table, @indid, @i)
      end

      -- ADD TO TABLE
      insert into #spcnsttab
        (cnst_id,cnst_type,cnst_column, cnst_name, cnst_nonblank_name,cnst_keys, cnst_2type)
      values (@cnstid, @cnstdes, NULL, @cnstname, @cnstname, @keys, @cnsttype)
    end

    else
    if @cnsttype = 'F '
    begin
      -- OBTAIN TWO TABLE IDs
      declare @fkeyid int, @rkeyid int
      select @fkeyid = fkeyid, @rkeyid = rkeyid from sysreferences where constid = @cnstid

      -- USE CURSOR OVER FOREIGN KEY COLUMNS TO BUILD COLUMN LISTS
      --  (NOTE: @keys HAS THE FKEY AND @cnstdes HAS THE RKEY COLUMN LIST)
      declare fkey_curs cursor for select fkey, rkey from sysforeignkeys where constid = @cnstid
      open fkey_curs
      declare @fkeycol smallint, @rkeycol smallint
      fetch fkey_curs into @fkeycol, @rkeycol
      select @keys = col_name(@fkeyid, @fkeycol), @cnstdes = col_name(@rkeyid, @rkeycol)
      fetch fkey_curs into @fkeycol, @rkeycol
      while @@fetch_status >= 0
      begin
        select  @keys = @keys + ', ' + col_name(@fkeyid, @fkeycol),
            @cnstdes = @cnstdes + ', ' + col_name(@rkeyid, @rkeycol)
        fetch fkey_curs into @fkeycol, @rkeycol
      end
      deallocate fkey_curs

      -- ADD ROWS FOR BOTH SIDES OF FOREIGN KEY
      insert into #spcnsttab 
        (cnst_id, cnst_type,cnst_name,cnst_nonblank_name,
          cnst_keys, cnst_disabled, 
          cnst_notrepl, cnst_2type)
      values 
        (@cnstid, 'FOREIGN KEY', @cnstname, @cnstname,
          @keys, ObjectProperty(@cnstid, 'CnstIsDisabled'),
          ObjectProperty(@cnstid, 'CnstIsNotRepl'), @cnsttype)
      insert into #spcnsttab
        (cnst_id,cnst_type,cnst_name,cnst_nonblank_name,
          cnst_keys,
          cnst_2type)
      select
        @cnstid,' ', ' ', @cnstname,
          'REFERENCES ' + db_name()
            + '.' + rtrim(user_name(ObjectProperty(@rkeyid,'ownerid')))
            + '.' + object_name(@rkeyid) + ' ('+@cnstdes + ')',
          @cnsttype
    end

    else
    if @cnsttype = 'C '
    begin
      select @i = 1
      select @cnstdes = text from syscomments where id = @cnstid and colid = @i
      while @cnstdes is not null
      begin
        if @i=1
          -- Check constraint
          insert into  #spcnsttab
            (cnst_id, cnst_type, cnst_column, cnst_name ,cnst_nonblank_name,
              cnst_keys, cnst_disabled, cnst_notrepl, cnst_2type)
          select  @cnstid, 'CHECK',
            case when info = 0 then NULL ELSE col_name(@objid ,info) end,
            @cnstname ,@cnstname ,substring(@cnstdes,1,2000),
            ObjectProperty(@cnstid, 'CnstIsDisabled'),
            ObjectProperty(@cnstid, 'CnstIsNotRepl'),
            @cnsttype
          from sysobjects  where id = @cnstid
        else
          insert into #spcnsttab (cnst_id,cnst_type,cnst_name,cnst_nonblank_name,cnst_keys, cnst_2type)
          select  @cnstid,' ' ,' ' ,@cnstname ,substring(@cnstdes,1,2000), @cnsttype

        if len(@cnstdes) > 2000
          insert into #spcnsttab (cnst_id,cnst_type,cnst_name,cnst_nonblank_name,cnst_keys, cnst_2type)
          select  @cnstid,' ' ,' ' ,@cnstname ,substring(@cnstdes,2001,2000), @cnsttype

        select @cnstdes = null
        select @i = @i + 1
        select @cnstdes = text from syscomments where id = @cnstid and colid = @i
      end
    end

    else
    if (@cnsttype = 'D ')
    begin
      select @i = 1
      select @cnstdes = text from syscomments where id = @cnstid and colid = @i
      while @cnstdes is not null
      begin
        if @i=1
          insert into  #spcnsttab
            (cnst_id,cnst_type, cnst_column, cnst_name ,cnst_nonblank_name ,cnst_keys, cnst_2type)
          select @cnstid, 'DEFAULT', col_name(@objid ,info),
            @cnstname ,@cnstname ,@cnstdes, @cnsttype
          from sysobjects where id = @cnstid
        else
          insert into #spcnsttab (cnst_id,cnst_type,cnst_name,cnst_nonblank_name,cnst_keys, cnst_2type)
          select  @cnstid,' ' ,' ' ,@cnstname ,@cnstdes, @cnsttype

        select @i = @i + 1
        select @cnstdes = null
        select @cnstdes = text from syscomments where id = @cnstid and colid = @i
      end
    end

    fetch cnst_csr into @cnstid ,@cnsttype ,@cnstname
  end    --of major loop
  deallocate cnst_csr

  -- Find any rules or defaults bound by the sp_bind... method.
  insert into #spcnsttab (cnst_id,cnst_type,cnst_name,cnst_nonblank_name,cnst_keys, cnst_2type)
  select c.domain,'RULE on column ' + c.name + ' (bound with sp_bindrule)',
    object_name(c.domain), object_name(c.domain), text, 'R '
  from  syscolumns c, syscomments m
  where  c.id = @objid and m.id = c.domain and ObjectProperty(c.domain, 'IsRule') = 1

  insert into #spcnsttab (cnst_id,cnst_type,cnst_name,cnst_nonblank_name,cnst_keys, cnst_2type)
  select c.cdefault, 'DEFAULT on column ' + c.name + ' (bound with sp_bindefault)',
    object_name(c.cdefault),object_name(c.cdefault), text, 'D '
  from  syscolumns c,syscomments m
  where  c.id = @objid and m.id = c.cdefault and ObjectProperty(c.cdefault, 'IsConstraint') = 0

--select * from #spcnsttab

  UPDATE c SET df = 'CONSTRAINT ' + t.cnst_name + ' DEFAULT ' + t.cnst_keys
  FROM #coldef AS c JOIN #spcnsttab AS t ON t.cnst_column = c.name
  WHERE t.cnst_type = 'DEFAULT'

  UPDATE c SET ck = 'CONSTRAINT ' + t.cnst_name + ' CHECK ' + t.cnst_keys
  FROM #coldef AS c JOIN #spcnsttab AS t ON t.cnst_column = c.name
  WHERE t.cnst_type = 'CHECK' AND t.cnst_column IS NOT NULL

  declare @lastcolid int
  select @lastcolid = max(colid) from #coldef

  create table #tabledef (line varchar(255), id int IDENTITY PRIMARY KEY)
  insert #tabledef(line) select 'CREATE TABLE ' + @table
  insert #tabledef(line) select '('
  insert #tabledef(line)
  select '  ' + convert(char(30),c.name) + c.type +
    case when c.length > 0 then '(' + convert(varchar(3),length) + ')' else ' ' end +
    ' ' + nullability + 
    CASE WHEN pk IS NULL THEN '' ELSE ' ' + pk END +
    CASE WHEN fk IS NULL THEN '' ELSE ' ' + fk END +
    CASE WHEN ck IS NULL THEN '' ELSE ' ' + ck END +
    CASE WHEN df IS NULL THEN '' ELSE ' ' + df END +
    CASE WHEN colid = @lastcolid THEN '' ELSE ',' END
  from #coldef AS c
  insert #tabledef(line) select ')'

  INSERT #tabledef(line)
    SELECT 'ALTER TABLE ' + @table + ' ADD CONSTRAINT ' + t.cnst_name + ' ' + t.cnst_type + '(' + t.cnst_keys + ')'
    FROM #spcnsttab AS t
    WHERE t.cnst_type LIKE 'PRIMARY KEY%'

  INSERT #tabledef(line)
    SELECT 'ALTER TABLE ' + @table + ' ADD CONSTRAINT ' + t.cnst_name + ' ' + t.cnst_type + '(' + t.cnst_keys + ')'
    FROM #spcnsttab AS t
    WHERE t.cnst_type = 'CHECK' AND t.cnst_column IS NULL

  SELECT cnst_id, CAST('ALTER TABLE ' + @table + ' ADD CONSTRAINT ' + t.cnst_name + ' ' + t.cnst_type + ' (' + t.cnst_keys + ')' AS VARCHAR(255)) AS fkdef
  INTO #fks
    FROM #spcnsttab AS t
    WHERE t.cnst_type = 'FOREIGN KEY'
  UPDATE f SET fkdef = fkdef + ' ' + t.cnst_keys 
  FROM #fks AS f JOIN #spcnsttab AS t ON f.cnst_id = t.cnst_id
  WHERE t.cnst_type = ''

  INSERT #tabledef(line)
    SELECT fkdef FROM #fks

  SELECT line FROM #tabledef
GO
