-- $Header:   //ARCHIVE/Archive/DB/sql/tU_ExceptionType_A.sqv   1.0   Jan 22 2001 14:43:16   BRENT  $
--
-- tU_ExceptionType_A.sql
--

USE Object
GO

---------------------------------------------------------------------------------------------------
-- tU_ExceptionType_A
PRINT 'Creating trigger tU_ExceptionType_A'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tU_ExceptionType_A' AND type = 'TR')
  DROP TRIGGER tU_ExceptionType_A
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/tU_ExceptionType_A.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jan 22 2001 14:43:16  $
-- $Author:   BRENT  $
-- Update trigger for ExceptionType_A table. If there are any rows affected the ExceptionType_T
-- is updated to reflect the changes.
CREATE TRIGGER tU_ExceptionType_A on ExceptionType_A FOR UPDATE AS
  IF @@ROWCOUNT > 0
  BEGIN
    -- Note that we cannot reflect the ExceptionType_A changes into ExceptionType_T with
    -- an UPDATE statement since there may be ExceptionType_A records being updated that
    -- were not previously in ExceptionType_T due to the old records having either AlertState
    -- or Priority values that precluded these records from being in ExceptionType_T (only
    -- ExceptionType_A records with defined AlertState and Priority values are recorded in
    -- ExceptionType_T). Instead we DELETE any old matching records and then INSERT all matching
    -- ones.

    -- We must first delete any ExceptionType_T records that match based on the cid and Exception
    -- columns.
    DELETE ett FROM ExceptionType_T AS ett JOIN ExceptionType_C AS etc ON 
      ett.cid = CAST(SUBSTRING(etc.name, 1, CHARINDEX('_', etc.name) - 1) AS CHAR(4))
    JOIN deleted AS eta ON 
      eta.id = etc.id
      AND eta._Exception = ett.Exception

    -- (Re)insert records that have defined AlertState and Priority values.
    INSERT ExceptionType_T
      SELECT DISTINCT
        CAST(SUBSTRING(etc.name, 1, CHARINDEX('_', etc.name) - 1) AS CHAR(4)) AS cid,
        eta._Exception AS Exception,
        CAST(als._Desc AS VARCHAR(8)) AS AlertState,
        CAST(pri._Priority AS VARCHAR(12)) AS Priority,
        CAST(SUBSTRING(eta._AlertState, 18, 1) AS INT) AS AlertStateID,
        CAST(SUBSTRING(eta._Priority, 18, 1) AS INT) AS PriorityID,
        eta._Desc AS Format,
        eta._Salience AS Salience
      FROM
        inserted AS eta
        JOIN ExceptionType_V etc ON etc.id = eta.id
        JOIN AlertState_V AS als ON als.id = CAST(SUBSTRING(eta._AlertState, 18, 1) AS INT)
        JOIN SixLevelPri_V AS pri ON pri.id = CAST(SUBSTRING(eta._Priority, 18, 1) AS INT)
        JOIN containment AS c ON c.dst_cid = 'EXTP' AND c.dst_id = eta.id
      WHERE
        c.deleted=0
  END
GO
