-- $Header:   //ARCHIVE/Archive/DB/sql/tU_MessageDescription_A.sqv   1.0   Jan 22 2001 14:43:48   BRENT  $
--
-- tU_MessageDescription_A.sql
--

USE Object
GO

---------------------------------------------------------------------------------------------------
-- tU_MessageDescription_A
PRINT 'Creating trigger tU_MessageDescription_A'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tU_MessageDescription_A' AND type = 'TR')
  DROP TRIGGER tU_MessageDescription_A
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/tU_MessageDescription_A.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jan 22 2001 14:43:48  $
-- $Author:   BRENT  $
-- Custom update trigger for MessageDescription table.
-- If one row is affected and the SearchExpression, ChildLookupPattern, or ProcessingPattern is
-- updated then the appropriate notification(s) are sent out.
-- If one or more rows are affected then the MessageDescription_T table is rebuilt.
CREATE TRIGGER tU_MessageDescription_A ON MessageDescription_A FOR UPDATE AS
  DECLARE @rc int
  SELECT @rc = @@rowcount
  -- Notify dispatcher of any changes to rule processing attributes of message description
  IF @rc = 1
  BEGIN
    IF UPDATE(_SearchExpression) or UPDATE(_ChildLookupPattern) or UPDATE(_ProcessingPattern)
    BEGIN
      DECLARE
        @id ObjID,
        @SearchExpression VARCHAR(255),
        @ChildLookupPattern VARCHAR(255),
        @ProcessingPattern VARCHAR(255)
      SELECT 
        @id = id,
        @SearchExpression = _SearchExpression,
        @ChildLookupPattern = _ChildLookupPattern,
        @ProcessingPattern = _ProcessingPattern
      FROM
        inserted
      IF UPDATE(_SearchExpression)
        EXEC asisp_notifyMessageDescription 21 /* MessageDescriptionUpdated */, @id, 0, @SearchExpression
      IF UPDATE(_ChildLookupPattern)
        EXEC asisp_notifyMessageDescription 21 /* MessageDescriptionUpdated */, @id, 1, @ChildLookupPattern
      IF UPDATE(_ProcessingPattern)
        EXEC asisp_notifyMessageDescription 21 /* MessageDescriptionUpdated */, @id, 2, @ProcessingPattern
    end
  end
  IF @rc > 0
  BEGIN
    -- Repopulate the MessageDescription_T table FROM the MessageDescription view.
    -- Must be done with a DELETE followed by an INSERT since the rows before and after
    -- may be different given that we are joining to StateType_V, AlertState_V, and SixLevelPri_V.
    DELETE mdt FROM MessageDescription_T AS mdt
      JOIN deleted AS d ON d.id = mdt.msgd_id

    INSERT MessageDescription_T
      SELECT
        msgd._ParentCID AS obj_cid,
        msgd._ParentID AS obj_id,
        msgd.id AS msgd_id,
        msgd._MsgID AS MsgID,
        CAST(sttp._State AS VARCHAR(24)) AS State,
        CAST (alrs._Desc AS VARCHAR(12)) AS AlertState,
        CAST (pri6._Priority AS VARCHAR(12)) AS Priority,
        msgd._StateID AS StateID,
        msgd._AlertStateID AS AlertStateID,
        msgd. _PriorityID AS PriorityID,
        msgd._Flags AS Flags,
        msgd._SearchExpression AS SearchExpression,
        msgd._ChildLookupPattern AS ChildLookupPattern,
        msgd._ProcessingPattern AS ProcessingPattern,
        msgd._Desc AS Description
      FROM
        MessageDescription_V AS msgd
        JOIN inserted AS i ON i.id = msgd.id
        JOIN StateType_V AS sttp ON sttp.id = msgd._StateID
        JOIN AlertState_V AS alrs ON alrs.id = msgd._AlertStateID
        JOIN SixLevelPri_V AS pri6 ON pri6.id = msgd._PriorityID
      WHERE
        msgd.deleted = 0
  END
GO
