-- $Header:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv   1.32   Feb 13 2001 19:13:38   BRENT  $
--
-- RODMObject.sql
--
-- RODM-related database objects that reside in the TBSM Object database
--

USE Object
GO

-- Drop old tables/views
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'SNAClassDef' AND type = 'V')
  DROP VIEW SNAClassDef
GO
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'SNAClassDefModel' AND type = 'V')
  DROP VIEW SNAClassDefModel
GO
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'SNADisplayResourceType' AND type = 'U')
  DROP TABLE SNADisplayResourceType
GO
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'SNAFieldDef' AND type = 'V')
  DROP VIEW SNAFieldDef
GO
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'SNAFieldDefModel' AND type = 'V')
  DROP VIEW SNAFieldDefModel
GO
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'SNAFieldLinkMap' AND type = 'V')
  DROP VIEW SNAFieldLinkMap
GO
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'SNAIconCrossRef' AND type = 'U')
  DROP TABLE SNAIconCrossRef
GO
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'SNA_isa' AND type = 'V')
  DROP VIEW SNA_isa
GO
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'SNA_isa_chain' AND type = 'V')
  DROP VIEW SNA_isa_chain
GO
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMManagedLink' AND type = 'U')
  DROP TABLE RODMManagedLink
GO
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMManagedObject' AND type = 'U')
  DROP TABLE RODMManagedObject
GO

---------------------------------------------------------------------------------------------------
-- Define RODM format, action, and data types
EXEC asisp_defineFormatType 5, 'RODM', 'Events FOR RODM Managed Objects'

EXEC asisp_defineActionType 5, 0, 'OBJECT_DISCOVERY',      'Discovery record FOR a given object instance.'
EXEC asisp_defineActionType 5, 1, 'CONNECTION_STATUS',     'Event describing status of connection between TBSM Pump and RODM.'
EXEC asisp_defineActionType 5, 2, 'OBJECT_REGISTRATION',   'Registration message specifying whether to receive notifications of changes to a given object/field.'
EXEC asisp_defineActionType 5, 3, 'REGISTRATION_RESPONSE', 'Response message FOR a given OBJECT_REGISTRATION message. Used to indicate whether a given registration was successful.'
EXEC asisp_defineActionType 5, 4, 'OBJECT_STATE',          'Event specifying value change to a registered field on an object'

EXEC asisp_defineDataType 5, 0, 'DATE_TIME',        'Event timestamp. Format: YYYYMMDDHH:MM:SS.UUUUUU'
EXEC asisp_defineDataType 5, 1, 'SEQ_NO',           'Event sequence #. The SMF-ID concatenated with the trap match number associated with trap setting.'
EXEC asisp_defineDataType 5, 2, 'OBJ_TYPE_ID',      'Object type'
EXEC asisp_defineDataType 5, 47, 'SWITCH',          'Indicates whether object/field is being registered. Values are "ON" or "OFF"'
EXEC asisp_defineDataType 5, 73, 'PARM_1',          'Parameter 1'
EXEC asisp_defineDataType 5, 74, 'PARM_2',          'Parameter 2'
EXEC asisp_defineDataType 5, 75, 'PARM_3',          'Parameter 3'
EXEC asisp_defineDataType 5, 76, 'PARM_4',          'Parameter 4'
EXEC asisp_defineDataType 5, 84, 'CORRELATION_ID',  'String specified by caller in registration messages and returned in response messages'
EXEC asisp_defineDataType 5, 85, 'RESP_TYPE',       'Used in registration messages to indicate type of response requested.
EX : Exception  only send response record if error
DEF : Definite  always send a response record
NONE : no response reqd. This is the default if omitted.'

---------------------------------------------------------------------------------------------------
-- Create views of tables in the RODMLoad database

---------------------------------------------------------------------------------------------------
-- RODMClassMap
PRINT 'Creating view RODMClassMap'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMClassMap' AND type = 'V')
  DROP VIEW RODMClassMap
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
CREATE VIEW RODMClassMap AS SELECT * FROM RODMLoad..RODMClassMap
GO


---------------------------------------------------------------------------------------------------
-- RODMFieldMap
PRINT 'Creating view RODMFieldMap'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMFieldMap' AND type = 'V')
  DROP VIEW RODMFieldMap
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
CREATE VIEW RODMFieldMap AS SELECT * FROM RODMLoad..RODMFieldMap
GO


---------------------------------------------------------------------------------------------------
-- RODMFieldLinkMap
PRINT 'Creating view RODMFieldLinkMap'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMFieldLinkMap' AND type = 'V')
  DROP VIEW RODMFieldLinkMap
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
CREATE VIEW RODMFieldLinkMap AS SELECT * FROM RODMLoad..RODMFieldLinkMap
GO


---------------------------------------------------------------------------------------------------
-- RODMClassLinkMap
PRINT 'Creating view RODMClassLinkMap'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMClassLinkMap' AND type = 'V')
  DROP VIEW RODMClassLinkMap
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
CREATE VIEW RODMClassLinkMap AS SELECT * FROM RODMLoad..RODMClassLinkMap
GO


---------------------------------------------------------------------------------------------------
-- RODMNameField
PRINT 'Creating view RODMNameField'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMNameField' AND type = 'V')
  DROP VIEW RODMNameField
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
CREATE VIEW RODMNameField AS SELECT * FROM RODMLoad..RODMNameField
GO


---------------------------------------------------------------------------------------------------
-- RODMUnmanagedClass
PRINT 'Creating view RODMUnmanagedClass'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMUnmanagedClass' AND type = 'V')
  DROP VIEW RODMUnmanagedClass
GO


---------------------------------------------------------------------------------------------------
-- RODMTopLevelClass
PRINT 'Creating view RODMTopLevelClass'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMTopLevelClass' AND type = 'V')
  DROP VIEW RODMTopLevelClass
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
CREATE VIEW RODMTopLevelClass AS SELECT * FROM RODMLoad..RODMTopLevelClass
GO


---------------------------------------------------------------------------------------------------
-- RODMClassIdLookup
PRINT 'Creating view RODMClassIdLookup'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMClassIdLookup' AND type = 'V')
  DROP VIEW RODMClassIdLookup
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
CREATE VIEW RODMClassIdLookup AS SELECT * FROM RODMLoad..RODMClassIdLookup
GO


---------------------------------------------------------------------------------------------------
-- RODM_isa
PRINT 'Creating view RODM_isa'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RODM_isa' AND type = 'V')
  DROP VIEW RODM_isa
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
CREATE VIEW RODM_isa AS SELECT * FROM RODMLoad..RODM_isa
GO


---------------------------------------------------------------------------------------------------
-- RODM_isa_chain
PRINT 'Creating view RODM_isa_chain'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RODM_isa_chain' AND type = 'V')
  DROP VIEW RODM_isa_chain
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
CREATE VIEW RODM_isa_chain AS SELECT * FROM RODMLoad..RODM_isa_chain
GO


---------------------------------------------------------------------------------------------------
-- RODMObject
PRINT 'Creating view RODMObject'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMObject' AND type = 'V')
  DROP VIEW RODMObject
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
CREATE VIEW RODMObject AS SELECT * FROM RODMLoad..RODMObject
GO


---------------------------------------------------------------------------------------------------
-- RODMLink
PRINT 'Creating view RODMLink'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMLink' AND type = 'V')
  DROP VIEW RODMLink
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
CREATE VIEW RODMLink AS SELECT * FROM RODMLoad..RODMLink
GO


---------------------------------------------------------------------------------------------------
-- RODMObjValScalar
PRINT 'Creating view RODMObjValScalar'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMObjValScalar' AND type = 'V')
  DROP VIEW RODMObjValScalar
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
CREATE VIEW RODMObjValScalar AS SELECT * FROM RODMLoad..RODMObjValScalar
GO


---------------------------------------------------------------------------------------------------
-- RODMObjValSelfDefining
PRINT 'Creating view RODMObjValSelfDefining'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMObjValSelfDefining' AND type = 'V')
  DROP VIEW RODMObjValSelfDefining
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
CREATE VIEW RODMObjValSelfDefining AS SELECT * FROM RODMLoad..RODMObjValSelfDefining
GO


---------------------------------------------------------------------------------------------------
-- RODMObjValIndexList
PRINT 'Creating view RODMObjValIndexList'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMObjValIndexList' AND type = 'V')
  DROP VIEW RODMObjValIndexList
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
CREATE VIEW RODMObjValIndexList AS SELECT * FROM RODMLoad..RODMObjValIndexList
GO


IF NOT EXISTS (SELECT * FROM master..sysdatabases WHERE name = 'RODM')
BEGIN
  PRINT 'Creating database RODM'
  CREATE DATABASE RODM
END
GO

USE RODM
GO

EXEC Meta..DefineMetaViews RODM
GO


---------------------------------------------------------------------------------------------------
-- Create RODMManagedObject and RODMManagedLink tables

EXEC master..CreateRODMManagedObject RODM
EXEC master..CreateRODMManagedLink RODM




---------------------------------------------------------------------------------------------------
/*
   Create and populate the RODMStateChangeMap table that the RODMManagedObject 
   update trigger will use to create exceptions and/or messages.
*/

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMStateChangeMap' AND type = 'U')
BEGIN
  PRINT 'Creating table RODMStateChangeMap'
  EXEC("
  CREATE TABLE RODMStateChangeMap
  (
    id          INT           CONSTRAINT RODMStateChangeMap_ID IDENTITY(1,1),
    stateVar    VARCHAR(32),
    whereClause VARCHAR(255),
    eventCID    ClassID,
    eventName   VARCHAR(255),
    eventText   VARCHAR(255),
    alertState  INT NULL,
    priority    INT NULL,
    salience    TINYINT,
    exclusive   TINYINT,
    enabled     TINYINT
  )
  ")

  EXEC("
  -- alert states: 3=red, 2=yellow, 1=green
  -- priorities: 1=critical, 2=high, 3=medium, 4=low, 5=ignore

  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'operationalState', 
            'new._operationalState = 0', 
            'EXCP', 'Operational State disabled', '%cname% %name% Operational State disabled', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'operationalState', 
            'old._operationalState = 0', 
            'EXCP', 'Operational State disabled', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'operationalState', 
            'new._operationalState = 255', 
            'EXCP', 'Operational State is not currently set', '%cname% %name% Operational State is not currently set', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'operationalState', 
            'old._operationalState = 255', 
            'EXCP', 'Operational State is not currently set', 'OK', 192, 0, 1

  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'usageState', 
            'new._usageState = 0', 
            'EXCP', 'Usage State idle', '%cname% %name% Usage State idle', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'usageState', 
            'old._usageState = 0', 
            'EXCP', 'Usage State idle', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'usageState', 
            'new._usageState = 1', 
            'EXCP', 'Usage State active', '%cname% %name% Usage State active', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'usageState', 
            'old._usageState = 1', 
            'EXCP', 'Usage State active', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'usageState', 
            'new._usageState = 2', 
            'EXCP', 'Usage State busy', '%cname% %name% Usage State busy', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'usageState', 
            'old._usageState = 2', 
            'EXCP', 'Usage State busy', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'usageState', 
            'new._usageState = 255', 
            'EXCP', 'Usage State is not currently set', '%cname% %name% Usage State is not currently set', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'usageState', 
            'old._usageState = 255', 
            'EXCP', 'Usage State is not currently set', 'OK', 192, 0, 1

  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'administrativeState', 
            'new._administrativeState = 0', 
            'EXCP', 'Administrative State locked', '%cname% %name% Administrative State locked', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'administrativeState', 
            'old._administrativeState = 0', 
            'EXCP', 'Administrative State locked', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'administrativeState', 
            'new._administrativeState = 1', 
            'EXCP', 'Administrative State unlocked', '%cname% %name% Administrative State unlocked', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'administrativeState', 
            'old._administrativeState = 1', 
            'EXCP', 'Administrative State unlocked', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'administrativeState', 
            'new._administrativeState = 2', 
            'EXCP', 'Administrative State shutting down', '%cname% %name% Administrative State shutting down', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'administrativeState', 
            'old._administrativeState = 2', 
            'EXCP', 'Administrative State shutting down', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'administrativeState', 
            'new._administrativeState = 255', 
            'EXCP', 'Administrative State is not currently set', '%cname% %name% Administrative State is not currently set', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'administrativeState', 
            'old._administrativeState = 255', 
            'EXCP', 'Administrative State is not currently set', 'OK', 192, 0, 1

  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'availabilityStatus', 
            '(old._availabilityStatus & 0x80) = 0 AND (new._availabilityStatus & 0x80) <> 0', 
            'EXCP', 'Availability Status in test', '%cname% %name% Availability Status in test', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'availabilityStatus', 
            '(old._availabilityStatus & 0x80) <> 0 AND (new._availabilityStatus & 0x80) = 0', 
            'EXCP', 'Availability Status in test', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'availabilityStatus', 
            '(old._availabilityStatus & 0x40) = 0 AND (new._availabilityStatus & 0x40) <> 0', 
            'EXCP', 'Availability Status failed', '%cname% %name% Availability Status failed', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'availabilityStatus', 
            '(old._availabilityStatus & 0x40) <> 0 AND (new._availabilityStatus & 0x40) = 0', 
            'EXCP', 'Availability Status failed', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'availabilityStatus', 
            '(old._availabilityStatus & 0x20) = 0 AND (new._availabilityStatus & 0x20) <> 0', 
            'EXCP', 'Availability Status power off', '%cname% %name% Availability Status power off', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'availabilityStatus', 
            '(old._availabilityStatus & 0x20) <> 0 AND (new._availabilityStatus & 0x20) = 0', 
            'EXCP', 'Availability Status power off', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'availabilityStatus', 
            '(old._availabilityStatus & 0x10) = 0 AND (new._availabilityStatus & 0x10) <> 0', 
            'EXCP', 'Availability Status offline', '%cname% %name% Availability Status offline', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'availabilityStatus', 
            '(old._availabilityStatus & 0x10) <> 0 AND (new._availabilityStatus & 0x10) = 0', 
            'EXCP', 'Availability Status offline', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'availabilityStatus', 
            '(old._availabilityStatus & 0x08) = 0 AND (new._availabilityStatus & 0x08) <> 0', 
            'EXCP', 'Availability Status off duty', '%cname% %name% Availability Status off duty', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'availabilityStatus', 
            '(old._availabilityStatus & 0x08) <> 0 AND (new._availabilityStatus & 0x08) = 0', 
            'EXCP', 'Availability Status off duty', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'availabilityStatus', 
            '(old._availabilityStatus & 0x04) = 0 AND (new._availabilityStatus & 0x04) <> 0', 
            'EXCP', 'Availability Status dependency', '%cname% %name% Availability Status dependency', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'availabilityStatus', 
            '(old._availabilityStatus & 0x04) <> 0 AND (new._availabilityStatus & 0x04) = 0', 
            'EXCP', 'Availability Status dependency', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'availabilityStatus', 
            '(old._availabilityStatus & 0x02) = 0 AND (new._availabilityStatus & 0x02) <> 0', 
            'EXCP', 'Availability Status degraded', '%cname% %name% Availability Status degraded', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'availabilityStatus', 
            '(old._availabilityStatus & 0x02) <> 0 AND (new._availabilityStatus & 0x02) = 0', 
            'EXCP', 'Availability Status degraded', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'availabilityStatus', 
            '(old._availabilityStatus & 0x01) = 0 AND (new._availabilityStatus & 0x01) <> 0', 
            'EXCP', 'Availability Status not installed', '%cname% %name% Availability Status not installed', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'availabilityStatus', 
            '(old._availabilityStatus & 0x01) <> 0 AND (new._availabilityStatus & 0x01) = 0', 
            'EXCP', 'Availability Status not installed', 'OK', 192, 0, 1

  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'proceduralStatus', 
            '(old._proceduralStatus & 0x80) = 0 AND (new._proceduralStatus & 0x80) <> 0', 
            'EXCP', 'Procedural Status initialization required', '%cname% %name% Procedural Status initialization required', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'proceduralStatus', 
            '(old._proceduralStatus & 0x80) <> 0 AND (new._proceduralStatus & 0x80) = 0', 
            'EXCP', 'Procedural Status initialization required', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'proceduralStatus', 
            '(old._proceduralStatus & 0x40) = 0 AND (new._proceduralStatus & 0x40) <> 0', 
            'EXCP', 'Procedural Status not initialized', '%cname% %name% Procedural Status not initialized', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'proceduralStatus', 
            '(old._proceduralStatus & 0x40) <> 0 AND (new._proceduralStatus & 0x40) = 0', 
            'EXCP', 'Procedural Status not initialized', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'proceduralStatus', 
            '(old._proceduralStatus & 0x20) = 0 AND (new._proceduralStatus & 0x20) <> 0', 
            'EXCP', 'Procedural Status initializing', '%cname% %name% Procedural Status initializing', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'proceduralStatus', 
            '(old._proceduralStatus & 0x20) <> 0 AND (new._proceduralStatus & 0x20) = 0', 
            'EXCP', 'Procedural Status initializing', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'proceduralStatus', 
            '(old._proceduralStatus & 0x10) = 0 AND (new._proceduralStatus & 0x10) <> 0', 
            'EXCP', 'Procedural Status reporting', '%cname% %name% Procedural Status reporting', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'proceduralStatus', 
            '(old._proceduralStatus & 0x10) <> 0 AND (new._proceduralStatus & 0x10) = 0', 
            'EXCP', 'Procedural Status reporting', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'proceduralStatus', 
            '(old._proceduralStatus & 0x08) = 0 AND (new._proceduralStatus & 0x08) <> 0', 
            'EXCP', 'Procedural Status terminating', '%cname% %name% Procedural Status terminating', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'proceduralStatus', 
            '(old._proceduralStatus & 0x08) <> 0 AND (new._proceduralStatus & 0x08) = 0', 
            'EXCP', 'Procedural Status terminating', 'OK', 192, 0, 1

  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'unknownStatus', 
            'new._unknownStatus = 1', 
            'EXCP', 'Unknown Status', '%cname% %name% Unknown Status', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'unknownStatus', 
            'old._unknownStatus = 1', 
            'EXCP', 'Unknown Status', 'OK', 192, 0, 1

  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'new._nativeStatus = 0',
            'EXCP', 'Native Status active', '%cname% %name% Native Status active', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'old._nativeStatus = 0',
            'EXCP', 'Native Status active', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'new._nativeStatus = 1',
            'EXCP', 'Native Status active with session', '%cname% %name% Native Status active with session', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'old._nativeStatus = 1',
            'EXCP', 'Native Status active with session', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'new._nativeStatus = 2',
            'EXCP', 'Native Status inactive', '%cname% %name% Native Status inactive', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'old._nativeStatus = 2',
            'EXCP', 'Native Status inactive', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'new._nativeStatus = 3',
            'EXCP', 'Native Status never active', '%cname% %name% Native Status never active', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'old._nativeStatus = 3',
            'EXCP', 'Native Status never active', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'new._nativeStatus = 4',
            'EXCP', 'Native Status pending active', '%cname% %name% Native Status pending active', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'old._nativeStatus = 4',
            'EXCP', 'Native Status pending active', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'new._nativeStatus = 5',
            'EXCP', 'Native Status pending inactive', '%cname% %name% Native Status pending inactive', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'old._nativeStatus = 5',
            'EXCP', 'Native Status pending inactive', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'new._nativeStatus = 6',
            'EXCP', 'Native Status connectable', '%cname% %name% Native Status connectable', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'old._nativeStatus = 6',
            'EXCP', 'Native Status connectable', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'new._nativeStatus = 7',
            'EXCP', 'Native Status routable', '%cname% %name% Native Status routable', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'old._nativeStatus = 7',
            'EXCP', 'Native Status routable', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'new._nativeStatus = 8',
            'EXCP', 'Native Status operative', '%cname% %name% Native Status operative', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'old._nativeStatus = 8',
            'EXCP', 'Native Status operative', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'new._nativeStatus = 9',
            'EXCP', 'Native Status congested', '%cname% %name% Native Status congested', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'old._nativeStatus = 9',
            'EXCP', 'Native Status congested', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'new._nativeStatus = 10',
            'EXCP', 'Native Status released', '%cname% %name% Native Status released', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'old._nativeStatus = 10',
            'EXCP', 'Native Status released', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'new._nativeStatus = 11',
            'EXCP', 'Native Status reset', '%cname% %name% Native Status reset', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'old._nativeStatus = 11',
            'EXCP', 'Native Status reset', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'new._nativeStatus = 12',
            'EXCP', 'Native Status inoperative', '%cname% %name% Native Status inoperative', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'old._nativeStatus = 12',
            'EXCP', 'Native Status inoperative', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'new._nativeStatus = 13',
            'EXCP', 'Native Status pending active or inactive', '%cname% %name% Native Status pending active or inactive', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'old._nativeStatus = 13',
            'EXCP', 'Native Status pending active or inactive', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'new._nativeStatus = 14',
            'EXCP', 'Native Status released or reset', '%cname% %name% Native Status released or reset', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'old._nativeStatus = 14',
            'EXCP', 'Native Status released or reset', 'OK', 192, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'new._nativeStatus = 255',
            'EXCP', 'Native Status is not currently set', '%cname% %name% Native Status is not currently set', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'nativeStatus',
            'old._nativeStatus = 255',
            'EXCP', 'Native Status is not currently set', 'OK', 192, 0, 1

  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'DisplayStatus',
            'new._DisplayStatus = 129',
            'MESG', 'Display Status satisfactory', '%cname% %name% Display Status Satisfactory', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'DisplayStatus',
            'new._DisplayStatus = 130',
            'MESG', 'Display Status unsatisfactory', '%cname% %name% Display Status Unsatisfactory', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'DisplayStatus',
            'new._DisplayStatus = 131',
            'MESG', 'Display Status intermediate', '%cname% %name% Display Status Intermediate', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'DisplayStatus',
            'new._DisplayStatus = 132',
            'MESG', 'Display Status unknown', '%cname% %name% Display Status Unknown', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'DisplayStatus',
            'new._DisplayStatus = 144',
            'MESG', 'Display Status medium satisfactory', '%cname% %name% Display Status Medium satisfactory', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'DisplayStatus',
            'new._DisplayStatus = 145',
            'MESG', 'Display Status low satisfactory', '%cname% %name% Display Status Low satisfactory', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'DisplayStatus',
            'new._DisplayStatus = 160',
            'MESG', 'Display Status medium unsatisfactory', '%cname% %name% Display Status Medium unsatisfactory', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'DisplayStatus',
            'new._DisplayStatus = 161',
            'MESG', 'Display Status low unsatisfactory', '%cname% %name% Display Status Low unsatisfactory', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'DisplayStatus',
            'new._DisplayStatus = 133',
            'MESG', 'Display Status degraded', '%cname% %name% Display Status Degraded', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'DisplayStatus',
            'new._DisplayStatus = 134',
            'MESG', 'Display Status severely degraded', '%cname% %name% Display Status Severely degraded', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'DisplayStatus',
            '(new._DisplayStatus >= 136 AND new._DisplayStatus <= 143)',
            'MESG', 'Display Status positive user-defined (%newValue%)', '%cname% %name% Display Status Positive user-defined (%newValue%)', 128, 0, 1
  INSERT RODMStateChangeMap (stateVar,whereClause,eventCID,eventName,eventText,salience,exclusive,enabled)
     SELECT 'DisplayStatus',
            '(new._DisplayStatus >= 152 AND new._DisplayStatus <= 159)',
            'MESG', 'Display Status negative user-defined (%newValue%)', '%cname% %name% Display Status Negative user-defined (%newValue%)', 128, 0, 1

  -- alert states: 3=red, 2=yellow, 1=green
  -- priorities: 1=critical, 2=high, 3=medium, 4=low, 5=ignore
  ")
END
GO


---------------------------------------------------------------------------------------------------
-- tI_RODMManagedLink
PRINT 'Creating trigger tI_RODMManagedLink'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tI_RODMManagedLink' AND type = 'TR')
  DROP trigger tI_RODMManagedLink
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
CREATE TRIGGER tI_RODMManagedLink ON RODMManagedLink FOR INSERT
AS
  DECLARE @link_type VARCHAR(4)
  DECLARE @src_cid ClassID
  DECLARE @dst_cid ClassID
  DECLARE @sql VARCHAR(500)

  IF EXISTS 
  (
    SELECT * 
    FROM inserted 
    WHERE link_type <> '' AND src_cid <> '' AND dst_cid <> ''
  )
  BEGIN
    -- Create temp table with the source and destination object ids. 
    -- This is used in the dynamic sql that follows since the 'inserted'
    -- pseudo-table cannot be referenced in the dynamic sql.
    SELECT link_type, src_cid, src_id, dst_cid, dst_id 
    INTO #new
    FROM inserted 
    WHERE link_type <> '' AND src_cid <> '' AND dst_cid <> ''

    -- Insert LinkCreated notifications
    INSERT Object..PendingNotification(type, fields, detail)
      SELECT
        e.type,
        lt.link_type + '|' + 
        CONVERT(VARCHAR(9), ISNULL(src.cno, 0))  + '|' +
        CONVERT(VARCHAR(9), ISNULL(i.src_id, 0)) + '|' +
        CONVERT(VARCHAR(9), ISNULL(dst.cno, 0))  + '|' +
        CONVERT(VARCHAR(9), ISNULL(i.dst_id, 0)) + '|' +
        CONVERT(VARCHAR(9), ISNULL(lt.flags, 0)),
        ''
      FROM
        Object..EventType AS e,
        #new AS i
        JOIN obj_class AS src ON src.cid = i.src_cid
        JOIN obj_class AS dst ON dst.cid = i.dst_cid
        JOIN link_type AS lt ON lt.link_type = i.link_type
      WHERE
        e.name = 'LinkCreated'

    -- Insert rows into the individual link tables
    DECLARE linkCursor CURSOR FOR 
      SELECT DISTINCT link_type, src_cid, dst_cid 
      FROM #new
    OPEN linkCursor
    FETCH NEXT FROM linkCursor INTO @link_type, @src_cid, @dst_cid
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      -- Insert link records
      SET @sql = 'INSERT Object..' + RTRIM(@src_cid) + '_' + RTRIM(@dst_cid) + '_' + @link_type + ' (src_id,dst_id) ' +
                 'SELECT src_id, dst_id FROM #new AS new ' +
                 'WHERE src_cid=''' + @src_cid + ''' AND dst_cid=''' + @dst_cid + ''' AND link_type=''' + @link_type + ''' ' + 
                 'AND NOT EXISTS (SELECT * FROM Object..' + RTRIM(@src_cid) + '_' + RTRIM(@dst_cid) + '_' + @link_type + ' AS old ' +
                 'WHERE new.src_id = old.src_id AND new.dst_id = old.dst_id)'
      PRINT 'tI_RODMManagedLink: ' + @sql
      EXEC(@sql)
    
      FETCH NEXT FROM linkCursor INTO @link_type, @src_cid, @dst_cid
    END
    DEALLOCATE linkCursor
  END
GO

---------------------------------------------------------------------------------------------------
-- tD_RODMManagedLink
PRINT 'Creating trigger tD_RODMManagedLink'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tD_RODMManagedLink' AND type = 'TR')
  DROP TRIGGER tD_RODMManagedLink
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
CREATE TRIGGER tD_RODMManagedLink ON RODMManagedLink FOR DELETE
AS
  DECLARE @link_type VARCHAR(4)
  DECLARE @src_cid ClassID
  DECLARE @dst_cid ClassID
  DECLARE @sql VARCHAR(256)

  IF EXISTS
  (
    SELECT *
    FROM deleted
    WHERE link_type <> '' AND src_cid <> '' AND dst_cid <> ''
  )
  BEGIN
    -- Create temp table with the source and destination object ids. 
    -- This is used in the dynamic sql that follows since the 'deleted'
    -- pseudo-table cannot be referenced in the dynamic sql.
    SELECT link_type, src_cid, src_id, dst_cid, dst_id 
    INTO #old
    FROM deleted
    WHERE link_type <> '' AND src_cid <> '' AND dst_cid <> ''

    -- Insert LinkDeleted notifications
    INSERT Object..PendingNotification(type, fields, detail)
      SELECT
        e.type,
        lt.link_type + '|' + 
        CONVERT(VARCHAR(9), ISNULL(src.cno, 0))  + '|' +
        CONVERT(VARCHAR(9), ISNULL(d.src_id, 0)) + '|' +
        CONVERT(VARCHAR(9), ISNULL(dst.cno, 0))  + '|' +
        CONVERT(VARCHAR(9), ISNULL(d.dst_id, 0)) + '|' +
        CONVERT(VARCHAR(9), ISNULL(lt.flags, 0)),
        ''
      FROM
        Object..EventType AS e,
        #old AS d
        JOIN obj_class AS src ON src.cid = d.src_cid
        JOIN obj_class AS dst ON dst.cid = d.dst_cid
        JOIN link_type AS lt ON lt.link_type = d.link_type
      WHERE
        e.name = 'LinkDeleted'

    -- Delete the individual link table records
    DECLARE linkCursor CURSOR FOR 
      SELECT DISTINCT link_type, src_cid, dst_cid
      FROM #old
    OPEN linkCursor
    FETCH NEXT FROM linkCursor INTO @link_type, @src_cid, @dst_cid
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      SET @sql = 'DELETE l FROM Object..' + RTRIM(@src_cid) + '_' + RTRIM(@dst_cid) + '_' + @link_type + ' AS l ' +
                 'JOIN #old AS o ON o.src_id = l.src_id AND o.dst_id = l.dst_id'
      PRINT 'tI_RODMManagedLink: ' + @sql
      EXEC(@sql)

      FETCH NEXT FROM linkCursor INTO @link_type, @src_cid, @dst_cid
    END
    DEALLOCATE linkCursor
  END
GO


---------------------------------------------------------------------------------------------------
-- tU_RODMManagedLink
PRINT 'Creating trigger tU_RODMManagedLink'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tU_RODMManagedLink' AND type = 'TR')
  DROP TRIGGER tU_RODMManagedLink
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
CREATE TRIGGER tU_RODMManagedLink ON RODMManagedLink FOR UPDATE
AS
  DECLARE @dlink_type VARCHAR(4), @ilink_type VARCHAR(4)
  DECLARE @dsrc_cid ClassID, @isrc_cid ClassID
  DECLARE @ddst_cid ClassID, @idst_cid ClassID
  DECLARE @sql VARCHAR(500)

  IF EXISTS
  (
    SELECT * FROM inserted AS i JOIN deleted AS d ON
      d.SrcClass = i.SrcClass AND d.SrcMyName = i.SrcMyName AND d.SrcField = i.SrcField
      AND d.DstClass = i.DstClass AND d.DstMyName = i.DstMyName AND d.DstField = i.DstField
    WHERE
      d.link_type <> i.link_type 
      OR d.src_cid <> i.src_cid 
      OR d.dst_cid <> i.dst_cid
      AND 
      (
        (d.link_type <> '' AND d.src_cid <> '' AND d.dst_cid <> '')
        OR
        (i.link_type <> '' AND i.src_cid <> '' AND i.dst_cid <> '')
      )
  )
  BEGIN
    -- Create temp table with the source and destination object ids. 
    -- This is used in the dynamic sql that follows since the 'deleted'
    -- pseudo-table cannot be referenced in the dynamic sql.
    SELECT
      d.link_type AS dlink_type,
      d.src_cid AS dsrc_cid,
      d.src_id AS dsrc_id,
      d.dst_cid AS ddst_cid,
      d.dst_id AS ddst_id, 
      i.link_type AS ilink_type,
      i.src_cid AS isrc_cid,
      i.src_id AS isrc_id,
      i.dst_cid AS idst_cid,
      i.dst_id AS idst_id
    INTO #update
    FROM inserted AS i JOIN deleted AS d ON
      d.SrcClass = i.SrcClass AND d.SrcMyName = i.SrcMyName AND d.SrcField = i.SrcField
      AND d.DstClass = i.DstClass AND d.DstMyName = i.DstMyName AND d.DstField = i.DstField
    WHERE
      d.link_type <> i.link_type 
      OR d.src_cid <> i.src_cid 
      OR d.dst_cid <> i.dst_cid
      AND 
      (
        (d.link_type <> '' AND d.src_cid <> '' AND d.dst_cid <> '')
        OR
        (i.link_type <> '' AND i.src_cid <> '' AND i.dst_cid <> '')
      )
    DECLARE linkCursor CURSOR FOR
      SELECT DISTINCT dlink_type, dsrc_cid, ddst_cid, ilink_type, isrc_cid, idst_cid 
      FROM #update
    OPEN linkCursor
    FETCH NEXT FROM linkCursor INTO @dlink_type, @dsrc_cid, @ddst_cid, @ilink_type, @isrc_cid, @idst_cid
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      IF @dlink_type <> '' AND @dsrc_cid <> '' AND @ddst_cid <> '' -- Must delete old records
      BEGIN
        -- Delete link records
        SET @sql = 'DELETE l FROM Object..' + RTRIM(@dsrc_cid) + '_' + RTRIM(@ddst_cid) + '_' + @dlink_type + ' AS l ' +
                   'JOIN #update AS old ON old.dsrc_id = l.src_id AND old.ddst_id = l.dst_id ' +
                   'WHERE old.dsrc_cid = ''' + @dsrc_cid + ''' ' + 
                   'AND old.ddst_cid = ''' + @ddst_cid + ''' ' + 
                   'AND old.dlink_type = ''' + @dlink_type + ''''
        PRINT 'tU_RODMManagedLink: ' + @sql
        EXEC(@sql)
    
        -- Insert LinkDeleted notifications
        INSERT Object..PendingNotification(type, fields, detail)
          SELECT
            e.type,
            @dlink_type + '|' + 
            CONVERT(VARCHAR(9), ISNULL(src.cno, 0))  + '|' +
            CONVERT(VARCHAR(9), ISNULL(u.dsrc_id, 0)) + '|' +
            CONVERT(VARCHAR(9), ISNULL(dst.cno, 0))  + '|' +
            CONVERT(VARCHAR(9), ISNULL(u.ddst_id, 0)) + '|' +
            CONVERT(VARCHAR(9), ISNULL(lt.flags, 0)),
            ''
          FROM
            Object..EventType AS e,
            #update AS u
              JOIN obj_class AS src ON src.cid = u.dsrc_cid
              JOIN obj_class AS dst ON dst.cid = u.ddst_cid
              JOIN link_type AS lt ON lt.link_type = u.dlink_type
          WHERE
            e.name = 'LinkDeleted'
            AND u.dsrc_cid = @dsrc_cid
            AND u.ddst_cid = @ddst_cid
            AND u.dlink_type = @dlink_type
      END

      IF @ilink_type <> '' AND @isrc_cid <> '' AND @idst_cid <> '' -- Insert new records
      BEGIN
        -- Insert link records
        SET @sql = 'INSERT Object..' + RTRIM(@isrc_cid) + '_' + RTRIM(@idst_cid) + '_' + @ilink_type + ' (src_id,dst_id) ' +
                   'SELECT isrc_id, idst_id FROM #update AS new ' +
                   'WHERE NOT EXISTS (SELECT * FROM Object..' + RTRIM(@isrc_cid) + '_' + RTRIM(@idst_cid) + '_' + @ilink_type + ' AS old ' +
                   'WHERE new.isrc_id = old.src_id AND new.idst_id = old.dst_id) ' +
                   'AND new.isrc_cid = ''' + @isrc_cid + ''' ' + 
                   'AND new.idst_cid = ''' + @idst_cid + ''' ' + 
                   'AND new.ilink_type = ''' + @ilink_type + ''''
        PRINT 'tU_RODMManagedLink: ' + @sql
        EXEC(@sql)
     
        -- Insert LinkCreated notifications
        INSERT Object..PendingNotification(type, fields, detail)
          SELECT
            e.type,
            @ilink_type + '|' + 
            CONVERT(VARCHAR(9), ISNULL(src.cno, 0))  + '|' +
            CONVERT(VARCHAR(9), ISNULL(u.isrc_id, 0)) + '|' +
            CONVERT(VARCHAR(9), ISNULL(dst.cno, 0))  + '|' +
            CONVERT(VARCHAR(9), ISNULL(u.idst_id, 0)) + '|' +
            CONVERT(VARCHAR(9), ISNULL(lt.flags, 0)),
            ''
          FROM
            Object..EventType AS e,
            #update AS u
              JOIN obj_class AS src ON src.cid = u.isrc_cid
              JOIN obj_class AS dst ON dst.cid = u.idst_cid
              JOIN link_type AS lt ON lt.link_type = u.ilink_type
          WHERE
            e.name = 'LinkCreated'
            AND src.cid = @isrc_cid
            AND dst.cid = @idst_cid
            AND u.ilink_type = @ilink_type
      END

      FETCH NEXT FROM linkCursor INTO @dlink_type, @dsrc_cid, @ddst_cid, @ilink_type, @isrc_cid, @idst_cid
    END
    DEALLOCATE linkCursor
  END
GO


---------------------------------------------------------------------------------------------------
-- tU_RODMManagedObject
PRINT 'Creating trigger tU_RODMManagedObject'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tU_RODMManagedObject' AND type = 'TR')
  DROP trigger tU_RODMManagedObject
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
-- Update trigger used ON the RODMManagedObject TABLE.
-- After asisp_processStagedRODMStateMsgs updates an object's attributes, this
-- trigger will CREATE exceptions AND/or messages based ON the new changes.
CREATE TRIGGER tU_RODMManagedObject ON RODMManagedObject FOR UPDATE
AS
  --PRINT 'tU_RODMManagedObject: ' + CONVERT(VARCHAR(40), GETDATE(), 113) + ': Start'
  DECLARE @id ObjID
  DECLARE @cid ClassID
  DECLARE @name VARCHAR(255)
  DECLARE @whereClause VARCHAR(255)
  DECLARE @exclusive TINYINT
  DECLARE @eventCID ClassID
  DECLARE @eventName VARCHAR(54)
  DECLARE @eventText VARCHAR(255)
  DECLARE @alertState INT
  DECLARE @priority INT
  DECLARE @oldAlertState INT
  DECLARE @oldPriority INT
  DECLARE @done TINYINT
  DECLARE @rows1 INT
  DECLARE @rows2 INT
  DECLARE @time DATETIME
  DECLARE @sql VARCHAR(2048)
  DECLARE @_RODMMyName VARCHAR(255)
  DECLARE @cname VARCHAR(24)

  DECLARE @oldOperationalState  TINYINT
  DECLARE @oldUsageState  TINYINT
  DECLARE @oldAdministrativeState TINYINT
  DECLARE @oldAvailabilityStatus TINYINT
  DECLARE @oldProceduralStatus TINYINT
  DECLARE @oldUnknownStatus TINYINT
  DECLARE @oldNativeStatus TINYINT
  DECLARE @oldDisplayStatus INT

  DECLARE @newOperationalState  TINYINT
  DECLARE @newUsageState TINYINT
  DECLARE @newAdministrativeState TINYINT
  DECLARE @newAvailabilityStatus TINYINT
  DECLARE @newProceduralStatus TINYINT
  DECLARE @newUnknownStatus TINYINT
  DECLARE @newNativeStatus TINYINT
  DECLARE @newDisplayStatus INT

  DECLARE @stateName VARCHAR(32)
  DECLARE @stateNum INT
  DECLARE @oldValue INT
  DECLARE @newValue INT

  --PRINT 'tU_RODMManagedObject: ' + CONVERT(VARCHAR(40), GETDATE(), 113) + ': Checking for updated columns'
  IF UPDATE(_operationalState) 
     OR UPDATE(_usageState)
     OR UPDATE(_administrativeState)
     OR UPDATE(_availabilityStatus)
     OR UPDATE(_proceduralStatus)
     OR UPDATE(_unknownStatus)
     OR UPDATE(_nativeStatus) 
     OR UPDATE(_DisplayStatus) 
  BEGIN
    --PRINT 'tU_RODMManagedObject: ' + CONVERT(VARCHAR(40), GETDATE(), 113) + ': Create a temp TABLE to hold the actions that will be performed. '
    -- Create a temp TABLE to hold the actions that will be performed. 
    SELECT CAST('' AS VARCHAR(2048)) AS action INTO #events

    -- copy the inserted/deleted tables to temporary tables that can be used in dynamic SQL statements
    --PRINT 'tU_RODMManagedObject: ' + CONVERT(VARCHAR(40), GETDATE(), 113) + ': SELECT * INTO #new FROM inserted'
    SELECT * INTO #new FROM inserted
    --PRINT 'tU_RODMManagedObject: ' + CONVERT(VARCHAR(40), GETDATE(), 113) + ': SELECT * INTO #old FROM deleted'
    SELECT * INTO #old FROM deleted

    --PRINT 'tU_RODMManagedObject: ' + CONVERT(VARCHAR(40), GETDATE(), 113) + ': declaring outerCursor'
    DECLARE outerCursor INSENSITIVE CURSOR FOR 
      SELECT 
        new.id,
        new.cid,
        new.name,
        new._RODMMyName,
        old._operationalState,
        old._usageState,
        old._administrativeState,
        old._availabilityStatus,
        old._proceduralStatus,
        old._unknownStatus,
        old._nativeStatus,
        old._DisplayStatus,
        new._operationalState,
        new._usageState,
        new._administrativeState,
        new._availabilityStatus,
        new._proceduralStatus,
        new._unknownStatus,
        new._nativeStatus,
        new._DisplayStatus
      FROM inserted AS new JOIN deleted AS old ON old.id = new.id 
      WHERE old._operationalState <> new._operationalState
         OR old._usageState <> new._usageState
         OR old._administrativeState <> new._administrativeState
         OR old._availabilityStatus <> new._availabilityStatus
         OR old._proceduralStatus <> new._proceduralStatus
         OR old._unknownStatus <> new._unknownStatus
         OR old._nativeStatus <> new._nativeStatus
         OR old._DisplayStatus <> new._DisplayStatus
    OPEN outerCursor
    FETCH NEXT FROM outerCursor INTO @id, @cid, @name, @_RODMMyName,
       @oldOperationalState,@oldUsageState,@oldAdministrativeState,@oldAvailabilityStatus,@oldProceduralStatus,@oldUnknownStatus,@oldNativeStatus,@oldDisplayStatus,
       @newOperationalState,@newUsageState,@newAdministrativeState,@newAvailabilityStatus,@newProceduralStatus,@newUnknownStatus,@newNativeStatus,@newDisplayStatus
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      --PRINT 'tU_RODMManagedObject: ' + CONVERT(VARCHAR(40), GETDATE(), 113) + ': cid=' + @cid + ' id=' + CAST(@id AS VARCHAR) + ' name=' + @name
      SELECT @cname = cname FROM obj_class WHERE cid = @cid

      SET @stateNum = 1
      WHILE @stateNum <= 8 BEGIN
        IF @stateNum = 1 BEGIN
          SET @stateName = 'operationalState'
          SET @oldValue = @oldOperationalState
          SET @newValue = @newOperationalState
        END
        ELSE IF @stateNum = 2 BEGIN
          SET @stateName = 'usageState'
          SET @oldValue = @oldUsageState
          SET @newValue = @newUsageState
        END
        ELSE IF @stateNum = 3 BEGIN
          SET @stateName = 'administrativeState'
          SET @oldValue = @oldAdministrativeState
          SET @newValue = @newAdministrativeState
        END
        ELSE IF @stateNum = 4 BEGIN
          SET @stateName = 'availabilityStatus'
          SET @oldValue = @oldAvailabilityStatus
          SET @newValue = @newAvailabilityStatus
        END
        ELSE IF @stateNum = 5 BEGIN
          SET @stateName = 'proceduralStatus'
          SET @oldValue = @oldProceduralStatus
          SET @newValue = @newProceduralStatus
        END
        ELSE IF @stateNum = 6 BEGIN
          SET @stateName = 'unknownStatus'
          SET @oldValue = @oldUnknownStatus
          SET @newValue = @newUnknownStatus
        END
        ELSE IF @stateNum = 7 BEGIN
          SET @stateName = 'nativeStatus'
          SET @oldValue = @oldNativeStatus
          SET @newValue = @newNativeStatus
        END
        ELSE IF @stateNum = 8 BEGIN
          SET @stateName = 'DisplayStatus'
          SET @oldValue = @oldDisplayStatus
          SET @newValue = @newDisplayStatus
        END

        IF @oldValue <> @newValue
        BEGIN
          SET @done = 0
          DECLARE innerCursor INSENSITIVE CURSOR FOR
            SELECT whereClause, exclusive, eventCID, eventName,  eventText, alertState, priority
            FROM RODMStateChangeMap
            WHERE enabled = 1 AND stateVar = @stateName
            ORDER BY salience DESC
          OPEN innerCursor
          FETCH NEXT FROM innerCursor INTO @whereClause, @exclusive, @eventCID, @eventName, 
                                           @eventText, @alertState, @priority
          WHILE @@FETCH_STATUS <> -1 AND @done = 0
          BEGIN
            -- Perform substitution of variables FROM instance data (e.g. - %name%)
            SELECT @eventName = replace(@eventName, '%name%', @name)
            SELECT @eventText = replace(@eventText, '%name%', @name)

            SELECT @eventName = replace(@eventName, '%cid%', @cid)
            SELECT @eventText = replace(@eventText, '%cid%', @cid)

            SELECT @eventName = replace(@eventName, '%cname%', @cname)
            SELECT @eventText = replace(@eventText, '%cname%', @cname)

            SELECT @eventName = replace(@eventName, '%RODMName%', @_RODMMyName)
            SELECT @eventText = replace(@eventText, '%RODMName%', @_RODMMyName)

            SELECT @eventName = replace(@eventName, '%oldValue%', CAST(@oldValue AS VARCHAR))
            SELECT @eventText = replace(@eventText, '%oldValue%', CAST(@oldValue AS VARCHAR))

            SELECT @eventName = replace(@eventName, '%newValue%', CAST(@newValue AS VARCHAR))
            SELECT @eventText = replace(@eventText, '%newValue%', CAST(@newValue AS VARCHAR))

            SELECT @eventName = replace(@eventName, '%stateName%', @stateName)
            SELECT @eventText = replace(@eventText, '%stateName%', @stateName)

            SELECT @rows1 = count(*) FROM #events

            SELECT @time = CURRENT_TIMESTAMP

            IF @eventCID = 'EXCP'
            BEGIN
              SELECT @sql = 'INSERT #events SELECT ''EXEC Object..asisp_createEXCP ' + 
                            + '@name=''''' + @eventName + ''''''
                            + ',@parent_cid=''''' + @cid + ''''''
                            + ',@parent_id=' + CAST(@id AS VARCHAR)
                            + ',@DATE=''''' + CAST(@time AS VARCHAR) + ''''''
                            + ',@TIME=''''' + CAST(@time AS VARCHAR) + ''''''
                          + ',@EXCP_NAME=''''' + @eventName + ''''''
                            + ',@EXCP_CD=''''' + @eventText + ''''''
            END
            ELSE
            BEGIN
              SELECT @sql = 'INSERT #events SELECT ''EXEC Object..asisp_createMESG ' + 
                            + '@name=''''' + @eventName + ''''''
                            + ',@parent_cid=''''' + @cid + ''''''
                            + ',@parent_id=' + CAST(@id AS VARCHAR)
                            + ',@DATE=''''' + CAST(@time AS VARCHAR) + ''''''
                            + ',@TIME=''''' + CAST(@time AS VARCHAR) + ''''''
                            + ',@MESSAGE=''''' + @eventName + ''''''
                            + ',@REASON_CD=''''' + @eventText + ''''''
            END

            IF @alertState is not null
              SELECT @sql = @sql + ',@AlertState=' + CAST(@alertState AS VARCHAR)

            IF @priority is not null
              SELECT @sql = @sql + ',@Priority=' + CAST(@priority AS VARCHAR)

            SELECT @sql = @sql + ''' FROM #new new, #old old WHERE ' + @whereClause + 
                                 ' AND new._RODMMyName = ''' + @_RODMMyName + ''' AND old._RODMMyName = ''' + @_RODMMyName + ''''
            --PRINT @sql
            EXEC(@sql)

            SELECT @rows2 = count(*) FROM #events

            IF @rows2 > @rows1 AND @exclusive = 1 SET @done = 1

            FETCH NEXT FROM innerCursor INTO @whereClause, @exclusive, @eventCID, @eventName, 
                                             @eventText, @alertState, @priority
          END
          DEALLOCATE innerCursor
        END /* the values are different */

        SET @stateNum = @stateNum + 1
      END /* FOR each state variable */

      FETCH NEXT FROM outerCursor INTO @id, @cid, @name, @_RODMMyName,
         @oldOperationalState,@oldUsageState,@oldAdministrativeState,@oldAvailabilityStatus,@oldProceduralStatus,@oldUnknownStatus,@oldNativeStatus,@oldDisplayStatus,
         @newOperationalState,@newUsageState,@newAdministrativeState,@newAvailabilityStatus,@newProceduralStatus,@newUnknownStatus,@newNativeStatus,@newDisplayStatus
    END

    --PRINT 'tU_RODMManagedObject: ' + CONVERT(VARCHAR(40), GETDATE(), 113) + ': deallocating outerCursor'
    DEALLOCATE outerCursor
  END

  IF @rows1 > 0
  BEGIN
    --PRINT 'tU_RODMManagedObject: ' + CONVERT(VARCHAR(40), GETDATE(), 113) + ': declaring eventCursor'
    DECLARE eventCursor CURSOR FOR SELECT * FROM #events
    OPEN eventCursor
    FETCH NEXT FROM eventCursor INTO @sql

    WHILE @@FETCH_STATUS <> -1
    BEGIN

      IF len(@sql) > 0
      BEGIN
        --PRINT 'tU_RODMManagedObject: ' + CONVERT(VARCHAR(40), GETDATE(), 113) + ': sql=' + @sql
        --PRINT @sql
        EXEC(@sql)
      END

      FETCH NEXT FROM eventCursor INTO @sql
    END
    --PRINT 'tU_RODMManagedObject: ' + CONVERT(VARCHAR(40), GETDATE(), 113) + ': deallocating eventCursor'
    DEALLOCATE eventCursor
  END
  --PRINT 'tU_RODMManagedObject: ' + CONVERT(VARCHAR(40), GETDATE(), 113) + ': finish'
GO


USE Object
GO

---------------------------------------------------------------------------------------------------
-- RODMManagedObject
PRINT 'Creating view RODMManagedObject'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMManagedObject' AND type = 'V')
  DROP VIEW RODMManagedObject
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
CREATE VIEW RODMManagedObject AS SELECT * FROM RODM..RODMManagedObject
GO


---------------------------------------------------------------------------------------------------
-- RODMManagedLink
PRINT 'Creating view RODMManagedLink'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMManagedLink' AND type = 'V')
  DROP VIEW RODMManagedLink
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
CREATE VIEW RODMManagedLink AS SELECT * FROM RODM..RODMManagedLink
GO


---------------------------------------------------------------------------------------------------
-- DROP TABLE RODMDisplayResourceType
-- Maps SNA/TM Class/Role specifiers to icons
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMDisplayResourceType' AND type = 'U')
BEGIN
  PRINT 'Creating table RODMDisplayResourceType'
  EXEC("
  CREATE TABLE RODMDisplayResourceType
  (
    Class       VARCHAR(60)           CONSTRAINT RODMDisplayResourceType_PK PRIMARY KEY (Class, Role),
    Role        VARCHAR(60),
    Value       VARCHAR(60) NULL
  )
  ")
  EXEC("
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'aggregateGraph2', 'nnDomain', 'DUIXC_RTN_NN_DOMAIN_AGG'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'aggregateGraph2', 'nnDomainNetwork', 'DUIXC_RTN_NN_DOMAIN_NETWORK'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'aggregateGraph2', 'nnDomainNetworkCluster', 'DUIXC_RTN_NN_DOM_NET_CLUSTER'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'appnBrNN', '', 'DUIXC_RTN_APPN_BRNN'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'appnEN', '', 'DUIXC_RTN_APPN_EN'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'appnNN', '', 'DUIXC_RTN_APPN_NN'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'appnTransmissionGroup', '', 'DUIXC_LTN_APPN_TRN_GROUP'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'appnTransmissionGroupCircuit', '', 'DUIXC_LTN_APPN_TG_CIRCUIT'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'appnTransmissionGroupCircuit', 'appnTransGroupCircuitCN', 'DUIXC_LTN_APPN_TG_CIR_CN'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'appnTransmissionGroupCircuit', 'ntriTypeAppnTgCircuit', 'DUIXC_LTN_APPN_TG_CIR_NTRI'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'circuit2', 'interdomainCircuit', 'DUIXC_LTN_INT_DOMAIN_CIR_AGG'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'circuit2', 'interdomainNetworkCircuit', 'DUIXC_LTN_INT_DOM_NET_CIR_AGG'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'crossDomainResource', '', 'DUIXC_RTN_CROSS_DOMAIN_RES'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'crossDomainResourceManager', '', 'DUIXC_RTN_CDRM'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'definitionGroup', '', 'DUIXC_RTN_DEFINITION_GROUP'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'interchangeNode', '', 'DUIXC_RTN_INTERCHANGE_NODE'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'lenNode', '', 'DUIXC_RTN_LEN_NODE'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'logicalLink', '', 'DUIXC_RTN_LOGICAL_LINK_PU'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'logicalUnit', '', 'DUIXC_RTS_LU'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'luGroup', '', 'DUIXC_RTN_LU_GROUP'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'migrationDataHost', '', 'DUIXC_RTN_MIGRATION_DATA_HOST'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'port', '', 'DUIXC_RTN_PORT'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'snaLocalTopo', 'enLocalTopology', 'DUIXC_RTN_EN_LOCAL_TOP_AGG'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'snaLocalTopo', 'nnLocalTopology', 'DUIXC_RTN_NN_LOCAL_TOP_AGG'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'snaNode', '', 'DUIXC_RTN_SNA_NODE'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'subareaTransmissionGroupCircuit', '', 'DUIXC_LTN_SUBAREA_TG_CIR_AGG'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 't2-1Node', '', 'DUIXC_RTN_T2_1_NODE'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 't4Node', '', 'DUIXC_RTS_PU4'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 't4Node', 't4NodeGateway', 'DUIXC_RTS_GATEWAY_NCP'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 't5Node', '', 'DUIXC_RTS_HOST'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'virtualRoutingNode', '', 'DUIXC_RTN_VIR_ROUTING_NODE'
  INSERT RODMDisplayResourceType(Class, Role, Value) SELECT 'SNANetwork', '', 'DUIXC_RTA_NN_DOM_NET_CLUSTER'
  ")
END


---------------------------------------------------------------------------------------------------
-- DROP TABLE RODMIconCrossRef
-- Maps SNA/TM Class/Role specifiers to icons
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMIconCrossRef' AND type = 'U')
BEGIN
  PRINT 'Creating table RODMIconCrossRef'
  EXEC("
  CREATE TABLE RODMIconCrossRef
  (
    IconName            VARCHAR(8)    CONSTRAINT SNAIconCrossRef_PK PRIMARY KEY(IconName, DisplayResourceType),
    DisplayResourceType VARCHAR(60)
  )
  ")
  EXEC("
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1C00', 'DUIXC_RTN_CLUSTER_USER1'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1C01', 'DUIXC_RTN_CLUSTER_USER2'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1C02', 'DUIXC_RTN_CLUSTER_USER3'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1C03', 'DUIXC_RTN_CLUSTER_USER4'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1C04', 'DUIXC_RTN_CLUSTER_USER5'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1C05', 'DUIXC_RTN_CLUSTER_USER6'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1C06', 'DUIXC_RTN_CLUSTER_USER7'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1C07', 'DUIXC_RTN_CLUSTER_USER8'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1C08', 'DUIXC_RTN_CLUSTER_USER9'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1C09', 'DUIXC_RTN_CLUSTER_USER10'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1C0A', 'DUIXC_RTN_CLUSTER_USER11'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1C0B', 'DUIXC_RTN_CLUSTER_USER12'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1C0C', 'DUIXC_RTN_CLUSTER_USER13'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1C0D', 'DUIXC_RTN_CLUSTER_USER14'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1C0E', 'DUIXC_RTN_CLUSTER_USER15'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1C0F', 'DUIXC_RTN_CLUSTER_USER16'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1N00', 'DUIXC_RTA_NN_DOM_NET_CLUSTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1N00', 'DUIXC_RTN_IP_NETWORKS'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1N00', 'DUIXC_RTN_IP_NETWORK_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1N00', 'DUIXC_RTN_LMU_NETWORK_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1N00', 'DUIXC_RTN_NN_DOM_NET_CLUSTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU1N00', 'DUIXC_RTN_NTF_DOM_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2C00', 'DUIXC_RTN_HOST_USER1'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2C01', 'DUIXC_RTN_HOST_USER2'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2C02', 'DUIXC_RTN_HOST_USER3'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2C03', 'DUIXC_RTN_HOST_USER4'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2C04', 'DUIXC_RTN_HOST_USER5'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2C05', 'DUIXC_RTN_HOST_USER6'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2C19', 'DUIXC_RTN_HOST_AGG_USER10'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2C1A', 'DUIXC_RTN_HOST_AGG_USER11'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2C1B', 'DUIXC_RTN_HOST_AGG_USER12'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2C1C', 'DUIXC_RTN_HOST_AGG_USER13'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2C1D', 'DUIXC_RTN_HOST_AGG_USER14'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2C1E', 'DUIXC_RTN_HOST_AGG_USER15'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2C1F', 'DUIXC_RTN_HOST_AGG_USER16'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2G00', 'DUIXC_RTA_EN_LOCAL_TOP_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2G00', 'DUIXC_RTA_NN_LOCAL_TOP_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2G00', 'DUIXC_RTN_CICSPLEX_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2G00', 'DUIXC_RTN_CICS_SYSTEM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2G00', 'DUIXC_RTN_DH_LOCAL_TOP_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2G00', 'DUIXC_RTN_DOMAIN'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2G00', 'DUIXC_RTN_EN_LOCAL_TOP_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2G00', 'DUIXC_RTN_HOST'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2G00', 'DUIXC_RTN_HOST_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2G00', 'DUIXC_RTN_IC_LOCAL_TOP_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2G00', 'DUIXC_RTN_NN_LOCAL_TOP_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2G00', 'DUIXC_RTN_PROCESSOR_HOST'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2G00', 'DUIXC_RTN_SNA_LOCAL_TOP_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2G00', 'DUIXC_RTN_SYSTEM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2G00', 'DUIXC_RTN_SYSTEM_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2N00', 'DUIXC_RTN_AS_400'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2N00', 'DUIXC_RTN_AS_400_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2N00', 'DUIXC_RTN_INTERNET_HOST'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2N00', 'DUIXC_RTN_INTERNET_HOST_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2N00', 'DUIXC_RTN_MINI'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2N00', 'DUIXC_RTN_MINI_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2N00', 'DUIXC_RTS_AS_400'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU2N00', 'DUIXC_RTS_AS_400_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C00', 'DUIXC_RTN_APPL_USER1'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C01', 'DUIXC_RTN_APPL_USER2'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C02', 'DUIXC_RTN_APPL_USER3'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C03', 'DUIXC_RTN_APPL_USER4'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C04', 'DUIXC_RTN_APPL_USER5'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C05', 'DUIXC_RTN_APPL_USER6'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C06', 'DUIXC_RTN_APPL_USER7'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C07', 'DUIXC_RTN_APPL_USER8'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C08', 'DUIXC_RTN_APPL_USER9'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C09', 'DUIXC_RTN_APPL_USER10'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C0A', 'DUIXC_RTN_APPL_USER11'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C0B', 'DUIXC_RTN_APPL_USER12'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C0C', 'DUIXC_RTN_APPL_USER13'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C0D', 'DUIXC_RTN_APPL_USER14'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C0E', 'DUIXC_RTN_APPL_USER15'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C0F', 'DUIXC_RTN_APPL_USER16'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C10', 'DUIXC_RTN_APPL_AGG_USER1'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C11', 'DUIXC_RTN_APPL_AGG_USER2'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C12', 'DUIXC_RTN_APPL_AGG_USER3'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C13', 'DUIXC_RTN_APPL_AGG_USER4'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C14', 'DUIXC_RTN_APPL_AGG_USER5'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C15', 'DUIXC_RTN_APPL_AGG_USER6'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C16', 'DUIXC_RTN_APPL_AGG_USER7'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C17', 'DUIXC_RTN_APPL_AGG_USER8'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C18', 'DUIXC_RTN_APPL_AGG_USER9'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C19', 'DUIXC_RTN_APPL_AGG_USER10'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C1A', 'DUIXC_RTN_APPL_AGG_USER11'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C1B', 'DUIXC_RTN_APPL_AGG_USER12'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C1C', 'DUIXC_RTN_APPL_AGG_USER13'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C1D', 'DUIXC_RTN_APPL_AGG_USER14'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C1E', 'DUIXC_RTN_APPL_AGG_USER15'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3C1F', 'DUIXC_RTN_APPL_AGG_USER16'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_3270_EMUL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_ACCESS_SUPPORT'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_AGENT'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_APPL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_APPL_ACCESS_PATH'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_APPL_GROUP'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_APPL_SUBSYS'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_APP_LAN_DOMAIN_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_ATM_ADAPT_LAYER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_BRIDGE_APPL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_BUFFER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_BUFFER_POOL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_CNTL_PT_APPL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_COMM_SER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_CONN'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_CONTROL_PGM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_DATASET'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_DATASET_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_DATA_TABLE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_DBM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_DB_REQ'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_DISK_JOURNAL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_DOMAIN_MGR'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_DSA'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_ENTITY_MGR'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_EXTRAPART_TDQ'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_FILE_SER_APPL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_FIRMWARE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_HOST_ACCESS_PATH'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_INDIRECT_TDQ'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_INTRAPART_TDQ'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_IP_MANAGER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_LANDP_MANAGER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_LANGUAGE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_LAN_NETWORK_MGR'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_LAN_REQ'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_LAN_SERVER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_LMU_MANAGER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_LMU_MANAGING_LAN_R'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_LMU_MANAGING_LAN_S'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_LMU_MANAGING_MAC'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_LMU_MANAGING_NW_R'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_LMU_MANAGING_NW_S'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_LMU_MANAGING_WINDOW'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_LOCAL_FILE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_LOCAL_TRANS'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_LSRPOOL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_LU62_CONN'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_LU62_CONV'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_LU62_SESSION'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_MANAGER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_MEMORY'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_MICROCODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_MODENAME'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_MONITOR'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_MONITOR_MGR'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_MRO_CONN'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_MRO_CONV'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_MRO_SESSION'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_NETFIN_MGR'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_NETFIN_SERVICES'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_NETWORK_INTFACE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_NETWORK_MGR'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_NW_MANAGER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_OPERAT_SYS'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_PARALLEL_BRIDGE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_PERF_MONITOR'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_PGM_PROCESS'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_PROGRAM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_PROTOCOL_HANDLER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_PRT_SER_APPL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_QUEUE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_QUEUE_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_REL_DATA_BASE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_REMOTE_BRIDGE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_REMOTE_FILE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_REMOTE_TRANS'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_REM_OPS_AGENT'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_ROUTE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_SERVER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_SESSION'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_SMF_JOURNAL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_SYSDUMP_CODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_SYSTEM_MGR'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_TAPE_JOURNAL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_TASK'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_TEMP_STOR_QUEUE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_TOOLS'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_TP_MONITOR'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_TRACE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_TRANDUMP_CODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_TRANSACTION_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_TRANS_CLASS'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_USER_APPL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTN_UTILITY'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTS_CNTL_PT_APPL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTS_COMM_SER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTS_SER_PT_APPL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTS_SPC_USE_APP_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU3G00', 'DUIXC_RTS_VTAM_APPL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C00', 'DUIXC_RTN_NODE_USER1'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C01', 'DUIXC_RTN_NODE_USER2'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C02', 'DUIXC_RTN_NODE_USER3'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C03', 'DUIXC_RTN_NODE_USER4'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C04', 'DUIXC_RTN_NODE_USER5'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C05', 'DUIXC_RTN_NODE_USER6'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C06', 'DUIXC_RTN_NODE_USER7'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C07', 'DUIXC_RTN_NODE_USER8'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C08', 'DUIXC_RTN_NODE_USER9'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C09', 'DUIXC_RTN_NODE_USER10'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C0A', 'DUIXC_RTN_NODE_USER11'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C0B', 'DUIXC_RTN_NODE_USER12'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C0C', 'DUIXC_RTN_NODE_USER13'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C0D', 'DUIXC_RTN_NODE_USER14'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C0E', 'DUIXC_RTN_NODE_USER15'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C0F', 'DUIXC_RTN_NODE_USER16'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C10', 'DUIXC_RTN_NODE_AGG_USER1'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C11', 'DUIXC_RTN_NODE_AGG_USER2'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C12', 'DUIXC_RTN_NODE_AGG_USER3'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C13', 'DUIXC_RTN_NODE_AGG_USER4'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C14', 'DUIXC_RTN_NODE_AGG_USER5'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C15', 'DUIXC_RTN_NODE_AGG_USER6'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C16', 'DUIXC_RTN_NODE_AGG_USER7'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C17', 'DUIXC_RTN_NODE_AGG_USER8'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C18', 'DUIXC_RTN_NODE_AGG_USER9'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C19', 'DUIXC_RTN_NODE_AGG_USER10'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C1A', 'DUIXC_RTN_NODE_AGG_USER11'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C1B', 'DUIXC_RTN_NODE_AGG_USER12'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C1C', 'DUIXC_RTN_NODE_AGG_USER13'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C1D', 'DUIXC_RTN_NODE_AGG_USER14'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C1E', 'DUIXC_RTN_NODE_AGG_USER15'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4C1F', 'DUIXC_RTN_NODE_AGG_USER16'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_CDRM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_CHANNEL_BANK'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_COLL_OF_SPOTS_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_COMM_CTRL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_CONTROL_PT_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_COOLING_UNIT'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_DEFINITION_GROUP'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_DIAL_BU_UNIT'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_DOMAIN_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_DOM_CTRL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_EMS'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_ETHERNET_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_FAN'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_GENERIC_MAJ_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_GENERIC_MIN_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_GEN_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_INTERNET'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_INTERNET_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_INTERNET_ROUTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_INTERNET_ROUTER_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_LU_GROUP_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_NETWORK_MGMT'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_NETWORK_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_NODE_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_POWER_SUPPLY'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_PROCESSOR_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_SERVICE_PROCESSOR'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_SYSPLEX_TIMER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_SYSPLEX_TIMER_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTN_TEST_EQUIP'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTS_APPL_MAJ_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTS_CDRM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTS_CDRM_MAJ_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTS_CDRSC'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTS_CDRSC_MAJ_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTS_CHAN_ATT_MAJ_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTS_CONTROL_PT_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTS_ICA_MAJ_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTS_LAN_MAJ_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTS_LINK_STATION'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTS_LOC_NSNA_MAJ_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTS_LOC_SNA_MAJ_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTS_PACKET_MAJ_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTS_SER_PT_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4G00', 'DUIXC_RTS_XCA_MAJ_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N00', 'DUIXC_RTN_ATM_CLUSTER_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N00', 'DUIXC_RTN_FEP'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N00', 'DUIXC_RTN_FEP_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N00', 'DUIXC_RTN_FEP_HOST'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N01', 'DUIXC_RTN_ADDL_SERVER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N01', 'DUIXC_RTN_FILE_SERVER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N01', 'DUIXC_RTN_FILE_SER_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N01', 'DUIXC_RTN_IO_MODULE_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N01', 'DUIXC_RTN_LAN_FILE_SERVER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N01', 'DUIXC_RTN_NETFIN_SERVER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N01', 'DUIXC_RTN_NODE_LAN_SERVER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N01', 'DUIXC_RTN_NW_FILE_SERVER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N01', 'DUIXC_RTN_TME10_SERVER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N02', 'DUIXC_RTN_BRIDGE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N02', 'DUIXC_RTN_BRIDGE_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N02', 'DUIXC_RTN_GATEWAY_ROUTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N02', 'DUIXC_RTN_IP_SEGMENT_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N02', 'DUIXC_RTN_LAN_BRIDGE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N02', 'DUIXC_RTN_WAN_BRIDGE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N03', 'DUIXC_RTN_CLSTR_CTRL_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N03', 'DUIXC_RTN_CLUSTER_CTRL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N04', 'DUIXC_RTA_APPN_NN'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N04', 'DUIXC_RTN_APPN_NN'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N04', 'DUIXC_RTN_CONTROL_UNIT'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N04', 'DUIXC_RTN_PC_NETWORK_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N04', 'DUIXC_RTN_PC_NET_SEG_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N05', 'DUIXC_RTN_8230_CAU'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N05', 'DUIXC_RTN_ATM_CONC'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N05', 'DUIXC_RTN_CONCENTRATOR'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N05', 'DUIXC_RTN_CONCENTRATOR_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N05', 'DUIXC_RTN_CTRLED_ACC_UNIT'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N05', 'DUIXC_RTN_LAN_CONCENT_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N05', 'DUIXC_RTN_MAU'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N05', 'DUIXC_RTN_REMOTE_LINE_CONC'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N05', 'DUIXC_RTN_WIRING_CLOSET'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N06', 'DUIXC_RTA_INTERCHANGE_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N06', 'DUIXC_RTN_ATM_BRIDGE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N06', 'DUIXC_RTN_COMM_GATEWAY'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N06', 'DUIXC_RTN_COMM_GATEWAY_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N06', 'DUIXC_RTN_GATEWAY'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N06', 'DUIXC_RTN_GW_IWS'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N06', 'DUIXC_RTN_INTERCHANGE_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N06', 'DUIXC_RTN_PROTOCOL_CONVERTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N06', 'DUIXC_RTS_COMM_GATEWAY'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N06', 'DUIXC_RTS_COMM_GATEWAY_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N07', 'DUIXC_RTN_MUX'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N07', 'DUIXC_RTN_MUX_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N08', 'DUIXC_RTN_LAN_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N08', 'DUIXC_RTN_LAN_DB_NET_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N08', 'DUIXC_RTN_LAN_NETWORK_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N08', 'DUIXC_RTN_LAN_SEGMENT'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N08', 'DUIXC_RTN_LAN_SEGMENT2_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N08', 'DUIXC_RTN_LAN_SEGMENT_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N08', 'DUIXC_RTN_LAN_SVR_NET_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N08', 'DUIXC_RTN_LMU_NETWORKS'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N08', 'DUIXC_RTN_MAN_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N08', 'DUIXC_RTN_TR_SEGMENT'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N08', 'DUIXC_RTN_TR_SEGMENT_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N08', 'DUIXC_RTN_WORKGROUP_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N09', 'DUIXC_RTN_T1_RES_MGR'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0A', 'DUIXC_RTN_LAN_PRINT_SERVER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0A', 'DUIXC_RTN_NT_SERVER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0A', 'DUIXC_RTN_PRINT_SERVER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0A', 'DUIXC_RTN_PRINT_SER_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0B', 'DUIXC_RTN_PBX'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0B', 'DUIXC_RTN_PBX_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0C', 'DUIXC_RTN_APPN_SUBNET_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0C', 'DUIXC_RTN_APPN_SUBNET_BN_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0D', 'DUIXC_RTA_PORT'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0D', 'DUIXC_RTN_ATM_DEVICE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0D', 'DUIXC_RTN_ATM_MSS'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0D', 'DUIXC_RTN_ATM_SWITCH'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0D', 'DUIXC_RTN_DSU'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0D', 'DUIXC_RTN_LAN_ROUTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0D', 'DUIXC_RTN_MATRIX_SWITCH'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0D', 'DUIXC_RTN_PORT'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0D', 'DUIXC_RTN_ROUTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0D', 'DUIXC_RTN_SWITCH'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0D', 'DUIXC_RTN_SWITCH_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0D', 'DUIXC_RTN_WAN_ROUTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0D', 'DUIXC_RTS_SWITCHED_MAJ_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0E', 'DUIXC_RTA_T2_1_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0E', 'DUIXC_RTN_SNA_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0E', 'DUIXC_RTN_T2_1_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0E', 'DUIXC_RTS_PU1'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0E', 'DUIXC_RTS_PU1_ETHERNET'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0E', 'DUIXC_RTS_PU1_FRAME_RELAY'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0E', 'DUIXC_RTS_PU1_NTRI'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0F', 'DUIXC_RTA_VIR_ROUTING_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0F', 'DUIXC_RTN_IP_SUBNET_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0F', 'DUIXC_RTN_NETWORK_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0F', 'DUIXC_RTN_NT_DOM_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N0F', 'DUIXC_RTN_VIR_ROUTING_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N10', 'DUIXC_RTA_NN_DOMAIN_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N10', 'DUIXC_RTN_LAN_DOMAIN_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N10', 'DUIXC_RTN_LMU_DOMAIN_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N10', 'DUIXC_RTN_NN_DOMAIN_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N10', 'DUIXC_RTN_NT_BKUP_CTRL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N10', 'DUIXC_RTN_NT_PRI_CTRL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU4N10', 'DUIXC_RTN_TME10_REGION_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C00', 'DUIXC_RTN_TERM_USER1'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C01', 'DUIXC_RTN_TERM_USER2'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C02', 'DUIXC_RTN_TERM_USER3'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C03', 'DUIXC_RTN_TERM_USER4'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C04', 'DUIXC_RTN_TERM_USER5'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C05', 'DUIXC_RTN_TERM_USER6'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C06', 'DUIXC_RTN_TERM_USER7'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C07', 'DUIXC_RTN_TERM_USER8'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C08', 'DUIXC_RTN_TERM_USER9'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C09', 'DUIXC_RTN_TERM_USER10'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C0A', 'DUIXC_RTN_TERM_USER11'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C0B', 'DUIXC_RTN_TERM_USER12'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C0C', 'DUIXC_RTN_TERM_USER13'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C0D', 'DUIXC_RTN_TERM_USER14'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C0E', 'DUIXC_RTN_TERM_USER15'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C0F', 'DUIXC_RTN_TERM_USER16'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C10', 'DUIXC_RTN_TERM_AGG_USER1'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C11', 'DUIXC_RTN_TERM_AGG_USER2'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C12', 'DUIXC_RTN_TERM_AGG_USER3'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C13', 'DUIXC_RTN_TERM_AGG_USER4'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C14', 'DUIXC_RTN_TERM_AGG_USER5'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C15', 'DUIXC_RTN_TERM_AGG_USER6'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C16', 'DUIXC_RTN_TERM_AGG_USER7'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C17', 'DUIXC_RTN_TERM_AGG_USER8'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C18', 'DUIXC_RTN_TERM_AGG_USER9'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C19', 'DUIXC_RTN_TERM_AGG_USER10'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C1A', 'DUIXC_RTN_TERM_AGG_USER11'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C1B', 'DUIXC_RTN_TERM_AGG_USER12'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C1C', 'DUIXC_RTN_TERM_AGG_USER13'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C1D', 'DUIXC_RTN_TERM_AGG_USER14'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C1E', 'DUIXC_RTN_TERM_AGG_USER15'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5C1F', 'DUIXC_RTN_TERM_AGG_USER16'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTA_LEN_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_ATM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_CARD_RDR'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_CARD_RDR_PUNCH'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_CHECK_RDR'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_CLIENT_TERMINAL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_COIN_FEED'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_DL_ENTITY'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_END_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_ENVELOPE_DEP'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_FAX'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_GENERIC_TERM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_KEYBOARD'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_LEN_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_MAG_STRIPE_RDR'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_MAINT_DEVICE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_NPT_TERMINAL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_OPTICAL_SCANNER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_PHYS_LAYER_ENTITY'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_POS_TERM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_RJE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_SCANNER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_SELECTOR_PEN'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_SENSOR'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_TAPE_DRIVE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTN_THERMAL_SENSOR'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTS_NPT_TERMINAL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5G00', 'DUIXC_RTS_NTO_TERMINAL'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTA_APPN_EN'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_AIX_SYSTEM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_APPN_BRNN'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_APPN_EN'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_CROSS_DOMAIN_RES'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_DOS_SYSTEM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_LAN_REQUESTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_LMU_MANAGED_LAN_R'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_LMU_MANAGED_LAN_S'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_LMU_MANAGED_MAC'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_LMU_MANAGED_NW_R'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_LMU_MANAGED_NW_S'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_LMU_MANAGED_WINDOWS'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_LOGICAL_UNIT'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_LT_CLIENT_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_LU_GROUP'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_MAC_SYSTEM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_NETFIN_CLIENT'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_NETWARE_SYSTEM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_NT_WRKSTN'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_NW_FILE_REQUESTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_OS2_SYSTEM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_OS400_SYSTEM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_PWS_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_TME10_CLIENT'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_UNIX_SYSTEM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_WIN95_SYSTEM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_WIN95_WRKSTN'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_WINNT_SYSTEM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_WIN_SYSTEM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTN_WORK_STN'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTS_GENERIC_PU'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTS_LU'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N00', 'DUIXC_RTS_SPC_USE_LU_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N01', 'DUIXC_RTN_8209_ADAPTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N01', 'DUIXC_RTN_ADAPTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N01', 'DUIXC_RTN_ATM_INTRN_PORT'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N01', 'DUIXC_RTN_CAU_ADAPTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N01', 'DUIXC_RTN_CHAN_ADAPTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N01', 'DUIXC_RTN_COMM_ADAPTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N01', 'DUIXC_RTN_DIR_ATT_LK_ADAPTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N01', 'DUIXC_RTN_IP_INTERFACE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N01', 'DUIXC_RTN_LAN_ADAPTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N01', 'DUIXC_RTN_LAN_BRIDGE_ADPTR'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N01', 'DUIXC_RTN_LAN_STATION_ADPTR'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N01', 'DUIXC_RTN_LOOP_ADAPTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N01', 'DUIXC_RTN_PORT2'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU5N01', 'DUIXC_RTN_STATION_ADAPTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C00', 'DUIXC_RTN_TRAN_USER1'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C01', 'DUIXC_RTN_TRAN_USER2'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C02', 'DUIXC_RTN_TRAN_USER3'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C03', 'DUIXC_RTN_TRAN_USER4'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C04', 'DUIXC_RTN_TRAN_USER5'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C05', 'DUIXC_RTN_TRAN_USER6'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C06', 'DUIXC_RTN_TRAN_USER7'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C07', 'DUIXC_RTN_TRAN_USER8'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C08', 'DUIXC_RTN_TRAN_USER9'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C09', 'DUIXC_RTN_TRAN_USER10'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C0A', 'DUIXC_RTN_TRAN_USER11'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C0B', 'DUIXC_RTN_TRAN_USER12'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C0C', 'DUIXC_RTN_TRAN_USER13'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C0D', 'DUIXC_RTN_TRAN_USER14'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C0E', 'DUIXC_RTN_TRAN_USER15'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C0F', 'DUIXC_RTN_TRAN_USER16'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C10', 'DUIXC_RTN_TRAN_AGG_USER1'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C11', 'DUIXC_RTN_TRAN_AGG_USER2'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C12', 'DUIXC_RTN_TRAN_AGG_USER3'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C13', 'DUIXC_RTN_TRAN_AGG_USER4'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C14', 'DUIXC_RTN_TRAN_AGG_USER5'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C15', 'DUIXC_RTN_TRAN_AGG_USER6'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C16', 'DUIXC_RTN_TRAN_AGG_USER7'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C17', 'DUIXC_RTN_TRAN_AGG_USER8'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C18', 'DUIXC_RTN_TRAN_AGG_USER9'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C19', 'DUIXC_RTN_TRAN_AGG_USER10'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C1A', 'DUIXC_RTN_TRAN_AGG_USER11'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C1B', 'DUIXC_RTN_TRAN_AGG_USER12'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C1C', 'DUIXC_RTN_TRAN_AGG_USER13'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C1D', 'DUIXC_RTN_TRAN_AGG_USER14'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C1E', 'DUIXC_RTN_TRAN_AGG_USER15'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6C1F', 'DUIXC_RTN_TRAN_AGG_USER16'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6G00', 'DUIXC_RTA_LOGICAL_LINK_PU'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6G00', 'DUIXC_RTN_BRIDGE_CONN'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6G00', 'DUIXC_RTN_CSU'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6G00', 'DUIXC_RTN_DTS'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6G00', 'DUIXC_RTN_ECHO_CANCELLER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6G00', 'DUIXC_RTN_ECS'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6G00', 'DUIXC_RTN_GEN_TRANSCEIVER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6G00', 'DUIXC_RTN_LOGICAL_LINK_PU'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6G00', 'DUIXC_RTN_MULTIPORT_JUNCTION'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6G00', 'DUIXC_RTN_PAD'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6G00', 'DUIXC_RTN_PKGE_SWITCH_EQUIP'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6G00', 'DUIXC_RTN_REPEAT_LINE_CNVERT'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6G00', 'DUIXC_RTN_TRANSCEIVER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6G00', 'DUIXC_RTN_VIDEO_EQUIP'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6G00', 'DUIXC_RTS_TRANSCEIVER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6N00', 'DUIXC_RTN_CRPTO_UNIT'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6N00', 'DUIXC_RTN_ENCRYPTOR'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6N01', 'DUIXC_RTN_MICROWAVE_EQUIP'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6N01', 'DUIXC_RTN_SATELLITE_EQUIP'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6N02', 'DUIXC_RTN_ANALOG_MODEM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6N02', 'DUIXC_RTN_DIGITAL_MODEM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6N02', 'DUIXC_RTN_GENERIC_MODEM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU6N02', 'DUIXC_RTN_RF_MODEM'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C00', 'DUIXC_RTN_CONN_USER1'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C01', 'DUIXC_RTN_CONN_USER2'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C02', 'DUIXC_RTN_CONN_USER3'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C03', 'DUIXC_RTN_CONN_USER4'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C04', 'DUIXC_RTN_CONN_USER5'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C05', 'DUIXC_RTN_CONN_USER6'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C06', 'DUIXC_RTN_CONN_USER7'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C07', 'DUIXC_RTN_CONN_USER8'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C08', 'DUIXC_RTN_CONN_USER9'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C09', 'DUIXC_RTN_CONN_USER10'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C0A', 'DUIXC_RTN_CONN_USER11'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C0B', 'DUIXC_RTN_CONN_USER12'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C0C', 'DUIXC_RTN_CONN_USER13'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C0D', 'DUIXC_RTN_CONN_USER14'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C0E', 'DUIXC_RTN_CONN_USER15'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C0F', 'DUIXC_RTN_CONN_USER16'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C10', 'DUIXC_RTN_CONN_AGG_USER1'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C11', 'DUIXC_RTN_CONN_AGG_USER2'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C12', 'DUIXC_RTN_CONN_AGG_USER3'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C13', 'DUIXC_RTN_CONN_AGG_USER4'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C14', 'DUIXC_RTN_CONN_AGG_USER5'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C15', 'DUIXC_RTN_CONN_AGG_USER6'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C16', 'DUIXC_RTN_CONN_AGG_USER7'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C17', 'DUIXC_RTN_CONN_AGG_USER8'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C18', 'DUIXC_RTN_CONN_AGG_USER9'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C19', 'DUIXC_RTN_CONN_AGG_USER10'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C1A', 'DUIXC_RTN_CONN_AGG_USER11'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C1B', 'DUIXC_RTN_CONN_AGG_USER12'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C1C', 'DUIXC_RTN_CONN_AGG_USER13'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C1D', 'DUIXC_RTN_CONN_AGG_USER14'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C1E', 'DUIXC_RTN_CONN_AGG_USER15'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7C1F', 'DUIXC_RTN_CONN_AGG_USER16'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7G00', 'DUIXC_RTN_COUPLING_FACILITY'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7G00', 'DUIXC_RTN_COUPLING_FAC_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7G00', 'DUIXC_RTN_IP_BRIDGE_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7G00', 'DUIXC_RTN_IP_HUB_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'DUIU7G00', 'DUIXC_RTN_IP_ROUTER_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0003IND', 'DUIXC_RTN_DOC_FEED'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0003IND', 'DUIXC_RTN_PAPER_FEED'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0003IND', 'DUIXC_RTN_PLOTTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0003IND', 'DUIXC_RTN_POSTSCRIPT_PRT'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0003IND', 'DUIXC_RTN_PRINTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0109IND', 'DUIXC_RTN_DASD'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0109IND', 'DUIXC_RTN_VOLUME'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0109IND', 'DUIXC_RTN_VOL_GROUP_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0951IND', 'DUIXC_RTA_NN_DOMAIN_NETWORK'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0951IND', 'DUIXC_RTN_APPN_NETWORK'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0951IND', 'DUIXC_RTN_APPN_NET_CLUSTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0951IND', 'DUIXC_RTN_COMM_SITE_CLUSTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0951IND', 'DUIXC_RTN_DATA_CNTR_CLUSTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0951IND', 'DUIXC_RTN_GROUP_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0951IND', 'DUIXC_RTN_IP_LOCATION'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0951IND', 'DUIXC_RTN_MIXED_CLUSTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0951IND', 'DUIXC_RTN_NN_DOMAIN_NETWORK'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0951IND', 'DUIXC_RTN_NON_SNA_CLUSTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0951IND', 'DUIXC_RTN_OPER_VIEW_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0951IND', 'DUIXC_RTN_POLICY_REGION_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0951IND', 'DUIXC_RTN_REGION_CLUSTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0951IND', 'DUIXC_RTN_SNA_CLUSTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0951IND', 'DUIXC_RTN_SNA_DOMAIN_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0951IND', 'DUIXC_RTN_SUBGROUP_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0951IND', 'DUIXC_RTN_SYSTEM_CLUSTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0951IND', 'DUIXC_RTS_CLUSTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0951IND', 'DUIXC_RTS_COMM_SITE_CLUSTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0951IND', 'DUIXC_RTS_DATA_CNTR_CLUSTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0951IND', 'DUIXC_RTS_SYSTEM_CLUSTER'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0952IND', 'DUIXC_RTA_MIGRATION_DATA_HOST'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0952IND', 'DUIXC_RTN_MIGRATION_DATA_HOST'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0952IND', 'DUIXC_RTN_SNA_DOMAIN'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0952IND', 'DUIXC_RTS_GATEWAY_HOST'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0952IND', 'DUIXC_RTS_GATEWAY_HOST_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0952IND', 'DUIXC_RTS_HOST'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0952IND', 'DUIXC_RTS_HOST_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0953IND', 'DUIXC_RTS_GATEWAY_NCP'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0953IND', 'DUIXC_RTS_GATEWAY_NCP_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0953IND', 'DUIXC_RTS_PU4'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0953IND', 'DUIXC_RTS_PU4_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0954IND', 'DUIXC_RTN_IP_SYSTEM_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0954IND', 'DUIXC_RTN_LAN_WKSTN_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0954IND', 'DUIXC_RTN_MGD_NODE_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0954IND', 'DUIXC_RTN_OPEN_SYSTEM_AGG'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0954IND', 'DUIXC_RTS_PU20'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'I0955IND', 'DUIXC_RTS_PU21'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'NONE', 'DUIXC_RTN_GEN_NULL_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'NONE', 'DUIXC_RTN_NULL_NODE'
  INSERT RODMIconCrossRef(IconName, DisplayResourceType) SELECT 'NONE', 'DUIXC_RTS_NULL_NODE'
  ")
END

/*
   This sql file:

       - creates the tables necessary to hold staged RODM messages
       - creates the stored procedures that are used to stage messages - the
         procedures will be called FROM the event listener
       - adds to the StagedEventProc, EventProcMap, AND DATA_TYPE tables
         to tell the event listener how to call the procedures
*/

/*
  DROP TABLE StagedRODMConnectionStatus
  DROP TABLE StagedRODMResponseMessage
  DROP TABLE StagedRODMStatesMessage
*/


---------------------------------------------------------------------------------------------------
-- format type 05, action type 01

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'StagedRODMConnectionStatus' AND type = 'U')
BEGIN
  PRINT 'Creating table StagedRODMConnectionStatus'

  CREATE TABLE StagedRODMConnectionStatus -- object pump to application server
  ( 
    -- Standard staging TABLE columns
    timestamp                             CONSTRAINT StagedRODMConnectionStatus_PK PRIMARY KEY (timestamp),
    ctime           DATETIME              CONSTRAINT StagedRODMConnectionStatus_ctime_DF DEFAULT GETDATE(),
    src_cid         ClassID,
    src_id          ObjID,
    -- The remaining columns are FROM the documented protocol
    DATE_TIME       DATETIME NULL,    -- event timestamp
    SEQ_NO          VARCHAR(50) NULL, -- SMFID concatenated with the trap match number associated with trap setting
    OBJ_TYPE_ID     VARCHAR(12) NULL, -- 'RODMSTATUS', identifies this record AS a connection status message
    PARM_1          VARCHAR(8) NULL   -- 'UP' or 'DOWN', with some provision FOR expansion
  )
END
GO


---------------------------------------------------------------------------------------------------
-- Process_RODM_ConnectionStatus
PRINT 'Creating procedure Process_RODM_ConnectionStatus'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'Process_RODM_ConnectionStatus' AND type = 'P') 
  DROP PROCEDURE Process_RODM_ConnectionStatus
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
/*
 EXEC Process_RODM_ConnectionStatus @SourceCID='ROOT', @SourceID=0, @FormatType=5, @ActionType=1,
      @DATE_TIME='2000010100:00:00.000000', @SEQ_NO='0', @OBJ_TYPE_ID='RODMSTATUS', @PARM_1='UP'
*/
CREATE PROCEDURE Process_RODM_ConnectionStatus
  @SourceCID          ClassID,
  @SourceID           ObjID,
  @FormatType         SMALLINT,
  @ActionType         SMALLINT,
  @DATE_TIME          VARCHAR(30) = NULL, -- event timestamp in YYYYMMDDHH:MM:SS.UUUUUU format
  @SEQ_NO             VARCHAR(50) = NULL, -- SMFID concatenated with the trap match number associated with trap setting
  @OBJ_TYPE_ID        VARCHAR(12) = NULL, -- 'RODMSTATUS', identifies this record AS a connection status message
  @PARM_1             VARCHAR(8) = NULL   -- 'UP' or 'DOWN', with some provision FOR expansion
AS
  DECLARE @DATETIME DATETIME

  EXEC asisp_convertMVSDateTime @DATE_TIME, @DATETIME OUTPUT

  INSERT StagedRODMConnectionStatus(src_cid, src_id, DATE_TIME, SEQ_NO, OBJ_TYPE_ID, PARM_1)
    SELECT @SourceCID, @SourceID, @DATETIME, @SEQ_NO, @OBJ_TYPE_ID, @PARM_1
GO


---------------------------------------------------------------------------------------------------
-- format type 05, action type 03

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'StagedRODMResponseMessage' AND type = 'U')
BEGIN
  PRINT 'Creating table StagedRODMResponseMessage'

  CREATE TABLE StagedRODMResponseMessage -- object pump to application server
  (
    -- Standard staging TABLE columns
    timestamp                             CONSTRAINT StagedRODMResponseMessage_PK PRIMARY KEY (timestamp),
    ctime           DATETIME              CONSTRAINT StagedRODMResponseMessage_ctime_DF DEFAULT GETDATE(),
    src_cid         ClassID,
    src_id          ObjID,
    -- The remaining columns are FROM the documented protocol
    DATE_TIME       DATETIME NULL,     -- event timestamp
    SEQ_NO          VARCHAR(50) NULL,  -- the SMFID concatenated with the trap match number associated with trap setting
    OBJ_TYPE_ID     VARCHAR(4) NULL,   -- 'RESP', identifies record AS a RESPONSE message
    CORRELATION_ID  VARCHAR(255) NULL, -- correlation id sent ON requesting registration record
    PARM_1          INT NULL,          -- return code FROM the request
    PARM_2          INT NULL,          -- reason code FROM the request
    PARM_3          VARCHAR(255) NULL  -- error or informational messages
  )
END
GO


---------------------------------------------------------------------------------------------------
/*
   Create and populate the RODMAutoRegistrationMap table that is used when RODM Objects are 
   discovered to automatically register/unregister the objects.

  Columns:
    id - Unique row identifier
    description - free text to describe this mapping
    whereClause - valid where-clause (not including WHERE keyword) that can reference
                  any of the RODMManagedObject columns.
    priority    - integer value determining evaluation order. Rows with higher priority
                  are evaluated before those with lower priority.
    enabled     - boolean indicating whether the current row will be evaluated (0=disabled, 1=enabled)
*/

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMAutoRegistrationMap' AND type = 'U')
BEGIN
  PRINT 'Creating table RODMAutoRegistrationMap'
  EXEC("
  CREATE TABLE RODMAutoRegistrationMap
  (
    id                    INT IDENTITY(1,1),
    description           VARCHAR(255) NULL,
    whereClause           VARCHAR(500),
    priority              INT       CONSTRAINT RODMARM_priority_DF DEFAULT 0,
    enabled               TINYINT   CONSTRAINT RODMARM_enabled_DF DEFAULT 0
  )
  ALTER TABLE RODMAutoRegistrationMap ADD CONSTRAINT RODMARM_PK PRIMARY KEY (id)
  ")
  EXEC("
  INSERT INTO RODMAutoRegistrationMap(description, whereClause)
    VALUES
    (
      'Auto-register top level classes contained under the network', 
      'cid IN (SELECT cid FROM derived_obj_link WHERE link_type = ''PHYC'' AND src_cid = ''SNNT'')'
    )
  ")
END
GO


---------------------------------------------------------------------------------------------------
-- Process_RODM_ResponseMessage
PRINT 'Creating procedure Process_RODM_ResponseMessage'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'Process_RODM_ResponseMessage' AND type = 'P')
  DROP PROCEDURE Process_RODM_ResponseMessage
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
/*
 EXEC Process_RODM_ResponseMessage @SourceCID='ROOT', @SourceID=0, @FormatType=5, @ActionType=1,
      @DATE_TIME='2000010100:00:00.000000', @SEQ_NO='0', @OBJ_TYPE_ID='RESP', @CORRELATION_ID='correlate me',
      @PARM_1='0', @PARM_2='0'
*/
CREATE PROCEDURE Process_RODM_ResponseMessage
    @SourceCID         ClassID,
    @SourceID          ObjID,
    @FormatType        SMALLINT,
    @ActionType        SMALLINT,
    @DATE_TIME         VARCHAR(30) = NULL,  -- event timestamp in YYYYMMDDHH:MM:SS.UUUUUU format
    @SEQ_NO            VARCHAR(50) = NULL,  -- the SMFID concatenated with the trap match number associated with trap setting
    @OBJ_TYPE_ID       VARCHAR(4) = NULL,   -- 'RESP', identifies record AS a RESPONSE message
    @CORRELATION_ID    VARCHAR(255) = NULL, -- correlation id sent ON requesting registration record
    @PARM_1            VARCHAR(16) = NULL,  -- return code FROM the request
    @PARM_2            VARCHAR(16) = NULL,  -- reason code FROM the request
    @PARM_3            VARCHAR(255) = NULL  -- error or informational messages
AS
  DECLARE @DATETIME DATETIME

  EXEC asisp_convertMVSDateTime @DATE_TIME, @DATETIME OUTPUT

  DECLARE @PARM_1_INT INT
  DECLARE @PARM_2_INT INT

  SET @PARM_1_INT = CAST(@PARM_1 AS INT)
  SET @PARM_2_INT = CAST(@PARM_2 AS INT)

  INSERT StagedRODMResponseMessage(src_cid, src_id, DATE_TIME, SEQ_NO, OBJ_TYPE_ID, CORRELATION_ID, PARM_1, PARM_2, PARM_3)
    SELECT @SourceCID, @SourceID, @DATETIME, @SEQ_NO, @OBJ_TYPE_ID, @CORRELATION_ID, @PARM_1_INT, @PARM_2_INT, @PARM_3
GO


---------------------------------------------------------------------------------------------------
-- format type 05, action type 04

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'StagedRODMStatesMessage' AND type = 'U')
BEGIN
  PRINT 'Creating table StagedRODMStatesMessage'

  CREATE TABLE StagedRODMStatesMessage -- object pump to application server
  ( 
    -- Standard staging TABLE columns
    timestamp                             CONSTRAINT StagedRODMStatesMessage_PK PRIMARY KEY (timestamp),
    ctime           DATETIME              CONSTRAINT StagedRODMStatesMessage_ctime_DF DEFAULT GETDATE(),
    src_cid         ClassID,
    src_id          ObjID,
    -- The remaining columns are FROM the documented protocol
    DATE_TIME       DATETIME NULL,      -- event timestamp
    SEQ_NO          VARCHAR(50) NULL,   -- the SMFID concatenated with the trap match number associated with trap setting
    OBJ_TYPE_ID     VARCHAR(4) NULL,    -- 'RODM', identifies record AS a states message
    Class           VARCHAR(60) NULL,   -- RODM class name of the object
    MyName          VARCHAR(255) NULL,  -- RODM object name in RDN format
    RODMField       VARCHAR(255) NULL,  -- field name being monitored
    CharData        VARCHAR(256) NULL,  -- data FROM monitored field, in character format
    BinData         VARBINARY(128) NULL -- data FROM monitored field, in BINARY FOR easier manipulation later
  )
END
GO


---------------------------------------------------------------------------------------------------
-- Process_RODM_StatesMessage
PRINT 'Creating procedure Process_RODM_StatesMessage'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'Process_RODM_StatesMessage' AND type = 'P')
  DROP PROCEDURE Process_RODM_StatesMessage
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
/*
 EXEC Process_RODM_StatesMessage @SourceCID='ROOT', @SourceID=0, @FormatType=5, @ActionType=1,
      @DATE_TIME='2000010100:00:00.000000', @SEQ_NO='0', @OBJ_TYPE_ID='RESP',
      @PARM_1='1.1.1.1.1', @PARM_2='0.0.0.0.0', @PARM_3='states', @PARM_4='0001020304', 
*/
CREATE PROCEDURE Process_RODM_StatesMessage
  @SourceCID         ClassID,
  @SourceID          ObjID,
  @FormatType        SMALLINT,
  @ActionType        SMALLINT,
  @DATE_TIME         VARCHAR(30) = NULL,  -- event timestamp in YYYYMMDDHH:MM:SS.UUUUUU format
  @SEQ_NO            VARCHAR(50) = NULL,  -- the SMFID concatenated with the trap match number associated with trap setting
  @OBJ_TYPE_ID       VARCHAR(4) = NULL,   -- 'RODM', identifies record AS a RESPONSE message
  @PARM_1            VARCHAR(60) = NULL,  -- RODM class name of the object
  @PARM_2            VARCHAR(255) = NULL, -- RODM object name in RDN format
  @PARM_3            VARCHAR(255) = NULL, -- field name being monitored
  @PARM_4            VARCHAR(256) = NULL  -- data FROM monitored field in hex, two bytes per single byte in field
AS
  DECLARE @DATETIME DATETIME
  DECLARE @bindata varbinary(128)
  DECLARE @bytepos INT
  DECLARE @byte BINARY
  DECLARE @pos INT
  DECLARE @upperHexes char(16)
  DECLARE @lowerHexes char(16)
  DECLARE @datatype VARCHAR(16)
  DECLARE @class varchar(60)

  SET @upperHexes = '0123456789ABCDEF'
  SET @lowerHexes = '0123456789abcdef'

  EXEC asisp_convertMVSDateTime @DATE_TIME, @DATETIME OUTPUT

  SET @bytepos = 1
  SET @bindata = null

  -- changed from dotted class name to name we'll find in RODMManagedObject,
  -- for example: 1.3.18.0.0.1845 => t5Node
  SET @class = @PARM_1 -- use the input value as a default
  SELECT @class = Class FROM RODMClassMap WHERE RODMClass = @PARM_1

  SELECT @datatype = DataType FROM RODMLoad..RODMFieldMap WHERE RODMField = @PARM_3

  IF @datatype = 'ANONYMOUSVAR'
  BEGIN

    IF (@PARM_3 = 'states') -- make sure there's at least 7 bytes of status data
      WHILE len(@PARM_4) < 14
        SET @PARM_4 = @PARM_4 + '00'

    -- anonymousvar data is sent AS a character string of bytes in hex - let's convert the data to
    -- a BINARY string FOR easier manipulation when the data is applied to the objects

    WHILE @bytepos < len(@PARM_4)
    BEGIN
      SET @byte = 0
      SET @pos = patindex('%' + SUBSTRING(@PARM_4,@bytepos,1) + '%',@upperHexes)
      IF @pos = 0 SET @pos = patindex('%' + SUBSTRING(@PARM_4,@bytepos,1) + '%',@lowerHexes)
      IF @pos > 0 SET @byte = (@pos-1) * 16

      SET @pos = patindex('%' + SUBSTRING(@PARM_4,@bytepos+1,1) + '%',@upperHexes)
      IF @pos = 0 SET @pos = patindex('%' + SUBSTRING(@PARM_4,@bytepos+1,1) + '%',@lowerHexes)
      IF @pos > 0 SET @byte = @byte + @pos - 1

      IF @bindata is null
        SET @bindata = @byte
      ELSE
        SET @bindata = @bindata + @byte

      SET @bytepos = @bytepos + 2
    END

    INSERT StagedRODMStatesMessage(src_cid, src_id, DATE_TIME, SEQ_NO, OBJ_TYPE_ID, Class, MyName, RODMField, BinData)
      SELECT @SourceCID, @SourceID, @DATETIME, @SEQ_NO, @OBJ_TYPE_ID, @class, @PARM_2, @PARM_3, @bindata
  END
  ELSE
  BEGIN
    INSERT StagedRODMStatesMessage(src_cid, src_id, DATE_TIME, SEQ_NO, OBJ_TYPE_ID, Class, MyName, RODMField, CharData)
      SELECT @SourceCID, @SourceID, @DATETIME, @SEQ_NO, @OBJ_TYPE_ID, @class, @PARM_2, @PARM_3, @PARM_4
  END
GO

IF NOT EXISTS (SELECT * FROM PendingEventTableDef WHERE abbrev = 'RODM.Connect')
  INSERT INTO PendingEventTableDef(abbrev, name, numrows_yellow, numrows_red, oldest_yellow, oldest_red)
    VALUES ( 'RODM.Connect',  'StagedRODMConnectionStatus', 1000, 2000, 1, 2 )
IF NOT EXISTS (SELECT * FROM PendingEventTableDef WHERE abbrev = 'RODM.Response')
  INSERT INTO PendingEventTableDef(abbrev, name, numrows_yellow, numrows_red, oldest_yellow, oldest_red)
    VALUES ( 'RODM.Response', 'StagedRODMResponseMessage',  1000, 2000, 1, 2 )
IF NOT EXISTS (SELECT * FROM PendingEventTableDef WHERE abbrev = 'RODM.States')
  INSERT INTO PendingEventTableDef(abbrev, name, numrows_yellow, numrows_red, oldest_yellow, oldest_red)
    VALUES ( 'RODM.States',   'StagedRODMStatesMessage',    1000, 2000, 1, 2 )

EXEC asisp_defineStagedEventProc @ProcName='asisp_processStagedRODMConnections', @OrderNo=10, @Enabled=1, @MaxRows=20
EXEC asisp_defineStagedEventProc @ProcName='asisp_processStagedRODMResponses',   @OrderNo=10, @Enabled=1, @MaxRows=20
EXEC asisp_defineStagedEventProc @ProcName='asisp_processStagedRODMStateMsgs',   @OrderNo=10, @Enabled=1, @MaxRows=20

EXEC asisp_defineEventProcMapping @FormatType='RODM', @ActionType='CONNECTION_STATUS',      @ProcName='Process_RODM_ConnectionStatus', @PassEventText=0
EXEC asisp_defineEventProcMapping @FormatType='RODM', @ActionType='REGISTRATION_RESPONSE',  @ProcName='Process_RODM_ResponseMessage',  @PassEventText=0
EXEC asisp_defineEventProcMapping @FormatType='RODM', @ActionType='OBJECT_STATE',           @ProcName='Process_RODM_StatesMessage',    @PassEventText=0

EXEC asisp_defineDataType 5, 0,  'DATE_TIME',      'Timestamp'
EXEC asisp_defineDataType 5, 1,  'SEQ_NO',         'SMF-ID'
EXEC asisp_defineDataType 5, 2,  'OBJ_TYPE_ID',    'Record identifier'
EXEC asisp_defineDataType 5, 73, 'PARM_1',         'Connection status, return code, or class name of RODM object'
EXEC asisp_defineDataType 5, 74, 'PARM_2',         'Reason code or RODM object name in RDN format'
EXEC asisp_defineDataType 5, 75, 'PARM_3',         'Error or informational message or field name being monitored'
EXEC asisp_defineDataType 5, 76, 'PARM_4',         'Data FROM monitored field'
EXEC asisp_defineDataType 5, 84, 'CORRELATION_ID', 'Correlation ID string'

GO


---------------------------------------------------------------------------------------------------
PRINT 'Creating procedure _RODMRegister'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_RODMRegister' AND type = 'P')
  DROP PROCEDURE _RODMRegister
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
-- Implementation of the RODMRegister method.
-- This stored procedure is called to register an object or group of objects with RODM.  It's 
-- designed to be invoked FROM the GUI client.
CREATE PROCEDURE _RODMRegister
  @cid	              ClassID,
  @id                 ObjID,
  @RegistrationStatus TINYINT,  /* 0=unregister, 1=register, 2=re-register */
  @RegistrationScope  TINYINT,  /* 1=root obj only, 2=only descendents, 3=root AND descendents */
  @ResponseType       TINYINT   /* 0=OFF (never), 1=EX (exception only), 2=DEF (always) */
AS 
  RAISERROR('_RODMRegister: %s, %d, %d, %d', 1, 1, @cid, @id, @RegistrationStatus, @RegistrationScope)
  DECLARE @cmd VARCHAR(255)

  DECLARE @sql VARCHAR(255)
  SELECT @sql = 'DECLARE namec CURSOR FOR SELECT name FROM ' + cname + '_V WHERE id = ' + CAST(@id AS VARCHAR)
    FROM obj_class WHERE cid = @cid
  EXEC(@sql)
  OPEN namec
  DECLARE @name ObjName
  FETCH NEXT FROM namec INTO @name
  DEALLOCATE namec
  RAISERROR('_RODMRegister: name=''%s''', 0, 1, @name)

  SELECT @cmd = 'sh SendMVSRODMRegisterObjects.ksh'
    + ' -c' + @cid 
    + ' -i' + CAST(@id AS VARCHAR)
    + ' -n"' + @name + '"' 
    + ' -v ResponseType=' + CAST(@ResponseType AS VARCHAR)
    + ' -v RegistrationStatus=' + CAST(@RegistrationStatus AS VARCHAR)
    + ' -v RegistrationScope=' + CAST(@RegistrationScope AS VARCHAR)
  
  DECLARE @r INT
  RAISERROR('_RODMRegister: Executing the following: %s', 0, 1, @cmd)
-- Use the commented code below to capture the output from the command into the #output temp table,
-- which can then be examined for diagnostics.
--  CREATE TABLE #output (line VARCHAR(255) NULL)
--  INSERT #output
    EXEC @r = master..xp_cmdshell @cmd
  RAISERROR('_RODMRegister: Executed SendMVSRODMRegisterObjects.ksh. Exit code=%d', 0, 1, @r)
--  IF EXISTS (SELECT * FROM #output)
--    SELECT * FROM #output
  RETURN @r
GO


---------------------------------------------------------------------------------------------------
PRINT 'Creating procedure asisp_processStagedRODMConnections'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_processStagedRODMConnections' AND type = 'P')
  DROP PROCEDURE asisp_processStagedRODMConnections
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
/*
  Stored procedure to process RODM connection messages that have been staged.

  We always generate the appropriate RODMSTATUS exception on the operating systems (UP or DOWN).

  If the RODM connection just came up we re-register all the objects that should be registered
  to that system, and generate an audit message on the OS.
*/
CREATE PROCEDURE asisp_processStagedRODMConnections
  @rowlimit INT = 100,
  @verbose  TINYINT = 0
AS
  SET rowcount @rowlimit

  DECLARE @timestamp BINARY(8)
  DECLARE @ctime DATETIME

  DECLARE @src_cid ClassID -- should always be 'OS'
  DECLARE @src_id ObjID    -- object id of the operating system from which the connection message came from
  DECLARE @DATE_TIME DATETIME
  DECLARE @SEQ_NO VARCHAR(50)        -- SMFID
  DECLARE @OBJ_TYPE_ID VARCHAR(12)   -- 'RODMSTATUS'
  DECLARE @PARM_1 VARCHAR(8)          -- 'UP' or 'DOWN'

  DECLARE @time DATETIME

  DECLARE connectionCursor CURSOR FOR 
    SELECT src_cid, src_id, timestamp, ctime, DATE_TIME, SEQ_NO, OBJ_TYPE_ID, PARM_1 FROM StagedRODMConnectionStatus
  OPEN connectionCursor
  FETCH NEXT FROM connectionCursor INTO @src_cid, @src_id, @timestamp, @ctime, @DATE_TIME, @SEQ_NO, @OBJ_TYPE_ID, @PARM_1
  WHILE @@FETCH_STATUS = 0
  BEGIN

    IF @verbose <> 0 PRINT 'SEQ_NO=' + @SEQ_NO + ', status=' + @PARM_1 + ', time=' + CAST(@DATE_TIME AS VARCHAR)


    IF @PARM_1 = 'UP' BEGIN
      -- clear out an earlier exception that might indicate that the connection was down
      SELECT @time = CURRENT_TIMESTAMP
      exec asisp_createEXCP @name='RODMSTATUS DOWN', @parent_cid=@src_cid, @parent_id=@src_id,
                            @DATE=@time, @TIME=@time,
                            @SEQ_NO=@SEQ_NO,
                            @EXCP_NAME='RODMSTATUS DOWN', @EXCP_CD='OK'
      -- generate an exception to show the connection is up
      SELECT @time = CURRENT_TIMESTAMP
      exec asisp_createEXCP @name='RODMSTATUS UP', @parent_cid=@src_cid, @parent_id=@src_id,
                            @DATE=@time, @TIME=@time,
                            @SEQ_NO=@SEQ_NO,
                            @EXCP_NAME='RODMSTATUS UP', @EXCP_CD='TBSM Object Pump has restored connection to RODM'
    END
    ELSE BEGIN
      -- clear out an earlier exception that might indicate that the connection was up
      SELECT @time = CURRENT_TIMESTAMP
      exec asisp_createEXCP @name='RODMSTATUS UP', @parent_cid=@src_cid, @parent_id=@src_id,
                            @DATE=@time, @TIME=@time,
                            @SEQ_NO=@SEQ_NO,
                            @EXCP_NAME='RODMSTATUS UP', @EXCP_CD='OK'
      -- generate an exception to show the connection is down
      SELECT @time = CURRENT_TIMESTAMP
      exec asisp_createEXCP @name='RODMSTATUS DOWN', @parent_cid=@src_cid, @parent_id=@src_id,
                            @DATE=@time, @TIME=@time,
                            @SEQ_NO=@SEQ_NO,
                            @EXCP_NAME='RODMSTATUS DOWN', @EXCP_CD='TBSM Object Pump has lost connection to RODM'
    END

    IF @PARM_1 = 'UP'
    BEGIN
      SELECT @time = CURRENT_TIMESTAMP
      -- generate an audit record to indicate why objects are being re-registered
      exec asisp_createMESG @name='ASI_05\01', @parent_cid=@src_cid, @parent_id=@src_id,
                            @DATE=@time, @TIME=@time,
                            @SEQ_NO=@SEQ_NO,
                            @MESSAGE='ASI_05\01', 
                            @REASON_CD='RODM objects monitored by this operating system are being re-registered because the TBSM Object Pump has restored connection to RODM'
      -- re-register RODM objects monitored by this operating system
      exec _RODMRegister @cid=@src_cid, @id=@src_id, 
                         @RegistrationStatus=2, @RegistrationScope=2,
                         @ResponseType=1
    END

    DELETE FROM StagedRODMConnectionStatus WHERE timestamp = @timestamp

    FETCH NEXT FROM connectionCursor INTO @src_cid, @src_id, @timestamp, @ctime, @DATE_TIME, @SEQ_NO, @OBJ_TYPE_ID, @PARM_1
  END
  DEALLOCATE connectionCursor

  RETURN 0
GO

---------------------------------------------------------------------------------------------------
-- asisp_processStagedRODMResponses
PRINT 'Creating procedure asisp_processStagedRODMResponses'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_processStagedRODMResponses' AND type = 'P')
  DROP PROCEDURE asisp_processStagedRODMResponses
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
/*
  This is a stored procedure to process RODM responses to requests to register objects.

  An exception will be generated for the response and all exceptions in all other exception
  types will be removed.  For instance, if a successful response comes from a register action,
  we'll remove exceptions for unsuccessful registers, unsuccessful unregisters, and 
  successful unregisters.

  The response record indicates whether the registration action was successful or not in the
  @PARM_1 field.  We know whether the action was a register or unregister by looking at the
  correlation field.  When asisp_uploadGetRODMObjects builds register/unregister requests,
  it puts the action in the correlation field: "id=<objectID> action={register|unregister}".
*/
CREATE PROCEDURE asisp_processStagedRODMResponses
  @rowlimit INT = 100,
  @verbose  TINYINT = 0
AS
  SET rowcount @rowlimit

  DECLARE @timestamp BINARY(8)
  DECLARE @ctime DATETIME

  DECLARE @cid ClassID
  DECLARE @id ObjID

  DECLARE @DATE_TIME DATETIME
  DECLARE @SEQ_NO VARCHAR(50)        -- SMFID
  DECLARE @OBJ_TYPE_ID VARCHAR(12)   -- 'RESP'
  DECLARE @CORRELATION_ID VARCHAR(255)
  DECLARE @PARM_1 INT                -- return code FROM the request
  DECLARE @PARM_2 INT                -- reason code FROM the request
  DECLARE @PARM_3 VARCHAR(255)       -- error/informational message

  DECLARE @action tinyint
  DECLARE @time DATETIME
  DECLARE @desc varchar(255)

  DECLARE responseCursor CURSOR FOR 
    SELECT timestamp, ctime, DATE_TIME, SEQ_NO, OBJ_TYPE_ID, CORRELATION_ID, PARM_1, PARM_2, PARM_3 FROM StagedRODMResponseMessage
  OPEN responseCursor
  FETCH NEXT FROM responseCursor INTO @timestamp, @ctime, @DATE_TIME, @SEQ_NO, @OBJ_TYPE_ID, @CORRELATION_ID, @PARM_1, @PARM_2, @PARM_3
  WHILE @@FETCH_STATUS = 0 BEGIN

    IF @verbose <> 0 PRINT 'SEQ_NO=' + @SEQ_NO + ', correlation id=' + @CORRELATION_ID + 
                           ', return code=' + CAST(@PARM_1 AS VARCHAR) +
                           ', reason code=' + CAST(@PARM_2 AS VARCHAR) +
                           ', error/informational message=' + @PARM_3 +
                           ', time=' + CAST(@DATE_TIME AS VARCHAR)

    set @id = null
    if substring(@CORRELATION_ID, 1, 3) = 'id='
      select @id = cast(substring(@CORRELATION_ID, 4, patindex('% %', @CORRELATION_ID)-4) as int)

    if @id is not null
      select @cid = cid from RODMManagedObject where id = @id

    if @verbose <> 0 print 'cid=' + @cid + ', id=' + cast(@id as varchar)

    if @cid is not null and @id is not null begin

      if @CORRELATION_ID like '%action=unregister%'
        set @action=0
      else
        set @action=1

      -- remove any existing exceptions
      SELECT @time = CURRENT_TIMESTAMP
      if @action <> 1 or @PARM_1 <> 0 begin -- remove successful registration
        exec asisp_createEXCP @name='RODMREGISTER SUCCEEDED', @parent_cid=@cid, @parent_id=@id,
                              @DATE=@time, @TIME=@time,
                              @SEQ_NO=@SEQ_NO,
                              @EXCP_NAME='RODMREGISTER SUCCEEDED', @EXCP_CD='OK'
      end
      if @action <> 1 or @PARM_1 = 0 begin -- remove unsuccessful registration
        exec asisp_createEXCP @name='RODMREGISTER FAILED', @parent_cid=@cid, @parent_id=@id,
                              @DATE=@time, @TIME=@time,
                              @SEQ_NO=@SEQ_NO,
                              @EXCP_NAME='RODMREGISTER FAILED', @EXCP_CD='OK'
      end
      if @action = 1 or @PARM_1 <> 0 begin -- remove successful unregistration
        exec asisp_createEXCP @name='RODMUNREGISTER SUCCEEDED', @parent_cid=@cid, @parent_id=@id,
                              @DATE=@time, @TIME=@time,
                              @SEQ_NO=@SEQ_NO,
                              @EXCP_NAME='RODMUNREGISTER SUCCEEDED', @EXCP_CD='OK'
      end
      if @action = 1 or @PARM_1 = 0 begin -- remove unsuccessful unregistration
        exec asisp_createEXCP @name='RODMUNREGISTER FAILED', @parent_cid=@cid, @parent_id=@id,
                              @DATE=@time, @TIME=@time,
                              @SEQ_NO=@SEQ_NO,
                              @EXCP_NAME='RODMUNREGISTER FAILED', @EXCP_CD='OK'
      end

      SELECT @time = CURRENT_TIMESTAMP

      exec master..xp_asisprintf @desc output, ' correlation_id=%s, return code=%d, reason code=%d, message=%s',
                                 @CORRELATION_ID, @PARM_1, @PARM_2, @PARM_3

      if @action = 1 and @PARM_1 = 0 begin -- successful registration
        set @desc = 'RODM Register Succeeded:' + @desc
        exec asisp_createEXCP @name='RODMREGISTER SUCCEEDED', @parent_cid=@cid, @parent_id=@id,
                              @DATE=@time, @TIME=@time,
                              @SEQ_NO=@SEQ_NO,
                              @EXCP_NAME='RODMREGISTER SUCCEEDED', @EXCP_CD=@desc
      end
      else if @action = 1 and @PARM_1 <> 0 begin -- unsuccessful registration
        set @desc = 'RODM Register Failed:' + @desc
        exec asisp_createEXCP @name='RODMREGISTER FAILED', @parent_cid=@cid, @parent_id=@id,
                              @DATE=@time, @TIME=@time,
                              @SEQ_NO=@SEQ_NO,
                              @EXCP_NAME='RODMREGISTER FAILED', @EXCP_CD=@desc
      end
      if @action <> 1 and @PARM_1 = 0 begin -- successful unregistration
        set @desc = 'RODM Unregister Succeeded:' + @desc
        exec asisp_createEXCP @name='RODMUNREGISTER SUCCEEDED', @parent_cid=@cid, @parent_id=@id,
                              @DATE=@time, @TIME=@time,
                              @SEQ_NO=@SEQ_NO,
                              @EXCP_NAME='RODMUNREGISTER SUCCEEDED', @EXCP_CD=@desc
      end
      else if @action <> 1 and @PARM_1 <> 0 begin -- unsuccessful unregistration
        set @desc = 'RODM Unregister Failed:' + @desc
        exec asisp_createEXCP @name='RODMUNREGISTER FAILED', @parent_cid=@cid, @parent_id=@id,
                              @DATE=@time, @TIME=@time,
                              @SEQ_NO=@SEQ_NO,
                              @EXCP_NAME='RODMUNREGISTER FAILED', @EXCP_CD=@desc
      end
    end

    DELETE FROM StagedRODMResponseMessage WHERE timestamp = @timestamp

    FETCH NEXT FROM responseCursor INTO @timestamp, @ctime, @DATE_TIME, @SEQ_NO, @OBJ_TYPE_ID, @CORRELATION_ID, 
                                        @PARM_1, @PARM_2, @PARM_3
  END
  DEALLOCATE responseCursor

  RETURN 0
GO


---------------------------------------------------------------------------------------------------
-- asisp_processStagedRODMStateMsgs
PRINT 'Creating procedure asisp_processStagedRODMStateMsgs'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_processStagedRODMStateMsgs' AND type = 'P')
  DROP PROCEDURE asisp_processStagedRODMStateMsgs
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
/*
   This stored procedure takes monitored field messages staged FROM RODM AND applies the new
   value of the field to the appropriate object.  A single UPDATE is executed to SET all attributes
   AND a TABLE trigger will later CREATE the necessary exceptions AND/or messages.
*/
CREATE PROCEDURE asisp_processStagedRODMStateMsgs
  @rowlimit INT = 100,
  @verbose  TINYINT = 0
AS
  SET rowcount @rowlimit

  -- put staged rows in a temporary TABLE, process them, DELETE FROM the original TABLE

  DECLARE @timestamp BINARY(8)
  DECLARE @ctime DATETIME
  DECLARE @data  VARCHAR(256)
  DECLARE @RODMName VARCHAR(255)
  DECLARE @fieldName VARCHAR(255)

  DECLARE @UPDATE TINYINT
  DECLARE @byte INT

  CREATE TABLE #stateMsgs
  (
    -- Standard staging TABLE columns
    timestamp       BINARY(8),
    ctime           DATETIME,
    src_cid         ClassID,
    src_id          ObjID,
    -- The remaining columns are FROM the documented protocol
    DATE_TIME       DATETIME NULL,      -- event timestamp
    SEQ_NO          VARCHAR(50) NULL,   -- the SMFID concatenated with the trap match number associated with trap setting
    OBJ_TYPE_ID     VARCHAR(4) NULL,    -- 'RODM', identifies record AS a states message
    Class           VARCHAR(60) NULL,   -- RODM class name of the object
    MyName          VARCHAR(255) NULL,  -- RODM object name in RDN format
    RODMField       VARCHAR(255) NULL,  -- field name being monitored
    CharData        VARCHAR(256) NULL,  -- data FROM monitored field, in character format
    BinData         VARBINARY(128) NULL -- data FROM monitored field, in BINARY FOR easier manipulation later
  )

  INSERT INTO #stateMsgs SELECT * FROM StagedRODMStatesMessage

  UPDATE o SET 
    _operationalState = CAST(SUBSTRING(cast(BinData as binary(7)),1,1) AS INT), 
    _usageState = CAST(SUBSTRING(cast(BinData as binary(7)),2,1) AS INT),
    _administrativeState = CAST(SUBSTRING(cast(BinData as binary(7)),3,1) AS INT),
    _availabilityStatus = CAST(SUBSTRING(cast(BinData as binary(7)),4,1) AS INT),
    _proceduralStatus = CAST(SUBSTRING(cast(BinData as binary(7)),5,1) AS INT),
    _unknownStatus = CAST(SUBSTRING(cast(BinData as binary(7)),6,1) AS INT),
    _nativeStatus = CAST(SUBSTRING(cast(BinData as binary(7)),7,1) AS INT)
  FROM 
    RODMManagedObject o JOIN #stateMsgs s ON o._RODMMyName = s.MyName AND o._RODMClass = s.Class
    WHERE RODMField = 'states'

  UPDATE o SET 
    _DisplayStatus = CAST(CharData AS INT)
  FROM 
    RODMManagedObject o JOIN #stateMsgs s ON o._RODMMyName = s.MyName AND o._RODMClass = s.Class
    WHERE RODMField = 'DisplayStatus'

  DELETE s FROM StagedRODMStatesMessage AS s JOIN #stateMsgs AS t ON s.timestamp = t.timestamp

  RETURN 0
GO


---------------------------------------------------------------------------------------------------
-- asisp_uploadGetRODMObjects
PRINT 'Creating procedure asisp_uploadGetRODMObjects'

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_uploadGetRODMObjects' AND type = 'P')
  DROP PROCEDURE asisp_uploadGetRODMObjects
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
/*
   This stored procedure is called FROM the SendMVSRODMRegisterObjects shell script.  The script
   will find generate the list of all objects that need to be registered, unregistered, or
   re-registered, AND UPDATE the RODMManagedObject TABLE with the appropriate registration status.
*/
CREATE PROCEDURE asisp_uploadGetRODMObjects
  @cid                ClassID,
  @id                 ObjID,
  @RegistrationStatus TINYINT,  /* 0=unregister, 1=register, 2=re-register */
  @RegistrationScope  TINYINT,  /* 1=root obj only, 2=only descendents, 3=root AND descendents */
  @ResponseType       TINYINT   /* 0=OFF (never), 1=EX (exception only), 2=DEF (always) */
AS
  SET NOCOUNT ON

  CREATE TABLE #descendent -- table to hold the objects we need to register
  (
    cid   ClassID,
    id    ObjID,
    gen   INT
  )

  IF @RegistrationScope <> 1 -- include descendent objects
    IF @cid = 'OS'
    BEGIN
      -- If the calling object is an operating system then we want to perform the registration
      -- operation on all SNANetworks that are monitored by this OS. We iterate through all
      -- these networks and build up the table of descendent objects so we can do one monster
      -- registration.
      DECLARE osc INSENSITIVE CURSOR FOR SELECT id FROM SNANetwork_V WHERE _MonitoringOSID = @id
      OPEN osc
      DECLARE @snnt_id ObjID
      FETCH NEXT FROM osc INTO @snnt_id
      WHILE @@FETCH_STATUS <> -1
      BEGIN
        EXEC asisp_descendents 'SNNT', @snnt_id, 'IN (SELECT cid FROM isa_chain WHERE base_cid = ''RDMO'')' -- find RODMStatesObjects
        FETCH NEXT FROM osc INTO @snnt_id
      END
      DEALLOCATE osc
    END
    ELSE
      -- The calling object is expected to have a direct parent-child chain to RODMManagedObjects.
      EXEC asisp_descendents @cid, @id, 'IN (SELECT cid FROM isa_chain WHERE base_cid = ''RDMO'')' -- find RODMManagedObjects

  -- include the base object (unless it's an OS)
  IF @RegistrationScope <> 2 AND @cid <> 'OS' 
    INSERT #descendent SELECT @cid, @id, 0

  -- Filter out objects by registration status. If we're asked to reregister objects then send
  -- registration statements only for objects that have _RegistrationStatus=1. This will avoid
  -- floods of unregister statements.
  IF @RegistrationStatus = 2
    DELETE d FROM #descendent AS d
    JOIN RODMManagedObject AS rmo ON d.cid = rmo.cid AND d.id = rmo.id
    WHERE rmo._RegistrationStatus = 0

  -- Make any necessary change to RegistrationStatus
  IF @RegistrationStatus <> 2
    UPDATE rmo SET _RegistrationStatus = @RegistrationStatus 
    FROM RODMManagedObject AS rmo
    JOIN #descendent d ON rmo.cid = d.cid AND rmo.id = d.id

  -- If this is a re-register we don't want to send up unregister statements
  CREATE TABLE #monitored_fields
  (
    cid         ClassID,
    field_name  VARCHAR(40)
  )
  INSERT #monitored_fields SELECT 'RDSO', 'states'
  INSERT #monitored_fields SELECT 'RDMO', 'DisplayStatus'

  DECLARE @ResponseTypeStr VARCHAR(8)
  SELECT @ResponseTypeStr = v.name FROM enum_value AS v JOIN enumeration AS e ON e.eid = v.eid
  WHERE e.name ='RODMResponseType' AND v.value = @ResponseType
  IF @ResponseTypeStr IS NULL
  BEGIN
    RAISERROR('_RODMRegister: Invalid response type: %d', 11, 1, @ResponseType)
    RETURN -30000
  END

  -- Select results in upload format
  SELECT DISTINCT CAST(os.id AS VARCHAR) + '|'
       + '05\02' -- RODM INTERFACE\OBJECT REGISTRATION
       + '\73' + map.RODMClass -- PARM_1
       + '\74' + m._RODMMyName -- PARM_2
       + '\47' + CASE m._RegistrationStatus WHEN 1 THEN 'ON' ELSE 'OFF' END -- SWITCH
       + '\84id=' + CAST(m.id AS VARCHAR) + -- CORRELATION_ID - object id & ...
         ' action=' + CASE m._RegistrationStatus WHEN 1 THEN 'register' ELSE 'unregister' END + -- ... registration action
       + '\85' + @ResponseTypeStr
       + '\75' + mf.field_name -- PARM_3
       + '~'
       AS DATA_ROW
  FROM
    RODMManagedObject AS m
      JOIN #descendent AS d ON d.cid = m.cid AND d.id = m.id
      JOIN SNANetwork_V AS net ON m._NetworkID like '%' + net.name + '%'
      JOIN OS_V AS os ON os.id = net._MonitoringOSID
      JOIN RODMClassMap AS map ON map.cid = m.cid
      JOIN isa_chain AS ic ON ic.cid = m.cid
      JOIN #monitored_fields AS mf ON mf.cid = ic.base_cid
  WHERE
    os._UploadEnabled = 1
GO


---------------------------------------------------------------------------------------------------
-- RODM_object_discovery_backend
PRINT 'Creating procedure RODM_object_discovery_backend'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name='RODM_object_discovery_backend' AND type = 'P')
  DROP PROCEDURE RODM_object_discovery_backend
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/RODMObject.sqv  $
-- $Revision:   1.32  $
-- $Date:   Feb 13 2001 19:13:38  $
-- $Author:   BRENT  $
CREATE PROCEDURE RODM_object_discovery_backend
  @verbose TINYINT = 0
AS
  -- Update information for existing RODM objects

  UPDATE o SET
    mtime = 
      CASE
        WHEN o.mtime > r.mtime THEN o.mtime
        ELSE r.mtime
      END, 
      _DisplayResourceName = r._DisplayResourceName
    FROM Object..RODMManagedObject AS o
    JOIN RODMLoad..RODMManagedObject AS r 
         ON o._RODMClass = r._RODMClass AND o._RODMMyName = r._RODMMyName 
    WHERE r.DiscoverAction = 'UPDATE'

  -- Use the RODMAutoRegistrationMap table to update the RegistrationStatus column of the
  -- RODMManagedObject records that will be inserted into the Object database.
  -- NOTE: when the distributed database support is added the RODMAutoRegistrationMap
  -- table will probably have to be moved to the RODMLoad database.
  DECLARE regc CURSOR FOR
    SELECT
      whereClause
    FROM
      RODMAutoRegistrationMap
    WHERE
      enabled = 1
    ORDER BY
      priority DESC
  DECLARE @whereClause VARCHAR(500)
  OPEN regc
  FETCH NEXT FROM regc INTO @whereClause
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    EXEC("
    UPDATE RODMManagedObject
    SET _RegistrationStatus = 1
    FROM RODMLoad..RODMManagedObject AS RODMManagedObject
    WHERE " + @whereClause + "
      AND RODMManagedObject.DiscoverAction = 'ADD'
    ")
    FETCH NEXT FROM regc INTO @whereClause
  END
  DEALLOCATE regc

  -- Create table of networks to re-register based on rows that are being added that 
  -- have _RegistrationStatus = 1
  CREATE TABLE #network_to_register
  (
    id   ObjID
  )
  INSERT #network_to_register
    SELECT DISTINCT snnt.id
    FROM SNANetwork_V AS snnt
    JOIN containment AS c ON c.dst_cid = 'SNNT' AND c.dst_id = snnt.id
    JOIN RODMLoad..RODMManagedObject AS rmo ON rmo._NetworkID = snnt.name
  WHERE
    c.deleted = 0
    AND rmo.DiscoverAction = 'ADD'
    AND rmo._RegistrationStatus = 1

  -- Add newly-discovered RODM objects
  INSERT INTO Object..RODMManagedObject (name, cid, _RODMClass, _RODMMyName, _NetworkID, _DisplayResourceName) 
    SELECT name, cid, _RODMClass, _RODMMyName, _NetworkID, _DisplayResourceName
    FROM RODMLoad..RODMManagedObject 
    WHERE DiscoverAction = 'ADD'
  IF @@ERROR <> 0
  BEGIN
    RAISERROR('RODM_object_discovery_backend: Error adding RODMManagedObject records', 3, 1)
  END

  -- Insert links from networks to top-level objects
  DECLARE linkc INSENSITIVE CURSOR FOR
    SELECT DISTINCT RTRIM(rmo.cid) FROM RODMManagedObject AS rmo JOIN RODMTopLevelClass AS tlc ON tlc.cid = rmo.cid
  OPEN linkc
  DECLARE @cid VARCHAR(4)
  FETCH NEXT FROM linkc INTO @cid
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    EXEC("
    INSERT SNNT_" + @cid + "_PHYC(src_id, dst_id)
      SELECT net.id, rmo.id
      FROM
        RODMManagedObject AS rmo
        JOIN SNANetwork_V AS net ON net.name = rmo._NetworkID
        JOIN containment AS c ON c.dst_cid = 'SNNT' AND c.dst_id = net.id
      WHERE
        rmo.cid = '" + @cid + "'
        AND c.deleted = 0
        AND NOT EXISTS
        (
          SELECT * FROM SNNT_" + @cid + "_PHYC AS prev
          WHERE prev.src_id = net.id AND prev.dst_id = rmo.id
        )
    IF @@ERROR <> 0
    BEGIN
      RAISERROR('RODM_object_discovery_backend: Error adding top level links for " + @cid + "', 3, 1)
    END
    ")
    FETCH NEXT FROM linkc INTO @cid
  END
  DEALLOCATE linkc

  -- Update informaton on existing RODM links
  UPDATE o SET mtime = getdate()
    FROM Object..RODMManagedLink AS o
    JOIN RODMLoad..RODMManagedLink AS r
         ON  o.link_type  = r.link_type
         AND o.SrcClass   = r.SrcClass
         AND o.SrcMyName  = r.SrcMyName
         AND o.SrcField   = r.SrcField
         AND o.DstClass   = r.DstClass
         AND o.DstMyName  = r.DstMyName
         AND o.DstField   = r.DstField
    WHERE r.DiscoverAction = 'UPDATE'

  -- Add newly-discovered RODM links - we don't use the object id's from the RODMLoad database because the id
  -- field in the RODMManagedObject table uses the identity property and may have gotten a different object id
  -- when the object was created in this database
  INSERT INTO Object..RODMManagedLink (link_type, SrcClass, SrcMyName, DstClass, DstMyName, SrcField, DstField, 
                                       src_cid, src_id, dst_cid, dst_id)
     SELECT r.link_type, r.SrcClass, r.SrcMyName, r.DstClass, r.DstMyName, r.SrcField, r.DstField, 
            r.src_cid, so.id, r.dst_cid, do.id
     FROM RODMLoad..RODMManagedLink AS r 
       JOIN Object..RODMManagedObject AS so ON so._RODMClass = r.SrcClass AND so._RODMMyName = r.SrcMyName
       JOIN Object..RODMManagedObject AS do ON do._RODMClass = r.DstClass AND do._RODMMyName = r.DstMyName
     WHERE r.DiscoverAction = 'ADD'
  IF @@ERROR <> 0
  BEGIN
    RAISERROR('RODM_object_discovery_backend: Error inserting RODMManagedLink', 3, 1)
  END

  -- Delete RODM links that no longer exist
  DELETE o
    FROM Object..RODMManagedLink AS o 
    JOIN RODMLoad..RODMManagedLink AS r
         ON  o.link_type  = r.link_type
         AND o.SrcClass   = r.SrcClass
         AND o.SrcMyName  = r.SrcMyName
         AND o.SrcField   = r.SrcField
         AND o.DstClass   = r.DstClass
         AND o.DstMyName  = r.DstMyName
         AND o.DstField   = r.DstField
    WHERE r.DiscoverAction = 'DELETE'

  -- Call _RODMRegister for all networks that need it
  DECLARE regc CURSOR FOR
    SELECT id FROM #network_to_register
  OPEN regc
  DECLARE @snnt_id ObjID
  FETCH NEXT FROM regc INTO @snnt_id
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    EXEC _RODMRegister SNNT, @snnt_id,
      @RegistrationStatus = 2, -- re-register
      @RegistrationScope  = 3, -- root AND descendents
      @ResponseType       = 1  -- exceptions only
    FETCH NEXT FROM regc INTO @snnt_id
  END
  DEALLOCATE regc
GO
